/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#ifndef IMESH_H_
#define IMESH_H_

#include "iunknown.h"

#include "graphics/igraphics.h"

namespace hydra
{

class ISampler;
class IShader;

#pragma pack(1)
struct LIBAPI Batch {
    Batch();
    virtual ~Batch();

    int startIndex;
    int indexCount;

	float4 ambient;
	float4 diffuse;
	float4 specular;

    ///
    ISampler * sampler[MAX_SAMPLER_STAGE];
    IShader * shader;

    ///
    bool visible;
    unsigned long flag;
};
#pragma pack()


class IVertexbuffer;
class IIndexbuffer;

class LIBAPI IMesh: public IUnknown
{
    DECLARE_BASE_MODULE(IMesh)

public:
    IMesh();
    virtual ~IMesh();

public:
    //
    virtual bool loadFromFile(const char * pathName);
    virtual bool saveToFile(const char * pathName);

    //
    virtual void update(float deltaTime);
    virtual void draw();

    //
    void * clone();

    //property
public:
    void setTransform(float4x4 & transform) { m_transform = transform; }
    float4x4 getTransform() { return m_transform; }

protected:
    float4x4 m_transform;

	IVertexbuffer * m_vertexbuffer;
	IIndexbuffer * m_indexbuffer;

	Batch * m_batch;
};

//
DEFINE_BASE_MODULE(IMesh, Mesh)
DEFINE_BASE_LOAD_MODULE(IMesh, Mesh)

} //hydra

#endif //MESH_H_
