/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#ifndef ISAMPLER_H_
#define ISAMPLER_H_

#include "iunknown.h"

namespace hydra
{

class IImage;

class LIBAPI ISampler: public IUnknown
{
    DECLARE_BASE_MODULE(ISampler)

public:
	ISampler();

    virtual bool create(int width, int height, unsigned long format, int mipLevels);

    //
    virtual bool loadFromFile(const char * pathName);

    //
    virtual void * map() = 0;
    virtual void unmap() = 0;

	// property
public:
	inline int getWidth() 	{ return m_width; }
	inline int getHeight() 	{ return m_height; }

	//
	inline int getLockPitch()	{ return m_pitch; }

private:
	bool decode(IImage * iamge);

protected:
	unsigned long m_format;
	int m_width, m_height;
	int m_pitch, m_mipLevels;
	int m_linearSize;
};

//
DEFINE_BASE_MODULE(ISampler, Sampler)
DEFINE_BASE_LOAD_MODULE(ISampler, Sampler)

//
inline ISampler * createSampler(int width, int height, unsigned long format, int mipLevels)
{
    ISampler * sampler = createSampler();
    bool result = sampler->create(width, height, format, mipLevels);
	ASSERT(result)

    return sampler;
}
inline ISampler * createSampler(const char * name, int width, int height, unsigned long format, int mipLevels)
{
    ISampler * sampler = createSampler(name);
    bool result = sampler->create(width, height, format, mipLevels);
	ASSERT(result)

    return sampler;
}

} //hydra

#endif //ISAMPLER_H_
