/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#ifndef IVERTEXBUFFER_H_
#define IVERTEXBUFFER_H_

#include "iunknown.h"

// Primitives supported by draw-primitive API
enum {
	PT_POINTLIST = 0,
	PT_LINELIST,
	PT_LINESTRIP,
	PT_TRIANGLELIST,
	PT_TRIANGLESTRIP,
	PT_TRIANGLEFAN,
	PT_QUADLIST,
	PT_FORCE_DWORD = 0x7fffffff,	// force 32-bit size enum
};

// Flexible vertex format bits
#define FVF_RESERVED0        0x001
#define FVF_POSITION_MASK    0x400E
#define FVF_XYZ              0x002
#define FVF_XYZRHW           0x004
#define FVF_XYZB1            0x006
#define FVF_XYZB2            0x008
#define FVF_XYZB3            0x00a
#define FVF_XYZB4            0x00c
#define FVF_XYZB5            0x00e
#define FVF_XYZW             0x4002

#define FVF_NORMAL           0x010
#define FVF_PSIZE            0x020
#define FVF_DIFFUSE          0x040
#define FVF_SPECULAR         0x080

#define FVF_TEXCOUNT_MASK    0xf00
#define FVF_TEXCOUNT_SHIFT   8
#define FVF_TEX0             0x000
#define FVF_TEX1             0x100
#define FVF_TEX2             0x200
#define FVF_TEX3             0x300
#define FVF_TEX4             0x400
#define FVF_TEX5             0x500
#define FVF_TEX6             0x600
#define FVF_TEX7             0x700
#define FVF_TEX8             0x800

#define FVF_LASTBETA_UBYTE4   0x1000
#define FVF_LASTBETA_D3DCOLOR 0x8000

#define FVF_RESERVED2         0x6000  // 2 reserved bits

namespace hydra
{

class LIBAPI IVertexbuffer: public IUnknown
{
    DECLARE_BASE_MODULE(IVertexbuffer)

public:
	IVertexbuffer();

    virtual bool create(int length, unsigned long fvf);

	//
	virtual void * map() = 0;
	virtual void unmap() = 0;

	//
	virtual void bind() = 0;

	virtual void draw(int primitiveType) = 0;
	virtual void draw(int primitiveType, unsigned int startVertex, unsigned int vertexCount) = 0;

	//
	const char * getFVF(unsigned long fvf);
	unsigned int getFVFVertexSize(unsigned long fvf);

	// property
public:
    //
    inline int getLength() 			{ return m_length; }

    inline int getStride() 			{ return m_stride; }
    inline unsigned long getFvf() 	{ return m_fvf; }

protected:
    unsigned long m_fvf;
    int m_length;
    int m_stride;
};

//
DEFINE_BASE_MODULE(IVertexbuffer, Vertexbuffer)

//
inline IVertexbuffer * createVertexbuffer(int length, unsigned long fvf)
{
    IVertexbuffer * vb = createVertexbuffer();
    bool result = vb->create(length, fvf);
	ASSERT(result)

    return vb;
}

} //hydra

#endif //IVERTEXBUFFER_H_
