/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 
#ifndef VECTOR_H_
#define VECTOR_H_

#include "stdafx.h"

#include <math.h>

#if !defined(M_PI)
#define M_PI    (3.14159265358979323846f)
#endif //M_PI

#if !defined(M_1BYPI)
#define M_1BYPI (1.0f / M_PI)
#endif //M_1BYPI

#if !defined(TORADIAN)
#define TORADIAN(degree) ((degree) * (M_PI / 180.0f))
#endif //TORADIAN

#if !defined(TODEGREE)
#define TODEGREE(radian) ((radian) * (180.0f / M_PI))
#endif //TODEGREE

namespace hydra
{

/** fixed2 structor.
 * @remarks
 * @note
 */

struct LIBAPI fixed2 {
	fixed x, y;

	//
	fixed2()
	{
		x = y = 0;
	}
	fixed2(fixed _x, fixed _y)
	{
		x = _x; y = _y;
	}
};

/** fixed4 structor.
 * @remarks
 * @note
 */

struct LIBAPI fixed4 {
	union
	{
		struct
		{
			fixed left, top;
			fixed right, bottom;
		};
		fixed x, y, z, w;
	};
	
	//
	fixed4()
	{
		x = y = z = w = 0;
	}
	fixed4(fixed _x, fixed _y, fixed _z, fixed _w)
	{
		x = _x; y = _y; z = _z; w = _w;
	}
};

/** float2 structor.
 * @remarks
 * @note
 */

struct LIBAPI float2 {
	float x, y;

	//
	float2()
	{
		x = y = 0;
	}
	float2(float _x, float _y)
	{
		x = _x; y = _y;
	}
};
typedef float2 vec2;

/** float3 structor.
 * @remarks
 * @note
 */

struct LIBAPI float3 {
	float x, y, z;

	/// Constructors
	float3()
	{
		x = y = z = 0;
	}
	float3(float f)
	{
		x = y = z = f;
	}
	float3(float _x, float _y, float _z)
	{
		x = _x; y = _y; z = _z;
	}
	float3(const float f[3])
	{
		x = f[0]; y = f[1]; z = f[2];
	}

	/// access grants
	const float & operator[](int i) const
	{
		return (&x)[i];
	}
	float & operator[](int i)
	{
		return (&x)[i];
	}

	/// assignment operators
	float3 & operator += (const float3 & v)
	{
		x += v.x; y += v.y; z += v.z;
		return *this;
	}
	float3 & operator -= (const float3 & v)
	{
		x -= v.x; y -= v.y; z -= v.z;
		return *this;
	}
	float3 & operator *= (const float3 & v)
	{
		x *= v.x; y *= v.y; z *= v.z;
		return *this;
	}
	float3 & operator /= (const float3 & v)
	{
		x /= v.x; y /= v.y; z /= v.z;
		return *this;
	}
	float3 & operator *= (float s)
	{
		x *= s; y *= s; z *= s;
		return *this;
	}
	float3 & operator /= (float s)
	{
		x /= s; y /= s; z /= s;
		return *this;
	}
};
typedef float3 vec3;

/** float4 structor.
 * @remarks
 * @note
 */

struct LIBAPI float4 {
	float x, y, z, w;

	///
	float4()
	{
		x = y = z = w = 0;
	}
	float4(float _x, float _y, float _z, float _w)
	{
		x = _x; y = _y; z = _z; w = _w;
	}

	/// access grants
	const float & operator[](int i) const
	{
		return (&x)[i];
	}
	float & operator[](int i)
	{
		return (&x)[i];
	}
};
typedef float4 vec4;

/** float4x4 structor.
 * @remarks
 * @note
 */

struct LIBAPI float4x4 {
	union
	{
		struct
		{
			float _11, _12, _13, _14;
			float _21, _22, _23, _24;
			float _31, _32, _33, _34;
			float _41, _42, _43, _44;
		};
		float m[4][4];		// D3DSDK
	};

	float4x4()
	{
		_11 = 1.0f; _12 = _13 = _14 = 0;
		_22 = 1.0f; _21 = _23 = _24 = 0;
		_33 = 1.0f; _31 = _32 = _34 = 0;
		_44 = 1.0f; _41 = _42 = _43 = 0;
	}
	float4x4(float f11, float f12, float f13, float f14,
		float f21, float f22, float f23, float f24,
		float f31, float f32, float f33, float f34,
		float f41, float f42, float f43, float f44)
	{
		_11 = f11; _12 = f12; _13 = f13; _14 = f14;
		_21 = f21; _22 = f22; _23 = f23; _24 = f24;
		_31 = f31; _32 = f32; _33 = f33; _34 = f34;
		_41 = f41; _42 = f42; _43 = f43; _44 = f44;
	}

	/// access grants
	float & operator[](int i)
	{
		return (&_11)[i];
	}
	const float & operator[](int i) const
	{
		return (&_11)[i];
	}

	float & operator () (int row, int col)
	{
		return m[row][col];
	}
	const float & operator () (int row, int col) const
	{
		return m[row][col];
	}
	
	//
	void setIdentity();
	
	void setRotationX(const float angle);
	void setRotationY(const float angle);
	void setRotationZ(const float angle);
	void setRotationYawPitchRoll(const float yaw, const float pitch, const float roll);
	void setRotation(const float3 & dir, const float radius);

	void setTranslation(const float3 & pos);
	void setScaling(const float3 & scale);

	void setBillboard(const float3 & from, const float3 & to, const float3 & up);
	void translate(const float3 &v);
};
typedef float4x4 mat4;

/// Unary operators
inline float3 operator + (const float3 & v)
{
	return v;
}
inline float3 operator - (const float3 & v)
{
	return float3(-v.x, -v.y, -v.z);
}

/// Addition and subtraction
inline float3 operator + (const float3 & v1, const float3 & v2)
{
	return float3(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
}
inline float3 operator - (const float3 & v1, const float3 & v2)
{
	return float3(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z);
}

/// Memberwise multiplication and division
inline float3 operator * (const float3 & v1, const float3 & v2)
{
	return float3(v1.x * v2.x, v1.y * v2.y, v1.z * v2.z);
}
inline float3 operator / (const float3 & v1, const float3 & v2)
{
	return float3(v1.x / v2.x, v1.y / v2.y, v1.z / v2.z);
}

/// Vector dominance
inline int operator < (const float3 & v1, const float3 & v2)
{
	return v1.x < v2.x && v1.y < v2.y && v1.z < v2.z;
}
inline int operator <= (const float3 & v1, const float3 & v2)
{
	return v1.x <= v2.x && v1.y <= v2.y && v1.z <= v2.z;
}

/// Scalar multiplication and division
inline float3 operator * (const float3 & v, float s)
{
	return float3(s * v.x, s * v.y, s * v.z);
}
inline float3 operator * (float s, const float3 & v)
{
	return float3(s * v.x, s * v.y, s * v.z);
}
inline float3 operator / (const float3 & v, float s)
{
	return float3(v.x / s, v.y / s, v.z / s);
}

/// Bitwise equality
inline int operator == (const float3 & v1, const float3 & v2)
{
	return v1.x == v2.x && v1.y == v2.y && v1.z == v2.z;
}

/// Return memberwise min/max of input vectors
inline float3 minimize(const float3 & v1, const float3 & v2)
{
	return float3(v1.x < v2.x ? v1.x : v2.x,
		v1.y < v2.y ? v1.y : v2.y,
		v1.z < v2.z ? v1.z : v2.z);
}
inline float3 maximize(const float3& v1, const float3& v2)
{
	return float3(v1.x > v2.x ? v1.x : v2.x,
		v1.y > v2.y ? v1.y : v2.y,
		v1.z > v2.z ? v1.z : v2.z);
}

/// Returns vector with same direction and unit length
inline float3 normalize(const float3 & v)
{
	float mag = sqrt(v.x * v.x + v.y * v.y + v.z * v.z);
	if (mag != 0)
		return v / mag;

	return v;
}

/// Dot and cross product
inline float3 crossProduct(const float3 & v1, const float3 & v2)
{
	float3 result;
	result.x = v1.y * v2.z - v2.y * v1.z;
	result.y = v2.x * v1.z - v1.x * v2.z;
	result.z = v1.x * v2.y - v2.x * v1.y;

	return result;
}
inline float dotProduct(const float3 & v1, const float3 & v2)
{
	return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
}

} //hydra

#endif //VECTOR_H_
