/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#ifndef KERNEL_H_
#define KERNEL_H_

#include "module.h"

namespace hydra
{

/** kernel class.
 * @remarks
 * @note
 */
class LIBAPI Kernel
{
	DECLARE_SINGLETON(Kernel)

private:
	typedef void * (*PFNCREATEPROC)();
	typedef void (*PFNREMOVEPROC)();

	/** kernel node structor.
	 * @remarks
	 * @note
	 */
	struct Node
	{
	public:
		Node(const char * _name, const char * _description, PFNCREATEPROC _pfncreate, PFNREMOVEPROC _pfnremove, Module * _module)
		{
			prev = next = NULL;

			//
			name = _name;
			description = _description;

			pfncreate = _pfncreate;
			pfnremove = _pfnremove;

			module = _module;
		}
		~Node()
		{
			if (pfnremove)
			{
				pfnremove();
				pfnremove = NULL;
			}

			SAFE_DELETE(module);
		}

	public:
		/// prev node
		Node * prev;

		/// next node
		Node * next;

		/// module name
		const char * name;
		const char * description;

		///	module create function pointer.
		PFNCREATEPROC pfncreate;
		PFNREMOVEPROC pfnremove;

		/// Module class pointer.
		Module * module;
	};

private:
	/// module count
	int m_count;

	/// first module node pointer
	Node * m_first;

	/// last module node pointer
	Node * m_last;

private:
	/** remove module node
	 * @note
	 * @param node Module pointer.
	 * @returns none
	 * @par
	 */
	void remove(Node * node);

public:
	Kernel();
	virtual ~Kernel();

public:
	///
	bool registerModule(const char * name, const char * description, PFNCREATEPROC pfncreate, PFNREMOVEPROC pfnremove, Module * module);
	void unregisterModule(const char * name);

	///
	void * queryModule(const char * name, const char * ext);

	///
	bool loadModule(const char * name);

    ///
	void dump();

	// property
public:
	inline int getCount()	{ return m_count; }
};


///
#define registerKernelModule(name) \
	if (!Kernel::instance()->registerModule(name::mname, name::mdescription, name::fnmcreate, name::remove, NULL)) \
		return false

///
#define unregisterKernelModule(name) \
	Kernel::instance()->unregisterModule(name::mname)

///
#define queryKernelModule(name, ext) \
	Kernel::instance()->queryModule(name, ext)

///
#define loadKernelModule(name) \
	if (!Kernel::instance()->loadModule(name)) \
		return false

///
#define dumpKernelModule() \
	Kernel::instance()->dump()

} //hydra

#endif //KERNEL_H_
