/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "graphics/iimage.h"

#include "graphics/igraphics.h"

#include "kernel.h"

namespace hydra
{
IMPLEMENT_BASE_MODULE(IImage, usr/lib/graphics/image)

IImage::IImage()
{
	m_bits = NULL;
	m_linearSize = 0;
}
IImage::~IImage()
{
	SAFE_FREE(m_bits);
}

static const char * __formats[] =
{
	"FMT_UNKNOWN",

	"FMT_R8G8B8",
	"FMT_A8R8G8B8",

	"FMT_R5G6B5",
	"FMT_A1R5G5B5",
	"FMT_A4R4G4B4",

	"FMT_A8",
	"FMT_L8",
	"FMT_A8L8",

	"FMT_DXT1",
	"FMT_DXT3",
	"FMT_DXT5",
};
bool IImage::create(int width, int height, unsigned long format, int mipLevels)
{
	//TRACE("# IImage::create(%d, %d, %s, %d)\n", width, height, __formats[format], mipLevels)

	int bitsPerPixels = 1;
	switch (format)
	{
	case FMT_R8G8B8:
		bitsPerPixels = 3;
		break;
	case FMT_A8R8G8B8:
		bitsPerPixels = 4;
		break;

	case FMT_R5G6B5:
	case FMT_A1R5G5B5:
	case FMT_A4R4G4B4:
		bitsPerPixels = 2;
		break;

	case FMT_A8:
		bitsPerPixels = 1;
		break;

	case FMT_L8:
		bitsPerPixels = 1;
		break;

	case FMT_A8L8:
		bitsPerPixels = 2;
		break;

	case FMT_DXT1:
		bitsPerPixels = 8;
		break;

	case FMT_DXT3:
		bitsPerPixels = 16;
		break;

	case FMT_DXT5:
		bitsPerPixels = 16;
		break;
	}

	m_format = format;

	m_width = width;
	m_height = height;

	m_pitch = width * bitsPerPixels;

	//
	m_mipLevels = mipLevels;

	//
	m_linearSize = 0;

	//
	int w = width;
	int h = height;
	int levels = m_mipLevels ? m_mipLevels : 1;
	
	for (int n=0; n<levels; n++)
	{
		int size;
		if (format > FMT_A8L8)
			size = (((w + 3) >> 2) * ((h + 3) >> 2)) * bitsPerPixels;
		else
			size = (h + 1) * (w * bitsPerPixels);

		m_linearSize += size;

		w = max(w >> 1, 1);
		h = max(h >> 1, 1);
	}

	//TRACE(">> linerSize = %d\n", m_linearSize)

	//
	m_bits = MALLOC(m_linearSize, sizeof(char));
	ASSERT(m_bits)

	return true;
}
bool IImage::loadFromFile(const char * pathName)
{
	//TRACE("# Image::loadFromFile(%s)", pathName)

	//
	char fullPath[MAX_PATH];
	makeFullPath(fullPath, pathName);

	void * data = ::loadFromFile(fullPath);
	if (!data)
		return false;

	decode(data);
	free(data);

	return true;
}

} //hydra
