/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "graphics/imageDDS.h"

#include "graphics/igraphics.h"

#include "kernel.h"

namespace hydra
{
IMPLEMENT_MODULE(ImageDDS, usr/lib/graphics/image.dds)

ImageDDS::ImageDDS()
{

}

//
enum {
	DDSD_CAPS = 0x00000001,
	DDSD_HEIGHT = 0x00000002,
	DDSD_WIDTH = 0x00000004,
	DDSD_PITCH = 0x00000008,
	DDSD_PIXELFORMAT = 0x00001000,
	DDSD_MIPMAPCOUNT = 0x00020000,
	DDSD_LINEARSIZE = 0x00080000,
	DDSD_DEPTH = 0x00800000,
};

enum {
	DDPF_ALPHAPIXELS = 0x00000001,
	DDPF_FOURCC = 0x00000004,
	DDPF_PALETTEINDEXED4 = 0x00000008,
	DDPF_PALETTEINDEXED8 = 0x00000020,
	DDPF_RGB = 0x00000040,
};

enum {
	DDSCAPS_COMPLEX	= 0x00000008,
	DDSCAPS_TEXTURE	= 0x00001000,
	DDSCAPS_MIPMAP = 0x00400000,
};

enum {
	DDSCAPS2_CUBEMAP = 0x00000008,
	DDSCAPS2_CUBEMAP_POSITIVEX = 0x00000400,
	DDSCAPS2_CUBEMAP_NEGATIVEX = 0x00000800,
	DDSCAPS2_CUBEMAP_POSITIVEY = 0x00001000,
	DDSCAPS2_CUBEMAP_NEGATIVEY = 0x00002000,
	DDSCAPS2_CUBEMAP_POSITIVEZ = 0x00004000,
	DDSCAPS2_CUBEMAP_NEGATIVEZ = 0x00008000,
	DDSCAPS2_VOLUME	= 0x00200000,
};

enum {
	DDS_FORMAT_RGBA5650,
	DDS_FORMAT_RGBA5550,
	DDS_FORMAT_RGBA5551,
	DDS_FORMAT_RGBA4444,
	DDS_FORMAT_RGBA8880,
	DDS_FORMAT_RGBA8888,
	DDS_FORMAT_INDEX4,
	DDS_FORMAT_INDEX8,
	DDS_FORMAT_DXT1,
	DDS_FORMAT_DXT3,
	DDS_FORMAT_DXT5,
};

enum {
	FOURCC_A16B16G16R16	= 36,

	FOURCC_A16B16G16R16F = 113,
	FOURCC_A32B32G32R32F = 116,

	FOURCC_DXT1	= 0x31545844,
	FOURCC_DXT2	= 0x32545844,
	FOURCC_DXT3	= 0x33545844,
	FOURCC_DXT4	= 0x34545844,
	FOURCC_DXT5	= 0x35545844,
};

//
struct DDSFORMAT {
	unsigned int size;
	unsigned int flags;
	unsigned int fourcc;
	unsigned int bpp;
	unsigned int red;
	unsigned int green;
	unsigned int blue;
	unsigned int alpha;
};
struct DDSCAPS {
	unsigned int caps;
	unsigned int caps2;
	unsigned int reserved[3];
};
struct DDSHEADER {
	unsigned int magic;
	unsigned int size;
	unsigned int flags;
	unsigned int height;
	unsigned int width;
	unsigned int pitch;
	unsigned int depth;
	unsigned int levels;
	unsigned int reserved[11];

	DDSFORMAT format;
	DDSCAPS caps;
};

#define FOURCC(c0, c1, c2, c3) (c0 | (c1 << 8) | (c2 << 16) | (c3 << 24))   /**< creates a FOURCC code */

//
bool ImageDDS::decode(void * data)
{
	DDSHEADER * header = (DDSHEADER *) data;
	if (header->magic != swapEndian((unsigned int) FOURCC('D','D','S',' ')))
		return false;

	int format = FMT_UNKNOWN;
	switch (swapEndian(header->format.fourcc))
	{
	case FOURCC('D','X','T','1'):
		format = FMT_DXT1;
		break;

	case FOURCC('D','X','T','2'):
	case FOURCC('D','X','T','3'):
		format = FMT_DXT3;
		break;
	case FOURCC('D','X','T','4'):
	case FOURCC('D','X','T','5'):
		format = FMT_DXT5;
		break;

	default:
		switch (header->format.bpp)
		{
		case 8:
		case 16:
			break;
		case 24:
			format = FMT_R8G8B8;
			break;
		case 32:
			if (header->format.red == swapEndian((unsigned int) 0x3FF00000))
			{
			}
			else
			{
				format = FMT_A8R8G8B8;
			}
			break;
		}
	}

	/*
	if ( ( DDSD_MIPMAPCOUNT & header->flags ) && header->level > 1 ) {
	if ( !( DDSCAPS_MIPMAP & caps->caps ) ) {
	m_shell->Error( "wrong mipmap texture\n" ) ;
	return false ;
	}
	n_levels = header->level ;
	}
	if ( ( DDSD_DEPTH & header->flags ) && header->depth > 1 ) {
	if ( !( DDSCAPS2_VOLUME & caps->caps2 ) ) {
	m_shell->Error( "wrong volume texture\n" ) ;
	return false ;
	}
	if ( DDSCAPS2_CUBEMAP & caps->caps2 ) {
	m_shell->Error( "wrong cubemap texture\n" ) ;
	return false ;
	}
	n_frames = header->depth ;
	is_volume = true ;
	}
	if ( DDSCAPS2_CUBEMAP & caps->caps2 ) {
	n_frames = 0 ;
	if ( DDSCAPS2_CUBEMAP_POSITIVEX & caps->caps2 ) n_frames ++ ;
	if ( DDSCAPS2_CUBEMAP_NEGATIVEX & caps->caps2 ) n_frames ++ ;
	if ( DDSCAPS2_CUBEMAP_POSITIVEY & caps->caps2 ) n_frames ++ ;
	if ( DDSCAPS2_CUBEMAP_NEGATIVEY & caps->caps2 ) n_frames ++ ;
	if ( DDSCAPS2_CUBEMAP_POSITIVEZ & caps->caps2 ) n_frames ++ ;
	if ( DDSCAPS2_CUBEMAP_NEGATIVEZ & caps->caps2 ) n_frames ++ ;
	if ( n_frames == 0 ) {
	m_shell->Warning( "wrong cubemap flags\n" ) ;
	n_frames = 1 ;
	}
	}
	*/

	//
	if (!create(swapEndian(header->width), swapEndian(header->height), format, swapEndian(header->levels)))
		return false;

	//
	char * bits = (char *) getBits();
	char * image = (char *) (header + 1);

	// DDPF_RGB
	if (swapEndian(header->format.flags) & DDPF_RGB)
	{
		int srcSkipBytes = swapEndian(header->format.bpp) >> 3;
		int destSkipBytes = srcSkipBytes;

		// ARGB
		for (int i = 0; i < (int) swapEndian(header->height); i ++)
		{
			for (int j = 0; j < (int) swapEndian(header->width); j++)
			{
				if (header->format.blue == 0xff)
				{
					// RGB <- BGR
					*bits = *(image + 2);
					*(bits + 1) = *(image + 1);
					*(bits + 2) = *image;
				}
				else
				{
					// RGB <- BGR
					*bits = *image;
					*(bits + 1) = *(image + 1);
					*(bits + 2) = *(image + 2);
				}

				if (srcSkipBytes == 4)
					*(bits + 3) = *(image + 3);

				//
				bits += destSkipBytes;
				image += srcSkipBytes;
			}
		}
	}

	// DDPF_FOURCC
	if (swapEndian(header->format.flags) & DDPF_FOURCC)
		memcpy(bits, image, m_linearSize);

    return true;
}

} //hydra
