/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "graphics/imesh.h"

#include "graphics/igraphics.h"
#include "graphics/isampler.h"
#include "graphics/ishader.h"

#include "graphics/iindexbuffer.h"
#include "graphics/ivertexbuffer.h"

#include "kernel.h"

namespace hydra
{
IMPLEMENT_BASE_MODULE(IMesh, usr/lib/graphics/mesh)

Batch::Batch()
{
    visible = true;

	//
    shader = NULL;

	for (int n=0; n<MAX_SAMPLER_STAGE; n++)
		sampler[n] = NULL;
}
Batch::~Batch()
{
    SAFE_RELEASE(shader);

	for (int n=0; n<MAX_SAMPLER_STAGE; n++)
		SAFE_RELEASE(sampler[n]);
}

//
IMesh::IMesh()
{
	m_batch = NULL;

	m_vertexbuffer = NULL;
	m_indexbuffer = NULL;
}
IMesh::~IMesh()
{
	SAFE_RELEASE(m_indexbuffer);
	SAFE_RELEASE(m_vertexbuffer);

	SAFE_DELETE_ARRAY(m_batch);
}

//
bool IMesh::loadFromFile(const char * pathName)
{
    return false;
}

bool IMesh::saveToFile(const char * pathName)
{
    return true;
}

//
void IMesh::update(float deltaTime)
{
}

static float4 dirFromSky(0.0f, -1.0f, 0.0f, 0.0f);

/// light directions (view space)
static float4 dirFromLight(0.577f, -0.577f, 0.577f, 0.0f);

/// light intensity
static float4 I_a(0.8f, 0.8f, 0.8f, 1.0f);        /// ambient
static float4 I_d(0.9f, 0.9f, 0.9f, 1.0f);        /// diffuse
static float4 I_s(0.9f, 0.9f, 0.9f, 1.0f);        /// specular
static float4 I_b(0.1f, 0.0f, 0.0f, 1.0f);        /// ground
static float4 I_c(0.9f, 0.9f, 1.0f, 1.0f);        /// sky

static float I_n = 32.0f;                         /// power

void IMesh::draw()
{
    //
	graphicsModule()->setRenderState(RS_CULLMODE, CULL_NONE);

    m_indexbuffer->bind();
    m_vertexbuffer->bind();

    //
	Batch * batch = m_batch;
	while (batch->shader)
    {
        IShader * shader = batch->shader;
        shader->begin();

     	float4x4 transform;
		shader->setMatrix("world", transform);

		shader->setVector("k_a", batch->ambient);
		shader->setVector("k_d", batch->diffuse);
		shader->setVector("k_s", batch->specular);

		shader->setVector("I_a", I_a);
		shader->setVector("I_d", I_d);
		shader->setVector("I_s", I_s);
		shader->setVector("I_b", I_b);
		shader->setVector("I_c", I_c);

		shader->setFloat("n", I_n);

		shader->setVector("dirFromSky", dirFromSky);
		shader->setVector("dirFromLight", dirFromLight);

        //
        for (int n = 0; n < MAX_SAMPLER_STAGE; n ++)
        {
            if (batch->sampler[n])
            {
                shader->setSampler(n, batch->sampler[n]);

                shader->setSamplerState(n, SAMP_MINFILTER, TEXF_ANISOTROPIC);
                shader->setSamplerState(n, SAMP_MAGFILTER, TEXF_ANISOTROPIC);
                shader->setSamplerState(n, SAMP_MIPFILTER, TEXF_LINEAR);
            }
        }

        m_indexbuffer->draw(PT_TRIANGLELIST, batch->startIndex, batch->indexCount, m_vertexbuffer->getLength());

        shader->end();

        batch ++;
    }

    graphicsModule()->flush();
}

//
void * IMesh::clone()
{
    return this;
}

} //hydra
