/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "graphics/ivertexbuffer.h"

#include "kernel.h"

namespace hydra
{

IMPLEMENT_BASE_MODULE(IVertexbuffer, usr/lib/graphics/vertexbuffer)

IVertexbuffer::IVertexbuffer()
{

}

//
bool IVertexbuffer::create(int length, unsigned long fvf)
{
	//TRACE("# IVertexbuffer::create(%d, %s)\n", length, getFVF(fvf))

	//
	m_stride = getFVFVertexSize(fvf);

	m_fvf = fvf;
	m_length = length;

	return true;
}

//
unsigned int IVertexbuffer::getFVFVertexSize(unsigned long fvf)
{
	unsigned int size = 0;

	//
	unsigned long position = fvf & FVF_POSITION_MASK;
	if (position == FVF_XYZ)
		size += sizeof(float) * 3;
	else if (position == FVF_XYZRHW)
		size += sizeof(float) * 4;
	else if (position >= FVF_XYZB1)
	{
		int weights = (position >> 1) - 2;
		size += sizeof(float) * (3 + weights);

		// index
		//size += sizeof(float) * (3 + weights);
	}

	//
	if (fvf & FVF_NORMAL)
		size += sizeof(float) * 3;
	if (fvf & FVF_PSIZE)
		size += sizeof(float);
	if (fvf & FVF_DIFFUSE)
		size += sizeof(unsigned long);
	if (fvf & FVF_SPECULAR)
		size += sizeof(unsigned long);

	//
	if (fvf & FVF_TEXCOUNT_MASK)
	{
		// only one tu/tv set currently supported
		int texCoords = ((fvf & FVF_TEXCOUNT_MASK) >> FVF_TEXCOUNT_SHIFT);
		if (texCoords > 8)
			texCoords = 8;

		size += sizeof(float) * (2 * texCoords);
	}

	return size;
}
const char * IVertexbuffer::getFVF(unsigned long fvf)
{
	static char FVF[MAX_PATH];

	//
	unsigned long position = fvf & FVF_POSITION_MASK;
	if (position == FVF_XYZ)
		strcpy(FVF, "FVF_XYZ");
	else if (position == FVF_XYZRHW)
		strcpy(FVF, "FVF_XYZRHW");
	else if (position >= FVF_XYZB1)
	{
		int weights = (position >> 1) - 2;

		char XYZBx[MAX_PATH];
		sprintf(XYZBx, "FVF_XYZB%d", weights);
		strcpy(FVF, XYZBx);

		if (fvf & FVF_LASTBETA_UBYTE4)
			strcat(FVF, " | FVF_LASTBETA_UBYTE4");

		if (fvf & FVF_LASTBETA_D3DCOLOR)
			strcat(FVF, " | FVF_LASTBETA_D3DCOLOR");
	}

	//
	if (fvf & FVF_NORMAL)
		strcat(FVF, " | FVF_NORMAL");
	if (fvf & FVF_PSIZE)
		strcat(FVF, " | FVF_PSIZE");
	if (fvf & FVF_DIFFUSE)
		strcat(FVF, " | FVF_DIFFUSE");
	if (fvf & FVF_SPECULAR)
		strcat(FVF, " | FVF_SPECULAR");

	//
	if (fvf & FVF_TEXCOUNT_MASK)
	{
		// only one tu/tv set currently supported
		int texCoords = ((fvf & FVF_TEXCOUNT_MASK) >> FVF_TEXCOUNT_SHIFT);
		if (texCoords > 8)
			texCoords = 8;

		char TEXx[MAX_PATH];
		sprintf(TEXx, " | FVF_TEX%d", texCoords);
		strcat(FVF, TEXx);
	}

	return FVF;
}

} //hydra

