/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "graphics/vector.h"

namespace hydra
{

void float4x4::setIdentity()
{
	_11 = 1.0f; _12 = 0.0f; _13 = 0.0f; _14 = 0.0f;
	_21 = 0.0f; _22 = 1.0f; _23 = 0.0f; _24 = 0.0f;
	_31 = 0.0f; _32 = 0.0f; _33 = 1.0f; _34 = 0.0f;
	_41 = 0.0f; _42 = 0.0f; _43 = 0.0f; _44 = 1.0f;
}

void float4x4::setRotationX(const float angle)
{
	float _sin = (float) sin(angle);
	float _cos = (float) cos(angle);

	_11 = 1.0f; _12 =  0.0f; _13 = 0.0f; _14 = 0.0f;
	_21 = 0.0f; _22 =  _cos; _23 = _sin; _24 = 0.0f;
	_31 = 0.0f; _32 = -_sin; _33 = _cos; _34 = 0.0f;
	_41 = 0.0f; _42 =  0.0f; _43 = 0.0f; _44 = 1.0f;
}
void float4x4::setRotationY(const float angle)
{
	float _sin = (float) sin(angle);
	float _cos = (float) cos(angle);

	_11 = _cos; _12 = 0.0f; _13 = -_sin; _14 = 0.0f;
	_21 = 0.0f; _22 = 1.0f; _23 = 0.0f;  _24 = 0.0f;
	_31 = _sin; _32 = 0.0f; _33 = _cos;  _34 = 0.0f;
	_41 = 0.0f; _42 = 0.0f; _43 = 0.0f;  _44 = 1.0f;
}
void float4x4::setRotationZ(const float angle)
{
	float _sin = (float) sin(angle);
	float _cos = (float) cos(angle);

	_11 = _cos;  _12 = _sin; _13 = 0.0f; _14 = 0.0f;
	_21 = -_sin; _22 = _cos; _23 = 0.0f; _24 = 0.0f;
	_31 = 0.0f;  _32 = 0.0f; _33 = 1.0f; _34 = 0.0f;
	_41 = 0.0f;  _42 = 0.0f; _43 = 0.0f; _44 = 1.0f;
}
void float4x4::setRotationYawPitchRoll(const float yaw, const float pitch, const float roll)
{
	float cosx = (float) cos(pitch);
	float sinx = (float) sin(pitch);
	float cosy = (float) cos(yaw);
	float siny = (float) sin(yaw);
	float cosz = (float) cos(roll);
	float sinz = (float) sin(roll);

	_11 = cosy * cosz + sinx * siny * sinz;
	_12 = -cosx * sinz;
	_13 = sinx * cosy * sinz - siny * cosz;

	_21 = cosy * sinz - sinx * siny * cosz;
	_22 = cosx * cosz;
	_23 = -siny * sinz - sinx * cosy * cosz;

	_31 = cosx * siny;
	_32 = sinx;
	_33 = cosx * cosy;

	_14 = _24 = _34 = 0.0f;
	_41 = _42 = _43 = 0.0f;
	_44 = 1.0f;
}

void float4x4::setRotation(const float3 & dir, const float radius)
{
	float _cos = (float) cos(radius);
	float _sin = (float) sin(radius);
	float3 v = normalize(dir);

	_11 = (v.x * v.x) * (1.0f - _cos) + _cos;
	_12 = (v.x * v.y) * (1.0f - _cos) - (v.z * _sin);
	_13 = (v.x * v.z) * (1.0f - _cos) + (v.y * _sin);

	_21 = (v.y * v.x) * (1.0f - _cos) + (v.z * _sin);
	_22 = (v.y * v.y) * (1.0f - _cos) + _cos ;
	_23 = (v.y * v.z) * (1.0f - _cos) - (v.x * _sin);

	_31 = (v.z * v.x) * (1.0f - _cos) - (v.y * _sin);
	_32 = (v.z * v.y) * (1.0f - _cos) + (v.x * _sin);
	_33 = (v.z * v.z) * (1.0f - _cos) + _cos;

	_14 = _24 = _34 = 0.0f;
	_41 = _42 = _43 = 0.0f;
	_44 = 1.0f;
}

//
void float4x4::setTranslation(const float3 & pos)
{
	_11 = 1.0f;  _12 = 0.0f;  _13 = 0.0f;  _14 = 0.0f;
	_21 = 0.0f;  _22 = 1.0f;  _23 = 0.0f;  _24 = 0.0f;
	_31 = 0.0f;  _32 = 0.0f;  _33 = 1.0f;  _34 = 0.0f;
	_41 = pos.x; _42 = pos.y; _43 = pos.z; _44 = 1.0f;
}

//
void float4x4::setScaling(const float3 & scale)
{
	_11 = scale.x; _12 = 0.0f;    _13 = 0.0f;    _14 = 0.0f;
	_21 = 0.0f;    _22 = scale.y; _23 = 0.0f;    _24 = 0.0f;
	_31 = 0.0f;    _32 = 0.0f;    _33 = scale.z; _34 = 0.0f;
	_41 = 0.0f;    _42 = 0.0f;    _43 = 0.0f;    _44 = 1.0f;
}

void float4x4::setBillboard(const float3 & from, const float3 & to, const float3 & up)
{
	float3 z = normalize(from - to);
	float3 y = normalize(up);
	float3 x = (y * z);
	z = x * y;

	_11 = x.x; _12 = x.y; _13 = x.z; _14 = 0.0f;
	_21 = y.x; _22 = y.y; _23 = y.z; _24 = 0.0f;
	_31 = z.x; _32 = z.y; _33 = z.z; _34 = 0.0f;
	_41 = _42 = _43 = 0; x.z; _44 = 1.0f;
}

void float4x4::translate(const float3 &v)
{
	_14 += (v.x * _11 + v.y * _12 + v.z * _13);
	_24 += (v.x * _21 + v.y * _22 + v.z * _23);
	_34 += (v.x * _31 + v.y * _32 + v.z * _33);
	_44 += (v.x * _41 + v.y * _42 + v.z * _43);
}

} //hydra

