/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "kernel.h"

namespace hydra
{

IMPLEMENT_SINGLETON(Kernel)

//
Kernel::Kernel()
{
	singleton = this;

	//
	m_count = 0;
	m_first = m_last = NULL;

	//
	initPath();
}
Kernel::~Kernel()
{
	// walk linked list and destroy all nodes
	Node * current = m_last;
	while (current)
	{
		//TRACE("unloadModule(%s)\n", current->name)

		Node * prev = current->prev;
		Node * next = current->next;

		// Remove node from List
		if (prev)
			prev->next = next;
		else
			m_first = next;

		if (next)
			next->prev = prev;
		else
			m_last = prev;

		// delete this node
		delete current;
		current = m_last;
	}

	//
	m_count = 0;
	m_first = m_last = NULL;

	//
	singleton = NULL;
}

//
bool Kernel::registerModule(const char * name, const char * description, PFNCREATEPROC pfncreate, PFNREMOVEPROC pfnremove, Module * module)
{
	//TRACE("# Kernel::registerModule(%s)\n", name)

	// find
	Node * node = m_first;
	while (node)
	{
		if (!strcmp(name, node->name))
			return true;

		// next Node
		node = node->next;
	}

	// append
	node = new Node(name, description, pfncreate, pfnremove, module);
	ASSERT(node)

	//
	node->next = NULL;
	node->prev = m_last;

	if (m_last)
		m_last->next = node;

	if (!m_first)
		m_first = node;

	m_last = node;

	//
	m_count ++;

	return true;
}
void Kernel::unregisterModule(const char * name)
{
	//TRACE("Kernel::unregisterMoudle(%s)\n", name)

	//
	Node * node = m_first;
	while (node)
	{
		if (!strcmp(name, node->name))
		{
			remove(node);
			break;
		}

		// next Node
		node = node->next;
	}
}

//
void * Kernel::queryModule(const char * name, const char * ext)
{
    //if (ext)
    //	TRACE("# Kernel::queryModule(%s%s)\n", name, ext)
    //else
    //    TRACE("# Kernel::queryModule(%s)\n", name)

    //
	Node * node = m_first;

	if (ext)
	{
	   	// make fullname
		char fullname[MAX_PATH];

		strcpy(fullname, name);
        strcat(fullname, ext);
		_strlwr(fullname);

        while (node)
        {
            if (!strcmp(fullname, node->name))
            {
                void * result = node->pfncreate();
                return result;
            }

            // next Node
            node = node->next;
        }

        node = m_first;
	}

	//
	while (node)
	{
		if (!strcmp(name, node->name))
		{
			void * result = node->pfncreate();
			return result;
		}

		// next Node
		node = node->next;
	}

	TRACE0(">> ERROR: not found(%s)!!!\n", name)

	return NULL;
}

//
bool Kernel::loadModule(const char * pathName)
{
	Module * module = new Module;
	if (!module->loadFromFile(pathName))
		return false;

	//
	const MODULEINFO * info = module->pfngetInfo();
	if (!registerModule(info->name, info->description, NULL, NULL, module))
		return false;

	return true;
}

//
void Kernel::dump()
{
	TRACE("# Kernel::dump()\n")

	//
	Node * node = m_first;
	while (node)
	{
		TRACE(">> %s : %s\n", node->name, node->description)

		// next Node
		node = node->next;
	}
}

//
void Kernel::remove(Node * node)
{
	if (node == m_first)
		m_first = m_first->next;

	// remove this list
	Node * prev = node->prev;
	Node * next = node->next;

	if (prev)
		prev->next = next;
	else
		m_first = next;

	if (next)
		next->prev = prev;
	else
		m_last = prev;

	// delete node
	delete node;
	node = NULL;

	//
	m_count --;
}

} //hydra
