/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "linkedlist.h"

namespace hydra
{

Linkedlist::Linkedlist()
{
	m_count = 0;

	//
	m_first = NULL;
	m_last = NULL;
}
Linkedlist::~Linkedlist()
{
	removeAll();
}

bool Linkedlist::add(IUnknown * unknown)
{
	Node * newNode = new Node(unknown);
	if (!newNode)
		return false;

	//
	newNode->next = NULL;
	newNode->prev = m_last;

	if (m_last)
		m_last->next = newNode;

	if (!m_first)
		m_first = newNode;

	m_last = newNode;

	//
	m_count ++;

	return true;
}

bool Linkedlist::insertAfter(Node * node, IUnknown * unknown)
{
	Node * newNode = new Node(unknown);
	if (!newNode)
		return false;

	if (!node)
	{
		m_first->prev = newNode;

		newNode->prev = NULL;
		newNode->next = m_first;

		m_first = newNode;
	}
	else
	{
		newNode->prev = node;
		newNode->next = node->next;
		if (node->next)
			node->next->prev = newNode;

		node->next = newNode;
	}

	//
	m_count ++;

	return true;
}
bool Linkedlist::insertBefore(Node * node, IUnknown * unknown)
{
	if (!node)
		add(unknown);
	else
	{
		Node * newNode = new Node(unknown);
		if (!newNode)
			return false;

		newNode->prev = node->prev;
		newNode->next = node;
		if (node->prev)
			node->prev->next = newNode;

		node->prev = newNode;

		// If we were pointing at the tail, we are no longer pointing
		// at it because we put a new element after the current one.
		if (m_first == node)
			m_first = newNode;
	}

	//
	m_count ++;

	return true;
}

void Linkedlist::removeNode(Node * node)
{
	if (node == m_first)
		m_first = m_first->next;

	// Remove this list
	Node * prev = node->prev;
	Node * next = node->next;

	if (prev)
		prev->next = next;
	else
		m_first = next;

	if (next)
		next->prev = prev;
	else
		m_last = prev;

   	delete node;
	node = NULL;

	//
	m_count --;
}
void Linkedlist::removeLink(Node * node)
{
	if (node == m_first)
		m_first = m_first->next;

	// Remove this list
	Node * prev = node->prev;
	Node * next = node->next;

	if (prev)
		prev->next = next;
	else
		m_first = next;

	if (next)
		next->prev = prev;
	else
		m_last = prev;

	//
	m_count --;
}
void Linkedlist::removeAll()
{
	// Walk linked list and destroy all Format nodes
	Node * current = m_first;
	while (current)
	{
		Node * prev = current->prev;
		Node * next = current->next;

		// Remove node from List
		if (prev)
			prev->next = next;
		else
			m_first = next;

		if (next)
			next->prev = prev;
		else
			m_last = prev;

		// Destroy this node
		delete current;

		// Move to next node in list
		current = next;
	}

	//
	m_count = 0;
	m_first = m_last = NULL;
}

Linkedlist::Node * Linkedlist::find(IUnknown * unknown)
{
	Node * node = m_first;
	while (node)
	{
		if (node->unknown == unknown)
			break;

		// Next Node
		node = node->next;
	}

	return node;
}

void Linkedlist::remove(IUnknown * unknown)
{
	Node * node = m_first;
	while (node)
	{
		if (node->unknown == unknown)
		{
			removeNode(node);
			break;
		}

		// Next Node
		node = node->next;
	}
}

} //hydra

