/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "module.h"

#if defined(__APPLE__) || defined(_LINUX)
#include <dlfcn.h>	// for dlopen, dlsym, dlclose
#endif //_LINUX

namespace hydra
{

Module::Module()
{
	m_handle = NULL;

	//
	pfnload = NULL;
	pfnunload = NULL;

	pfngetInfo = NULL;

	//
	m_pmodule = NULL;
}
Module::~Module()
{
	if (pfnunload)
	{
		pfnunload();
		pfnunload = NULL;
	}

	if (m_handle)
	{
#if defined(_WIN32)
		FreeLibrary(m_handle);
#elif defined(__APPLE__) || defined(_LINUX)
		dlclose(m_handle);
#endif //_WIN32

		m_handle = NULL;
	}
}

//
bool Module::loadFromFile(const char * pathName)
{
	//TRACE("# Module::loadFromFile(%s)\n", pathName)

	if (m_handle)
		return true;

	//
	char fullPath[MAX_PATH];

#if defined(_WIN32)
	char fileName[MAX_PATH];
	strcpy(fileName, pathName);
	strcat(fileName, ".dll");

	makeRootPath(fullPath, fileName);

	int length = (int) strlen(fileName);
	for (int i = 0; i < length; i++)
	{
		if (fullPath[i] == '\\')
			fullPath[i] = '/';
	}

	char * ptr = fullPath;
	if (!strncmp(fullPath, "\\\\", 2))
	{
		//convert universal path to windows path
		// ("//d/" -> "d:\")
		fullPath[1] = fullPath[2];
		fullPath[2] = '\\';
		ptr = fullPath + 1;
	}

	//
	m_handle = LoadLibrary(ptr);
	if (m_handle == NULL)
	{
		TRACE0(">> ERROR: LoadLibrary(%s)\n", ptr)
		return false;
	}
#elif defined(__APPLE__)
	char fileName[MAX_PATH];
	strcpy(fileName, pathName);
	strcat(fileName, ".dylib");

	makeRootPath(fullPath, fileName);

	//
	m_handle = dlopen(fullPath, RTLD_LAZY);
	if (m_handle == NULL)
	{
		TRACE0(">> ERROR: dlopen(%s): %s\n", fullPath, dlerror())
		return false;
	}
#elif defined(_LINUX)
	char fileName[MAX_PATH];
	strcpy(fileName, "lib");
	strcat(fileName, pathName);
	strcat(fileName, ".so");

	makeRootPath(fullPath, fileName);

	//
	m_handle = dlopen(fullPath, RTLD_LAZY);
	if (m_handle == NULL)
	{
		TRACE0(">> ERROR: dlopen(%s):%s\n", fullPath, dlerror())
		return false;
	}
#endif //_WIN32

	//
	pfnload = (PFNLOADMODULEPROC) getSymbol("loadModule");
	pfnunload = (PFNUNLOADMODULEPROC) getSymbol("unloadModule");

	//
	pfngetInfo = (PFNGETMODULEINFOPROC) getSymbol("getModuleInfo");

	//
	m_pmodule = pfnload();

	//
	const MODULEINFO * info = pfngetInfo();
	if (!strcmp(info->category, "AUDIO") || !strcmp(info->category, "GRAPHICS"))
	{
		ASSERT(m_pmodule)
	}

	return true;
}

//
void Module::dump()
{
	TRACE("# Module::dump()\n")

	//
	const MODULEINFO * info = pfngetInfo();
	ASSERT(info)

	TRACE(">> name: %s\n", info->name)
	TRACE(">> description: %s\n", info->description)
	TRACE(">> category: %s\n", info->category)
	TRACE(">> copyright: %s\n", info->copyright)
	TRACE(">> author: %s\n", info->author)
	TRACE(">> version: %s\n", info->version)
}

//
void * Module::getSymbol(const char * name)
{
	ASSERT(m_handle)

#if defined(_WIN32)
	void * symbol = (void *) GetProcAddress(m_handle, name);
	if (!symbol)
	{
		TRACE0(">> ERROR: GetProcAddress(%s) : %d\n", name, GetLastError())
	}
#elif defined(__APPLE__) || defined(_LINUX)
	void * symbol = dlsym(m_handle, name);
	if (!symbol)
	{
		TRACE0(">> ERROR: dlsym(%s) : %s\n", name, dlerror())
	}
#endif

	ASSERT(symbol)
	return symbol;
}

} //hydra
