//
// reflect.fx
//

// Transformations
float4x4 view;
float4x4 world;
float4x4 worldI;
float4x4 projection;

///////////////////////////////////////////////////////////////////////////////
struct VS_INPUT
{
    float3 position: POSITION;
    float3 normal: NORMAL;
    float3 diffuse: COLOR0;
    float  occ: TEXCOORD0;
    float2 tex0: TEXCOORD0;
    float2 tex1: TEXCOORD1;
    float  fog: FOG;
};

struct VS_OUTPUT
{
    float4 position: POSITION;
    float4 diffuse: COLOR0;
    float4 specular: COLOR1;
    float2 tex0: TEXCOORD0;
    float2 tex1: TEXCOORD1;
    float3 env: TEXCOORD2;
    float  reflect: TEXCOORD3;
    float  fog: FOG;
};

// Direction of light from sky (view space)
float4 dirFromSky = { 0.0f, -1.0f, 0.0f, 0 };            

// Light directions (view space)
float4 dirFromLight = {0.577, -0.577, 0.577, 0};
float4 lightPos = {-0.577, -0.577, 0.577,0.0};

// Light intensity
float4 I_a = float4(0.5f, 0.5f, 0.5f, 1.0f);        // Ambient
float4 I_d = float4(1.0f, 0.9f, 0.8f, 1.0f);        // Diffuse
float4 I_s = float4(1.0f, 1.0f, 1.0f, 1.0f);        // Specular
float4 I_b = float4(0.1f, 0.0f, 0.0f, 1.0f);        // Ground
float4 I_c = float4(0.9f, 0.9f, 1.0f, 1.0f);        // Sky

// material reflectivity
float4 k_a = float4(0.8f, 0.8f, 0.8f, 1.0f);        // Ambient
float4 k_d = float4(0.4f, 0.4f, 0.4f, 1.0f);        // Diffuse
float4 k_s = float4(0.5f, 0.5f, 0.5f, 1.0f);        // Specular

float n = 32.0f;                                // Power

//setFog(float start, float end) { fogEnd = end, fogStart = (1.0f / (end - start)); }
float fogStart = 1.0f;
float fogEnd   = 1.0f;

sampler TEX0 : register(s0);
sampler TEX1 : register(s1);
sampler TEX2 : register(s2);

float4x4 float4x4Inv(float4x4 m)
{
    float4x4 adj;
    
    // Calculate the adjoint matrix
    adj._11 = m._22*m._33*m._44 + m._23*m._34*m._42 + m._24*m._32*m._43 - m._22*m._34*m._43 - m._23*m._32*m._44 - m._24*m._33*m._42;
    adj._12 = m._12*m._34*m._43 + m._13*m._32*m._44 + m._14*m._33*m._42 - m._12*m._33*m._44 - m._13*m._34*m._42 - m._14*m._32*m._43;
    adj._13 = m._12*m._23*m._44 + m._13*m._24*m._42 + m._14*m._22*m._43 - m._12*m._24*m._43 - m._13*m._22*m._44 - m._14*m._23*m._42;
    adj._14 = m._12*m._24*m._33 + m._13*m._22*m._34 + m._14*m._23*m._32 - m._12*m._23*m._34 - m._13*m._24*m._32 - m._14*m._22*m._33;
    
    adj._21 = m._21*m._34*m._43 + m._23*m._31*m._44 + m._24*m._33*m._41 - m._21*m._33*m._44 - m._23*m._34*m._41 - m._24*m._31*m._43;
    adj._22 = m._11*m._33*m._44 + m._13*m._34*m._41 + m._14*m._31*m._43 - m._11*m._34*m._43 - m._13*m._31*m._44 - m._14*m._33*m._41;
    adj._23 = m._11*m._24*m._43 + m._13*m._21*m._44 + m._14*m._23*m._41 - m._11*m._23*m._44 - m._13*m._24*m._41 - m._14*m._21*m._43;
    adj._24 = m._11*m._23*m._34 + m._13*m._24*m._31 + m._14*m._21*m._33 - m._11*m._24*m._33 - m._13*m._21*m._34 - m._14*m._23*m._31;
    
    adj._31 = m._21*m._32*m._44 + m._22*m._34*m._41 + m._24*m._31*m._42 - m._21*m._34*m._42 - m._22*m._31*m._44 - m._24*m._32*m._41;
    adj._32 = m._11*m._34*m._42 + m._12*m._31*m._44 + m._14*m._32*m._41 - m._11*m._32*m._44 - m._12*m._34*m._41 - m._14*m._31*m._42;
    adj._33 = m._11*m._22*m._44 + m._12*m._24*m._41 + m._14*m._21*m._42 - m._11*m._24*m._42 - m._12*m._21*m._44 - m._14*m._22*m._41;
    adj._34 = m._11*m._24*m._32 + m._12*m._21*m._34 + m._14*m._22*m._31 - m._11*m._22*m._34 - m._12*m._24*m._31 - m._14*m._21*m._32;
    
    adj._41 = m._21*m._33*m._42 + m._22*m._31*m._43 + m._23*m._32*m._41 - m._21*m._32*m._43 - m._22*m._33*m._41 - m._23*m._31*m._42;
    adj._42 = m._11*m._32*m._43 + m._12*m._33*m._41 + m._13*m._31*m._42 - m._11*m._33*m._42 - m._12*m._31*m._43 - m._13*m._32*m._41;
    adj._43 = m._11*m._23*m._42 + m._12*m._21*m._43 + m._13*m._22*m._41 - m._11*m._22*m._43 - m._12*m._23*m._41 - m._13*m._21*m._42;
    adj._44 = m._11*m._22*m._33 + m._12*m._23*m._31 + m._13*m._21*m._32 - m._11*m._23*m._32 - m._12*m._21*m._33 - m._13*m._22*m._31;
    
    // Calculate the determinant
    float det = determinant(m);
    
    float4x4 result = 0;
    if( 0.0f != det )
        result = 1.0f / det * adj;
        
    return result;
}
/*
// very simple - we don't need lighting as it is provided by the lightmap.
///////////////////////////////////////////////////////////////////////////////
VS_OUTPUT vs_main(VS_INPUT IN)
{
    VS_OUTPUT OUT = (VS_OUTPUT) 0;

    float4x4 worldView = mul(world, view);

    float3 L = -dirFromLight;
    
    float3 P = mul(float4(IN.position, 1), (float4x3) worldView);	// position (view space)
    float3 N = normalize(mul(IN.normal, (float3x3) worldView));		// normal (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);					// reflection vector (view space)
	float3 V = -normalize(P);										// view direction (view space)

    float fog = (-P.z + fogEnd) * fogStart;
    
    float4 ambient = k_a * I_a;
    float4 diffuse = k_d * I_d * max(0, dot(N, L));
    float4 specular = k_s * I_s * pow(max(0, dot(R, V)), n/4);

    OUT.position = mul(float4(P, 1), projection);								// position (projected)
//    OUT.fog      = min(max(fog, 0), 1.0f);
        
    OUT.diffuse  = ambient + diffuse;
    OUT.specular = specular;
    
    float4x4 viewInv = float4x4Inv(view);
    OUT.env = mul(R, (float3x3) viewInv);
    
    OUT.reflect = 0.1f;
        
    OUT.tex0 = IN.tex0;
    OUT.tex1.x = max(0, dot(L, N));
	OUT.tex1.y = 0;
	    
    return OUT;
}
*/

// very simple - we don't need lighting as it is provided by the lightmap.
///////////////////////////////////////////////////////////////////////////////
VS_OUTPUT vs_main(VS_INPUT IN)
{
    VS_OUTPUT OUT = (VS_OUTPUT) 0;

    float4x4 worldView = mul(world, view);
    
    float3 L = -dirFromLight;
    float3 Y = -dirFromSky;											// hemisphere up axis

    float3 P = mul(float4(IN.position, 1), (float4x3) worldView);	// position (view space)
    float3 N = normalize(mul(IN.normal, (float3x3) worldView));		// normal (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);					// reflection vector (view space)
    float3 V = -normalize(P);										// view direction (view space)
    
    float fog = (-P.z + fogEnd) * fogStart;
    
    //float4 hemisphere = k_a * lerp(I_b, I_c, (dot(N, Y) + 1) / 2) * (1 - IN.occ);
    float4 hemisphere = k_a * lerp(I_b, I_c, (dot(N, Y) + 1) / 2) * (1 - IN.tex0.x);

    float temp = 1 - max(0, dot(N, L));
    
    float4 ambient = k_a * I_a;
    float4 diffuse = k_d * I_d * (1 - temp * temp);
    
    float4 specular = k_s * I_s * pow(max(0, dot(R, V)), n/4);

    OUT.position = mul(float4(P, 1), projection);								// position (projected)
    OUT.fog      = min(max(fog, 0), 1.0f);
        
    OUT.diffuse  = ambient + diffuse + hemisphere;
    OUT.specular = specular;
    
    float4x4 viewInv = float4x4Inv(view);
    OUT.env = mul(R, (float3x3) viewInv);
    
    OUT.reflect = 0.1f;
    
    //
    OUT.tex0 = IN.tex0;
    OUT.tex1 = IN.tex0;
    
    return OUT;
}

/*
// draws unskinned object with one texture and one directional light.
VS_OUTPUT vs_main(VS_INPUT IN)
{
    VS_OUTPUT OUT = (VS_OUTPUT) 0;

    float4x4 worldView = mul(world, view);

    float3 L = -dirFromSky;											// hemisphere up axis
    
    float3 P = mul(float4(IN.position, 1), (float4x3) worldView);	// position (view space)
    float3 N = normalize(mul(IN.normal, (float3x3) worldView));		// normal (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);					// reflection vector (view space)
    float3 V = -normalize(P);										// view direction (view space)

    float fog = (-P.z + fogEnd) * fogStart;

    float temp = 1 - max(0, dot(N, L));
        
    float4 ambient = k_a * I_a;
    float4 diffuse = k_d * I_d * (1 - temp * temp);    
    float4 specular = k_s * I_s * pow(max(0, dot(R, V)), n/4);
    
    OUT.position = mul(float4(P, 1), projection);					// position (projected)
    OUT.fog      = min(max(fog, 0), 1.0f);
    
    OUT.diffuse  = ambient + diffuse;								// diffuse 
    OUT.specular = specular;

    float4x4 viewInv = float4x4Inv(view);
    OUT.env = mul(R, (float3x3) viewInv);
    
    OUT.reflect = 0.1f;

    //
    OUT.tex0 = IN.tex0;
    OUT.tex1 = IN.tex0;

    return OUT;    
}
*/
/*
// Draws unskinned object with one texture and one directional light.
VS_OUTPUT vs_main(VS_INPUT IN)
{
    VS_OUTPUT OUT = (VS_OUTPUT) 0;

    float4x4 worldView = mul(world, view);

    float3 L = -dirFromLight;
    
    float3 P = mul(float4(IN.position, 1), (float4x3) worldView);	// position (view space)
    float3 N = normalize(mul(IN.normal, (float3x3) worldView));		// normal (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);					// reflection vector (view space)
    float3 V = -normalize(P);										// view direction (view space)
    
    float fog = (-P.z + fogEnd) * fogStart;
    
    float4 ambient = k_a * I_a;
    float4 diffuse = k_d * I_d * max(0, dot(N, L));
    float4 specular = k_s * I_s * pow(max(0, dot(R, V)), n/4);
    
    OUT.position = mul(float4(P, 1), projection);					// position (projected)
    OUT.fog      = min(max(fog, 0), 1.0f);
    
    OUT.diffuse  = ambient + diffuse;								// diffuse 
    OUT.specular = specular;										// specular

    float4x4 viewInv = float4x4Inv(view);
    OUT.env = mul(R, (float3x3) viewInv);
    
    OUT.reflect = 0.1f;

    //
    OUT.tex0 = IN.tex0;
    //OUT.tex1 = IN.tex0;

    return OUT;    
}
*/

float4 ps_main(VS_OUTPUT IN) : COLOR0
{
    float4 diffuse = ((tex2D(TEX0, IN.tex0) * IN.diffuse) + IN.specular) * (1 - IN.reflect);
    float4 toon = tex2D(TEX1, IN.tex1);
    float4 env = tex2D(TEX2, IN.env) * IN.reflect;
    
	float4 color = diffuse + env;// + toon;
//	color.w = 1.0f;
	
    return color;
}
