//
// Simple.vpu
//

// Transformations
float4x4 view;
float4x4 world;
float4x4 worldI;
float4x4 projection;

///////////////////////////////////////////////////////////////////////////////
struct VS_INPUT
{
    float3 position: POSITION;
    float3 normal: NORMAL;
    float3 diffuse: COLOR0;
    float  occ: TEXCOORD0;
    float2 tex0: TEXCOORD0;
    float2 tex1: TEXCOORD1;
    float  fog: FOG;
};

struct VS_OUTPUT
{
    float4 position: POSITION;
    float4 diffuse: COLOR0;
    float4 specular: COLOR1;
    float2 tex0: TEXCOORD0;
    float2 tex1: TEXCOORD1;
    float  fog: FOG;
};

// Direction of light from sky (view space)
float4 dirFromSky = { 0.0f, -1.0f, 0.0f, 0 };            

// Light directions (view space)
float4 dirFromLight = {0.577, -0.577, 0.577, 0};
float4 lightPos = {-0.577, -0.577, 0.577,0.0};

// Light intensity
float4 I_a = float4(0.5f, 0.5f, 0.5f, 1.0f);        // Ambient
float4 I_d = float4(1.0f, 0.9f, 0.8f, 1.0f);        // Diffuse
float4 I_s = float4(1.0f, 1.0f, 1.0f, 1.0f);        // Specular
float4 I_b = float4(0.1f, 0.0f, 0.0f, 1.0f);        // Ground
float4 I_c = float4(0.9f, 0.9f, 1.0f, 1.0f);        // Sky

// material reflectivity
float4 k_a = float4(0.8f, 0.8f, 0.8f, 1.0f);        // Ambient
float4 k_d = float4(0.4f, 0.4f, 0.4f, 1.0f);        // Diffuse
float4 k_s = float4(0.5f, 0.5f, 0.5f, 1.0f);        // Specular

float n = 32.0f;                                // Power

float4 G_c = float4(0.5f, 0.2f, 0.2f, 1.0f);
float4 G_a = float4(0.2f, 0.2f, 0.0f, 0.0f);
float  G_t = 0.015f;

//setFog(float start, float end) { fogEnd = end, fogStart = (1.0f / (end - start)); }
float fogStart = 1.0f;
float fogEnd   = 1.0f;

sampler TEX0 : register(s0);
sampler TEX1 : register(s1);

/*
// very simple - we don't need lighting as it is provided by the lightmap.
///////////////////////////////////////////////////////////////////////////////
VS_OUTPUT vs_main(VS_INPUT IN)
{
    VS_OUTPUT OUT = (VS_OUTPUT) 0;

    float4x4 worldView = mul(world, view);

    float3 L = -dirFromLight;
    
    float3 P = mul(float4(IN.position, 1), (float4x3) worldView);	// position (view space)
    float3 N = normalize(mul(IN.normal, (float3x3) worldView));		// normal (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);					// reflection vector (view space)
	float3 V = -normalize(P);										// view direction (view space)

    float fog = (-P.z + fogEnd) * fogStart;
    
    float4 ambient = k_a * I_a;
    float4 diffuse = k_d * I_d * max(0, dot(N, L));
    float4 specular = k_s * I_s * pow(max(0, dot(R, V)), n/4);

    OUT.position = mul(float4(P, 1), projection);								// position (projected)
//    OUT.fog      = min(max(fog, 0), 1.0f);
        
    OUT.diffuse  = ambient + diffuse;
    OUT.specular = specular;
    
    OUT.tex0 = IN.tex0;
    OUT.tex1.x = max(0, dot(L, N));
	OUT.tex1.y = 0;
	
    return OUT;
}
*/

// Glow
/*
VS_OUTPUT vs_main(VS_INPUT IN)
{
    VS_OUTPUT OUT = (VS_OUTPUT) 0;

    float4x4 worldView = mul(world, view);

    float3 L = -dirFromLight;
    
    float3 P = mul(float4(IN.position, 1), (float4x3) worldView);	// position (view space)
    float3 N = normalize(mul(IN.normal, (float3x3) worldView));		// normal (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);					// reflection vector (view space)
	float3 V = -normalize(P);										// view direction (view space)
    float3 A = float3(0, 0, 1);										// glow axis

    P += G_t * N;													// displaced position (view space)

    float power = dot(N, A);
    power *= power;
    power -= 1;
    power *= power;													// Power = (1 - (N.A)^2)^2 [ = ((N.A)^2 - 1)^2 ]
    
    OUT.position = mul(float4(P, 1), projection);					// projected position
    OUT.diffuse  = G_c * power + G_a;								// modulated glow color + glow ambient    
    //OUT.fog = 1.0f;

    return OUT;
}
*/

// very simple - we don't need lighting as it is provided by the lightmap.
///////////////////////////////////////////////////////////////////////////////
VS_OUTPUT vs_main(VS_INPUT IN)
{
    VS_OUTPUT OUT = (VS_OUTPUT) 0;

    float4x4 worldView = mul(world, view);
    
    float3 L = -dirFromLight;
    float3 Y = -dirFromSky;											// hemisphere up axis

    float3 P = mul(float4(IN.position, 1), (float4x3) worldView);	// position (view space)
    float3 N = normalize(mul(IN.normal, (float3x3) worldView));		// normal (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);					// reflection vector (view space)
    float3 V = -normalize(P);										// view direction (view space)
    
    float fog = (-P.z + fogEnd) * fogStart;
    
    //float4 hemisphere = k_a * lerp(I_b, I_c, (dot(N, Y) + 1) / 2) * (1 - IN.occ);
    float4 hemisphere = k_a * lerp(I_b, I_c, (dot(N, Y) + 1) / 2) * (1 - IN.tex0.x);

    float temp = 1 - max(0, dot(N, L));
    
    float4 ambient = k_a * I_a;
    float4 diffuse = k_d * I_d * (1 - temp * temp);
    float4 specular = k_s * I_s * pow(max(0, dot(R, V)), n/4);

    OUT.position = mul(float4(P, 1), projection);								// position (projected)
    OUT.fog      = min(max(fog, 0), 1.0f);
        
    OUT.diffuse  = ambient + diffuse + hemisphere;
    OUT.specular = specular;
    
    //
    OUT.tex0 = IN.tex0;
    OUT.tex1 = IN.tex0;
    
    return OUT;
}

// draws unskinned object with one texture and one directional light.
/*VS_OUTPUT vs_main(VS_INPUT IN)
{
    VS_OUTPUT OUT = (VS_OUTPUT) 0;

    float4x4 worldView = mul(world, view);

    float3 L = -dirFromSky;											// hemisphere up axis
    
    float3 P = mul(float4(IN.position, 1), (float4x3) worldView);	// position (view space)
    float3 N = normalize(mul(IN.normal, (float3x3) worldView));		// normal (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);					// reflection vector (view space)
    float3 V = -normalize(P);										// view direction (view space)

    float fog = (-P.z + fogEnd) * fogStart;

    float temp = 1 - max(0, dot(N, L));
        
    float4 ambient = k_a * I_a;
    float4 diffuse = k_d * I_d * (1 - temp * temp);    
    float4 specular = k_s * I_s * pow(max(0, dot(R, V)), n/4);
    
    OUT.position = mul(float4(P, 1), projection);					// position (projected)
    OUT.fog      = min(max(fog, 0), 1.0f);
    
    OUT.diffuse  = ambient + diffuse;								// diffuse 
    OUT.specular = specular;

    //
    OUT.tex0 = IN.tex0;
    OUT.tex1 = IN.tex0;

    return OUT;    
}*/

/*
// Draws unskinned object with one texture and one directional light.
VS_OUTPUT vs_main(VS_INPUT IN)
{
    VS_OUTPUT OUT = (VS_OUTPUT) 0;

    float4x4 worldView = mul(world, view);

    float3 L = -dirFromLight;
    
    float3 P = mul(float4(IN.position, 1), (float4x3) worldView);	// position (view space)
    float3 N = normalize(mul(IN.normal, (float3x3) worldView));		// normal (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);					// reflection vector (view space)
    float3 V = -normalize(P);										// view direction (view space)
    
    float fog = (-P.z + fogEnd) * fogStart;
    
    float4 ambient = k_a * I_a;
    float4 diffuse = k_d * I_d * max(0, dot(N, L));
    float4 specular = k_s * I_s * pow(max(0, dot(R, V)), n/4);
    
    OUT.position = mul(float4(P, 1), projection);					// position (projected)
    OUT.fog      = min(max(fog, 0), 1.0f);
    
    OUT.diffuse  = ambient + diffuse;								// diffuse 
    OUT.specular = specular;										// specular

    //
    OUT.tex0 = IN.tex0;
    //OUT.tex1 = IN.tex0;

    return OUT;    
}
*/

float4 ps_main(VS_OUTPUT IN) : COLOR0
{
    float4 diffuse = (tex2D(TEX0, IN.tex0) * IN.diffuse) + IN.specular;
    float4 toon = tex2D(TEX1, IN.tex1);

	float4 color = diffuse;// + toon;
	//color.w = 1.0f;
	
    return color;
}
