/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "direct3d9-cgshader.h"

#include "direct3d9-sampler.h"

#if defined(_NVCG)

namespace hydra
{

IMPLEMENT_MODULE(direct3d9CgShader, usr/lib/graphics/shader.cg)

direct3d9CgShader::direct3d9CgShader()
{

}

bool direct3d9CgShader::loadFromMemory(const char * code)
{
	CGcontext cgContext = _direct3d9->getCgContext();

	CGprofile profile = cgD3D9GetLatestVertexProfile();
	const char ** profileOpts = cgD3D9GetOptimalOptions(profile);

	m_vertexProgram = cgCreateProgram(cgContext,
		CG_SOURCE,
		code,
		profile,
		"vs_main",
		profileOpts);
	if (!m_vertexProgram)
	{
		const char * listing = cgGetLastListing(cgContext);
		if (!listing)
			listing = "No listing available!";

		TRACE("# direct3d9CgShader::loadFromMemory()\n")
		TRACE0(">> ERROR: cgCreateProgram(vs_main) : %s\n", listing)

		return false;
	}

	profile = cgD3D9GetLatestPixelProfile();
	profileOpts = cgD3D9GetOptimalOptions(profile);

	m_fragmentProgram = cgCreateProgram(cgContext,
		CG_SOURCE,
		code,
		profile,
		"ps_main",
		NULL);
	if (!m_fragmentProgram)
	{
		const char * listing = cgGetLastListing(cgContext);
		if (!listing)
			listing = "No listing available!";

		TRACE("# direct3d9CgShader::loadFromMemory()\n")
		TRACE0(">> ERROR: cgCreateProgram(ps_main) : %s\n", listing)

		return false;
	}

	return true;
}
bool direct3d9CgShader::loadFromMemory(const char * vpcode, const char * fpcode)
{
	return true;
}

//
void direct3d9CgShader::begin()
{
	cgD3D9LoadProgram(m_vertexProgram, false, 0);
	cgD3D9LoadProgram(m_fragmentProgram, false, 0);

	cgD3D9BindProgram(m_vertexProgram);
	cgD3D9BindProgram(m_fragmentProgram);
}
void direct3d9CgShader::end()
{
}

//
void direct3d9CgShader::setSampler(int stage, ISampler * sampler)
{
	if (sampler)
	{
		LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
		device->SetTexture(stage, ((direct3d9Sampler *) sampler)->getHandle());
		return;
	}

	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	device->SetTexture(stage, NULL);
}

static _D3DTEXTUREFILTERTYPE __TEXTUREFILTERTYPE[] =
{
	D3DTEXF_NONE,
	D3DTEXF_POINT,
	D3DTEXF_LINEAR,
	D3DTEXF_ANISOTROPIC,
};
static _D3DTEXTUREADDRESS __TEXTUREADDRESS[] =
{
	D3DTADDRESS_WRAP,
	D3DTADDRESS_CLAMP,
};
static _D3DSAMPLERSTATETYPE __SAMPLERSTATETYPE[] =
{
	D3DSAMP_ADDRESSU,
	D3DSAMP_ADDRESSV,
	D3DSAMP_MAGFILTER,
	D3DSAMP_MINFILTER,
	D3DSAMP_MIPFILTER,
};
void direct3d9CgShader::setSamplerState(unsigned long sampler, int type, unsigned long value)
{
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();

	switch (type)
	{
	case SAMP_ADDRESSU:
	case SAMP_ADDRESSV:
		device->SetSamplerState(sampler, __SAMPLERSTATETYPE[type], __TEXTUREADDRESS[value]);
		break;

	case SAMP_MAGFILTER:
	case SAMP_MINFILTER:
	case SAMP_MIPFILTER:
		device->SetSamplerState(sampler, __SAMPLERSTATETYPE[type], __TEXTUREFILTERTYPE[value]);
		break;
	}
}

} //hydra

#endif //_NVCG
