/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "direct3d9-indexbuffer.h"

#include "direct3d9-vertexbuffer.h"

namespace hydra
{
IMPLEMENT_MODULE(direct3d9Indexbuffer, usr/lib/graphics/indexbuffer)

direct3d9Indexbuffer::direct3d9Indexbuffer()
{
#if defined(_WIN32) || defined(_XBOX)
	m_buffer = NULL;
#endif //_WIN32
}

direct3d9Indexbuffer::~direct3d9Indexbuffer()
{
#if defined(_WIN32) || defined(_XBOX)
	if (m_buffer)
	{
		m_buffer->Release();
		m_buffer = NULL;
	}
#endif //_WIN32
}

bool direct3d9Indexbuffer::create(int length, unsigned long format)
{
	if (!IIndexbuffer::create(length, format))
		return false;

	//
	unsigned int stride;
	switch (format)
	{
	case FMT_INDEX16:
		stride = sizeof(unsigned short);
		break;
	case FMT_INDEX32:
		stride = sizeof(unsigned int);
		break;
	}

#if defined(_WIN32) || defined(_XBOX)
	D3DFORMAT __format[] = { D3DFMT_INDEX16, D3DFMT_INDEX32 };

	// Create d3d9IndexBuffer
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	if (FAILED(device->CreateIndexBuffer(length * stride,
		D3DUSAGE_WRITEONLY,
		__format[format],
		D3DPOOL_MANAGED,
		&m_buffer,
		NULL)))
		return false;
#endif //_WIN32

	return true;
}

void * direct3d9Indexbuffer::map()
{
	// Fill the vertex buffer. We are setting the tu and tv texture
	// coordinates, which range from 0.0 to 1.0
	void * indices = NULL;

#if defined(_WIN32) || defined(_XBOX)
	if (FAILED(m_buffer->Lock(0, 0, (void **) &indices, 0)))
		return NULL;
#endif //_WIN32

	return indices;
}
void direct3d9Indexbuffer::unmap()
{
#if defined(_WIN32) || defined(_XBOX)
	m_buffer->Unlock();
#endif //_WIN32
}

//
void direct3d9Indexbuffer::bind()
{
#if defined(_WIN32) || defined(_XBOX)
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	device->SetIndices(m_buffer);
#endif //_WIN32
}

#if defined(_WIN32) || defined(_XBOX)
// Primitives supported by draw-primitive API
static _D3DPRIMITIVETYPE __PRIMITIVETYPE[] = {
	D3DPT_POINTLIST,
	D3DPT_LINELIST,
	D3DPT_LINESTRIP,
	D3DPT_TRIANGLELIST,
	D3DPT_TRIANGLESTRIP,
	D3DPT_TRIANGLEFAN,
	D3DPT_TRIANGLEFAN,
	D3DPT_FORCE_DWORD, /* force 32-bit size enum */
};
static int __div[] = {
	3,
	3,
	2,
	3,
	2,
	2,
	2,
	D3DPT_FORCE_DWORD, /* force 32-bit size enum */
};
#endif //_WIN32

void direct3d9Indexbuffer::draw(int primitiveType)
{
#if defined(_WIN32) || defined(_XBOX)
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	device->DrawIndexedPrimitive(__PRIMITIVETYPE[primitiveType],
		0,
		0,
		0,
		0,
		m_length / __div[primitiveType]);
#endif //_WIN32
}
void direct3d9Indexbuffer::draw(int primitiveType, unsigned int startIndex, unsigned int indexCount, unsigned int numVertices)
{
#if defined(_WIN32) || defined(_XBOX)
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	device->DrawIndexedPrimitive(__PRIMITIVETYPE[primitiveType],
		0,
		0,
		numVertices,
		startIndex,
		indexCount / __div[primitiveType]);
#endif //_WIN32
}
void direct3d9Indexbuffer::draw(int primitiveType, unsigned int startVertex, unsigned int numVertices, unsigned int startIndex, unsigned int indexCount)
{
#if defined(_WIN32) || defined(_XBOX)
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	device->DrawIndexedPrimitive(__PRIMITIVETYPE[primitiveType],
		0,
		startVertex,
		numVertices,
		startIndex,
		indexCount / __div[primitiveType]);
#endif //_WIN32
}

} //hydra
