/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "direct3d9-sampler.h"

#include "graphics/iimage.h"

namespace hydra
{

IMPLEMENT_MODULE(direct3d9Sampler, usr/lib/graphics/sampler)

direct3d9Sampler::direct3d9Sampler()
{
#if defined(_WIN32) || defined(_XBOX)
	m_texture = NULL;
#endif //_WIN32
}
direct3d9Sampler::~direct3d9Sampler()
{
#if defined(_WIN32) || defined(_XBOX)
	if (m_texture)
	{
		m_texture->Release();
		m_texture = NULL;
	}
#endif //_WIN32
}

#if defined(_WIN32)
static _D3DFORMAT __FORMAT[] =
{
	D3DFMT_UNKNOWN,

	D3DFMT_X8R8G8B8,
	D3DFMT_A8R8G8B8,

	D3DFMT_R5G6B5,
	D3DFMT_A1R5G5B5,
	D3DFMT_A4R4G4B4,

	D3DFMT_A8,
	D3DFMT_L8,

	D3DFMT_A8L8,

	D3DFMT_DXT1,
	D3DFMT_DXT3,
	D3DFMT_DXT5,
};
#endif //_WIN32

bool direct3d9Sampler::create(int width, int height, unsigned long format, int mipLevels)
{
	if (!ISampler::create(width, height, format, mipLevels))
		return false;

#if defined(_WIN32)
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	if (FAILED(D3DXCreateTexture(device,
		width,
		height,
		m_mipLevels,
		0,
		__FORMAT[format],
		D3DPOOL_MANAGED,
		&m_texture)))
#endif //_WIN32

		return false;

	return true;
}

bool direct3d9Sampler::loadFromFile(const char * pathName)
{
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();

	//
	char fullPath[MAX_PATH];
	makeFullPath(fullPath, pathName);

	D3DXIMAGE_INFO info;
	if (FAILED(D3DXCreateTextureFromFileEx(device,
		fullPath, 
		D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT,
		0,
		D3DFMT_UNKNOWN,
		D3DPOOL_MANAGED,//D3DPOOL_DEFAULT,
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		0,
		&info,
		NULL, 
		&m_texture)))
	{
		TRACE0(">> ERROR: D3DXCreateTextureFromFile(%s)\n", pathName)
		return false;
	}

	m_width = info.Width;
	m_height = info.Height;

	//TRACE(">> width:%d, height:%d\n", m_width, m_height)

	return true;
}

//
void * direct3d9Sampler::map()
{
#if defined(_WIN32)
	if (FAILED(m_texture->LockRect(0, &m_locked, NULL, 0)))
		return NULL;

	return m_locked.pBits;
#else
    return NULL;
#endif //_WIN32
}
void direct3d9Sampler::unmap()
{
#if defined(_WIN32) || defined(_XBOX)
	m_texture->UnlockRect(0);
#endif //_WIN32
}

} //hydra
