/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "direct3d9-shader.h"

#include "direct3d9-sampler.h"

#if defined(_WIN32) || defined(_XBOX)

namespace hydra
{
IMPLEMENT_MODULE(direct3d9Shader, usr/lib/graphics/shader)

direct3d9Shader::direct3d9Shader()
{
	m_ext = ".fx";

	m_fragmentShader = NULL;
	m_fragmentConstTable = NULL;

	m_vertexShader = NULL;
	m_vertexConstTable = NULL;
}
direct3d9Shader::~direct3d9Shader()
{
	if (m_vertexConstTable)
	{
		m_vertexConstTable->Release();
		m_vertexConstTable = NULL;
	}
	if (m_fragmentConstTable)
	{
		m_fragmentConstTable->Release();
		m_fragmentConstTable = NULL;
	}

	if (m_fragmentShader)
	{
		m_fragmentShader->Release();
		m_fragmentShader = NULL;
	}
	if (m_vertexShader)
	{
		m_vertexShader->Release();
		m_vertexShader = NULL;
	}
}

//
bool direct3d9Shader::loadFromMemory(const char * code)
{
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	LPD3DXBUFFER buffer, errorMsg;

	// vertex shader
	const char * profile = D3DXGetVertexShaderProfile(device);
	if (FAILED(D3DXCompileShader(code,
		lstrlen(code),
		NULL,
		NULL,
		"vs_main",
		profile,
		0,
		&buffer,
		&errorMsg,
		&m_vertexConstTable)))
	{
		outputDebugString((char *) errorMsg->GetBufferPointer());
		errorMsg->Release();

		return false;
	}

	if (FAILED(device->CreateVertexShader((LPDWORD) buffer->GetBufferPointer(), &m_vertexShader)))
	{
		buffer->Release();
		return false;
	}
	buffer->Release();

	// fragment shader
	profile = D3DXGetPixelShaderProfile(device);
	if (FAILED(D3DXCompileShader(code,
		lstrlen(code),
		NULL,
		NULL,
		"ps_main",
		profile,
		0,
		&buffer,
		&errorMsg,
		&m_fragmentConstTable)))
	{
		outputDebugString((char *) errorMsg->GetBufferPointer());
		errorMsg->Release();

		return false;
	}

	if (FAILED(device->CreatePixelShader((LPDWORD) buffer->GetBufferPointer(), &m_fragmentShader)))
	{
		buffer->Release();
		return false;
	}
	buffer->Release();

	return true;
}
bool direct3d9Shader::loadFromMemory(const char * vpcode, const char * fpcode)
{
	return true;
}


//
void direct3d9Shader::begin()
{
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	device->SetPixelShader(m_fragmentShader);
	device->SetVertexShader(m_vertexShader);
}
void direct3d9Shader::end()
{

}

//
void direct3d9Shader::setTransform(float4x4 & transform)
{
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	m_vertexConstTable->SetMatrix(device, "world", (LPD3DXMATRIX) &transform);
}

//
void direct3d9Shader::setSampler(int stage, ISampler * sampler)
{
	if (sampler)
	{
		LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
		device->SetTexture(stage, ((direct3d9Sampler *) sampler)->getHandle());
		return;
	}

	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	device->SetTexture(stage, NULL);
}

static _D3DTEXTUREFILTERTYPE __TEXTUREFILTERTYPE[] =
{
	D3DTEXF_NONE,
	D3DTEXF_POINT,
	D3DTEXF_LINEAR,
	D3DTEXF_ANISOTROPIC,
};
static _D3DTEXTUREADDRESS __TEXTUREADDRESS[] =
{
	D3DTADDRESS_WRAP,
	D3DTADDRESS_CLAMP,
};
static _D3DSAMPLERSTATETYPE __SAMPLERSTATETYPE[] =
{
	D3DSAMP_ADDRESSU,
	D3DSAMP_ADDRESSV,
	D3DSAMP_MAGFILTER,
	D3DSAMP_MINFILTER,
	D3DSAMP_MIPFILTER,
};
void direct3d9Shader::setSamplerState(unsigned long sampler, int type, unsigned long value)
{
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();

	switch (type)
	{
	case SAMP_ADDRESSU:
	case SAMP_ADDRESSV:
		device->SetSamplerState(sampler, __SAMPLERSTATETYPE[type], __TEXTUREADDRESS[value]);
		break;

	case SAMP_MAGFILTER:
	case SAMP_MINFILTER:
	case SAMP_MIPFILTER:
		device->SetSamplerState(sampler, __SAMPLERSTATETYPE[type], __TEXTUREFILTERTYPE[value]);
		break;
	}
}

//
void direct3d9Shader::setFloat(const char * name, const float value)
{
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	m_vertexConstTable->SetFloat(device, name, value);
}
void direct3d9Shader::setFloatArray(const char * name, const float * value, int count)
{

}
void direct3d9Shader::setVector(const char * name, const float4 & value)
{
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	m_vertexConstTable->SetVector(device, name, (LPD3DXVECTOR4) &value);
}
void direct3d9Shader::setVectorArray(const char * name, const float4 * value, int count)
{

}
void direct3d9Shader::setMatrix(const char * name, const float4x4 & value)
{
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	m_vertexConstTable->SetMatrix(device, name, (LPD3DXMATRIX) &value);
}
void direct3d9Shader::setMatrixArray(const char * name, const float4x4 * value, int count)
{

}

} //hydra

#endif

