/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "direct3d9-vertexbuffer.h"

namespace hydra
{

IMPLEMENT_MODULE(direct3d9Vertexbuffer, usr/lib/graphics/vertexbuffer)

direct3d9Vertexbuffer::direct3d9Vertexbuffer()
{
#if defined(_WIN32) || defined(_XBOX)
	m_buffer = NULL;
	m_decl = NULL;
#endif //_WIN32
}

direct3d9Vertexbuffer::~direct3d9Vertexbuffer()
{
#if defined(_WIN32) || defined(_XBOX)
	if (m_decl)
	{
		m_decl->Release();
		m_decl = NULL;
	}

	if (m_buffer)
	{
		m_buffer->Release();
		m_buffer = NULL;
	}
#endif //_WIN32
}

//
bool direct3d9Vertexbuffer::create(int length, unsigned long fvf)
{
	if (!IVertexbuffer::create(length, fvf))
		return false;

#if defined(_WIN32) || defined(_XBOX)
	// Create VertexBuffer
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	if (FAILED(device->CreateVertexBuffer(length * m_stride,
		D3DUSAGE_WRITEONLY,
		fvf,
		D3DPOOL_DEFAULT,
		&m_buffer,
		NULL)))
		return false;

	// Create VertexDeclaration
	D3DVERTEXELEMENT9 decl[MAX_FVF_DECL_SIZE];
	D3DXDeclaratorFromFVF(fvf, decl);

	if (FAILED(device->CreateVertexDeclaration(decl,
		&m_decl)))
		return false;

// 	DWORD bfvf;
// 	D3DXFVFFromDeclarator(decl, &bfvf);
//
// 	outputDebugFormat(">> D3DXFVFFromDeclarator: %s\n", getFVF(bfvf));
#endif //_WIN32

	return true;
}

void * direct3d9Vertexbuffer::map()
{
	// Now we fill the vertex buffer. To do this, we need to Lock() the VB to
	// gain access to the vertices. This mechanism is required becuase vertex
	// buffers may be in device memory.
	void * vertices = NULL;

#if defined(_WIN32) || defined(_XBOX)
	if (FAILED(m_buffer->Lock(0, 0, (void**) &vertices, 0)))
		return NULL;
#endif //_WIN32

	return vertices;
}
void direct3d9Vertexbuffer::unmap()
{
#if defined(_WIN32) || defined(_XBOX)
	m_buffer->Unlock();
#endif //_WIN32
}

void direct3d9Vertexbuffer::bind()
{
#if defined(_WIN32) || defined(_XBOX)
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	device->SetVertexDeclaration(m_decl);
	device->SetStreamSource(0, m_buffer, 0, m_stride);
#endif //_WIN32
}

#if defined(_WIN32) || defined(_XBOX)
// Primitives supported by draw-primitive API
static _D3DPRIMITIVETYPE __PRIMITIVETYPE[] = {
	D3DPT_POINTLIST,
	D3DPT_LINELIST,
	D3DPT_LINESTRIP,
	D3DPT_TRIANGLELIST,
	D3DPT_TRIANGLESTRIP,
	D3DPT_TRIANGLEFAN,
	D3DPT_TRIANGLEFAN,
	D3DPT_FORCE_DWORD, /* force 32-bit size enum */
};

static int __div[] = {
	3,
	3,
	2,
	3,
	2,
	2,
	2,
	D3DPT_FORCE_DWORD, /* force 32-bit size enum */
};
#endif //_WIN32

void direct3d9Vertexbuffer::draw(int primitiveType)
{
	bind();

#if defined(_WIN32) || defined(_XBOX)
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	device->DrawPrimitive(__PRIMITIVETYPE[primitiveType], 0, m_length / __div[primitiveType]);
#endif //_WIN32
}
void direct3d9Vertexbuffer::draw(int primitiveType, unsigned int startVertex, unsigned int vertexCount)
{
#if defined(_WIN32) || defined(_XBOX)
	LPDIRECT3DDEVICE9 device = _direct3d9->getDevice();
	device->DrawPrimitive(__PRIMITIVETYPE[primitiveType], startVertex, vertexCount / __div[primitiveType]);
#endif //_WIN32
}

} //hydra
