/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "direct3d9-win32.h"

#if defined(_WIN32)
	extern HRESULT GetDXVersion(DWORD * pdwDirectXVersion, TCHAR * strDirectXVersion, int cchDirectXVersion);
	
	#if defined (_MSC_VER)
		#pragma comment(lib, "version.lib")
	#endif //_MSC_VER
#endif //_WIN32

namespace hydra
{
#if defined(_WIN32)
	static void setWindowPos(HWND hwnd, int x, int y, int width, int height);

	//
	static bool confirmDevice(D3DCAPS9 * caps, DWORD behavior);
	static DWORD findBehaviorFlags(D3DCAPS9 * caps);

	static D3DFORMAT findDepthStencilFormat(LPDIRECT3D9 d3d, int adapter, D3DDEVTYPE deviceType, D3DFORMAT format);
#endif //_WIN32


direct3d9Win32::direct3d9Win32()
{
}

//
direct3d9Win32::~direct3d9Win32()
{
#if defined(_WIN32)
#if defined(_NVCG)
	// tell the runtime to remove its reference to the D3D device.
	cgD3D9SetDevice(NULL);
#endif //_NVCG
#endif //_WIN32
}

//
bool direct3d9Win32::open(HWND hwnd)
{
	//TRACE("# direct3d9Win32::initialize()")

#if defined(_WIN32)
	// check DirectX Version
	DWORD version;
	TCHAR strVersion[10];

	GetDXVersion(&version, strVersion, 10);
	TRACE(">> Microsoft DirectX %s\n", strVersion)
	if (version < 0x90002)
		return false;

	//
	m_d3d = Direct3DCreate9(D3D_SDK_VERSION);
	if (!m_d3d)
		return false;

	// get the current desktop display mode, so we can set up a back
	// buffer of the same format
	D3DDISPLAYMODE dispMode;
	if (FAILED(m_d3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &dispMode)))
		return false;

	//
	D3DDEVTYPE deviceType = D3DDEVTYPE_HAL;

	D3DCAPS9 d3dCaps;
	if (FAILED(m_d3d->GetDeviceCaps(D3DADAPTER_DEFAULT, deviceType, &d3dCaps)))
	{
		deviceType = D3DDEVTYPE_REF;
		if (FAILED(m_d3d->GetDeviceCaps(D3DADAPTER_DEFAULT, deviceType, &d3dCaps)))
			return false;
	}

	//
	if (d3dCaps.PrimitiveMiscCaps & D3DPMISCCAPS_NULLREFERENCE)
		return false;

	//
	if (m_fullscreen)
	{
		const D3DFORMAT fmtFullscreenArray[] =
		{
			D3DFMT_A8R8G8B8,
			D3DFMT_X8R8G8B8,
			D3DFMT_X1R5G5B5,
			D3DFMT_A1R5G5B5,
			D3DFMT_R5G6B5,
		};

		const int numFullscreenFmts = sizeof(fmtFullscreenArray) / sizeof(fmtFullscreenArray[0]);
		D3DFORMAT fmtFullscreen;

		// find a pixel format that will be good for fullscreen back buffers
		for (int format = 0; format < numFullscreenFmts; format++)
		{
			if (SUCCEEDED(m_d3d->CheckDeviceType(D3DADAPTER_DEFAULT,
				deviceType,
				fmtFullscreenArray[format],
				fmtFullscreenArray[format],
				FALSE)))
			{
				fmtFullscreen = fmtFullscreenArray[format];
				break;
			}
		}

		//
		m_present.Windowed = FALSE;
		m_present.FullScreen_RefreshRateInHz = 60;

		m_present.BackBufferFormat = fmtFullscreen;
		m_present.BackBufferCount = 0;

		m_present.SwapEffect = D3DSWAPEFFECT_DISCARD;
	}
	else
	{
		setWindowPos(hwnd, 0, 0, m_width, m_height);

		//
		m_present.Windowed = TRUE;

		m_present.BackBufferFormat = dispMode.Format;
		m_present.BackBufferCount = 0;

		m_present.SwapEffect = D3DSWAPEFFECT_COPY;
	}

	m_present.hDeviceWindow = hwnd;

	m_present.BackBufferWidth = m_width;
	m_present.BackBufferHeight = m_height;

	m_present.Flags = D3DPRESENTFLAG_LOCKABLE_BACKBUFFER;
	m_present.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
	//m_present.PresentationInterval = D3DPRESENT_INTERVAL_ONE;

	m_present.EnableAutoDepthStencil = TRUE;
	m_present.AutoDepthStencilFormat = findDepthStencilFormat(m_d3d, D3DADAPTER_DEFAULT, deviceType, dispMode.Format);

	//
	D3DMULTISAMPLE_TYPE MultiSampleQuality = D3DMULTISAMPLE_NONE;//D3DMULTISAMPLE_2_SAMPLES;

	if (SUCCEEDED(m_d3d->CheckDeviceMultiSampleType(0,
		deviceType,
		m_present.BackBufferFormat,
		m_present.Windowed,
		MultiSampleQuality,
		NULL)))
	{
		if (SUCCEEDED(m_d3d->CheckDeviceMultiSampleType(0,
			deviceType,
			m_present.AutoDepthStencilFormat,
			m_present.Windowed,
			MultiSampleQuality, NULL)))
		{
			m_present.MultiSampleType = MultiSampleQuality;
			m_present.MultiSampleQuality = MultiSampleQuality;
		}
	}

	DWORD behavior = findBehaviorFlags(&d3dCaps) | D3DCREATE_FPU_PRESERVE;
	if (FAILED(m_d3d->CreateDevice(D3DADAPTER_DEFAULT,
		deviceType,
		hwnd,
		behavior,
		&m_present,
		&m_device)))
	{
		TRACE0(">> ERROR: Direct3D9::CreateDevice(D3DCREATE_HARDWARE_VERTEXPROCESSING)\n")

		//
		deviceType = D3DDEVTYPE_REF;
		if (FAILED(m_d3d->CreateDevice(D3DADAPTER_DEFAULT,
			deviceType,
			hwnd,
			behavior,
			&m_present,
			&m_device)))
		{
			TRACE0(">> ERROR: Direct3D9::CreateDevice(D3DCREATE_SOFTWARE_VERTEXPROCESSING)\n")
			return false;
		}
	}

	//
	D3DCAPS9 caps;
	m_device->GetDeviceCaps(&caps);
	if (caps.PrimitiveMiscCaps & D3DPMISCCAPS_NULLREFERENCE)
		return false;

	//
	char description[90];	// string to hold D3D device stats

	// store device description
	if (deviceType == D3DDEVTYPE_HAL)
		lstrcpy(description, "HAL");
	else if (deviceType == D3DDEVTYPE_REF)
		lstrcpy(description, "REF");

	if (behavior & D3DCREATE_HARDWARE_VERTEXPROCESSING && behavior & D3DCREATE_PUREDEVICE)
	{
		if (deviceType == D3DDEVTYPE_HAL)
			lstrcat(description, " (pure hw vp)");
		else
			lstrcat(description, " (simulated pure hw vp)");
	}
	else if (behavior & D3DCREATE_HARDWARE_VERTEXPROCESSING)
	{
		if (deviceType == D3DDEVTYPE_HAL)
			lstrcat(description, " (hw vp)");
		else
			lstrcat(description, " (simulated hw vp)");
	}
	else if (behavior & D3DCREATE_MIXED_VERTEXPROCESSING)
	{
		if (deviceType == D3DDEVTYPE_HAL)
			lstrcat(description, " (mixed vp)");
		else
			lstrcat(description, " (simulated mixed vp)");
	}
	else if (behavior & D3DCREATE_SOFTWARE_VERTEXPROCESSING)
	{
		lstrcat(description, " (sw vp)");
	}

	//
	char frameStat[90];		// string to hold frame stats

	// store render target surface desc
	D3DSURFACE_DESC backDesc;
	LPDIRECT3DSURFACE9 backBuffer = NULL;
	m_device->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &backBuffer);
	backBuffer->GetDesc(&backDesc);
	backBuffer->Release();

	lstrcpy(frameStat, formatToString(backDesc.Format));

	if (m_present.EnableAutoDepthStencil)
	{
		char depthFmt[100];
		wsprintf(depthFmt, " (%s)", formatToString(m_present.AutoDepthStencilFormat));
		lstrcat(frameStat, depthFmt);
	}
	else
		lstrcat(frameStat, " ");

	LPTSTR multiSample;
	switch (m_present.MultiSampleType)
	{
	case D3DMULTISAMPLE_NONMASKABLE:multiSample = TEXT(" (Nonmaskable Multisample)"); break;
	case D3DMULTISAMPLE_3_SAMPLES:  multiSample = TEXT(" (3x Multisample)"); break;
	case D3DMULTISAMPLE_5_SAMPLES:  multiSample = TEXT(" (5x Multisample)"); break;
	case D3DMULTISAMPLE_6_SAMPLES:  multiSample = TEXT(" (6x Multisample)"); break;
	case D3DMULTISAMPLE_7_SAMPLES:  multiSample = TEXT(" (7x Multisample)"); break;
	case D3DMULTISAMPLE_8_SAMPLES:  multiSample = TEXT(" (8x Multisample)"); break;
	case D3DMULTISAMPLE_9_SAMPLES:  multiSample = TEXT(" (9x Multisample)"); break;
	case D3DMULTISAMPLE_10_SAMPLES: multiSample = TEXT(" (10x Multisample)"); break;
	case D3DMULTISAMPLE_11_SAMPLES: multiSample = TEXT(" (11x Multisample)"); break;
	case D3DMULTISAMPLE_12_SAMPLES: multiSample = TEXT(" (12x Multisample)"); break;
	case D3DMULTISAMPLE_13_SAMPLES: multiSample = TEXT(" (13x Multisample)"); break;
	case D3DMULTISAMPLE_14_SAMPLES: multiSample = TEXT(" (14x Multisample)"); break;
	case D3DMULTISAMPLE_15_SAMPLES: multiSample = TEXT(" (15x Multisample)"); break;
	case D3DMULTISAMPLE_16_SAMPLES: multiSample = TEXT(" (16x Multisample)"); break;
	case D3DMULTISAMPLE_2_SAMPLES:  multiSample = TEXT(" (2x Multisample)"); break;
	case D3DMULTISAMPLE_4_SAMPLES:  multiSample = TEXT(" (4x Multisample)"); break;
	default:                        multiSample = TEXT(""); break;
	}
	lstrcat(frameStat, multiSample);

	//
	TRACE(">> %s %s: ", description, frameStat)

	//
	D3DADAPTER_IDENTIFIER9 identifier;
	if (FAILED(m_d3d->GetAdapterIdentifier(0, 0, &identifier)))
		return false;

	// driver version
	TRACE("%s\n", identifier.Description)

	TRACE(">> driver version: %d.%d.%d.%d ",
		HIWORD(identifier.DriverVersion.HighPart),
		LOWORD(identifier.DriverVersion.HighPart),
		HIWORD(identifier.DriverVersion.LowPart),
		LOWORD(identifier.DriverVersion.LowPart))

	// shader version
	WORD ps = LOWORD(caps.PixelShaderVersion);
	WORD vs = LOWORD(caps.VertexShaderVersion);
	TRACE("(ps_%x_%x, vs_%x_%x)\n", HIBYTE(ps), LOBYTE(ps), HIBYTE(vs), LOBYTE(vs))

#if defined(_NVCG)
	// nVidia Cg shader
	cgD3D9EnableDebugTracing(true);
	cgD3D9SetDevice(m_device);
#endif //_NVCG

#endif //_WIN32

	return IGraphics::open(hwnd);
}

#if defined(_WIN32)
static void setWindowPos(HWND hwnd, int x, int y, int width, int height)
{
	// set the window's initial style
	//DWORD style = WS_POPUP;

	DWORD style = WS_POPUP;
	//DWORD style = WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE;
	//style= style & (~WS_THICKFRAME);

	// set the window's initial width
	RECT rect;
	::SetRect(&rect, 0, 0, width, height);
	::AdjustWindowRect(&rect, style, FALSE);

	width = (rect.right  - rect.left);
	height = (rect.bottom - rect.top);

	//style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	::SetWindowLong(hwnd, GWL_STYLE, style);
	::SetWindowLong(hwnd, GWL_EXSTYLE, WS_EX_TOPMOST);

	::SetWindowPos(hwnd,
		HWND_NOTOPMOST,
		x,
		y,
		width,
		height,
		SWP_SHOWWINDOW);
}

static bool confirmDevice(D3DCAPS9 * caps, DWORD behavior)
{
	// debugging vertex shaders requires either REF or software vertex processing
	// and debugging pixel shaders requires REF.
#ifdef DEBUG_VS
	if (caps->DeviceType != D3DDEVTYPE_REF &&
		(behavior & D3DCREATE_SOFTWARE_VERTEXPROCESSING) == 0)
		return false;
#endif

#ifdef DEBUG_PS
	if (caps->DeviceType != D3DDEVTYPE_REF)
		return false;
#endif

	if (behavior & D3DCREATE_PUREDEVICE)
		return false; // GetTransform doesn't work on PUREDEVICE

	//
	if (behavior & D3DCREATE_HARDWARE_VERTEXPROCESSING)
		return false;


	// no fallback, so need ps1.1
	if (caps->PixelShaderVersion < D3DPS_VERSION(1, 1))
		return false;

	// if device doesn't support 1.1 vertex shaders in HW, switch to SWVP.
	if (caps->VertexShaderVersion < D3DVS_VERSION(1, 1))
	{
		if ((behavior & D3DCREATE_SOFTWARE_VERTEXPROCESSING) == 0)
			return false;
	}

	return true;
}
static DWORD findBehaviorFlags(D3DCAPS9 * caps)
{
	DWORD behavior = D3DCREATE_SOFTWARE_VERTEXPROCESSING;

	// confirm the device/format for HW vertex processing
	bool confirmed = false;
	if (caps->DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT)
	{
		//
		if (caps->DevCaps & D3DDEVCAPS_PUREDEVICE)
		{
			behavior = D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DCREATE_PUREDEVICE;
			confirmed = confirmDevice(caps, behavior);
		}

		//
		if (confirmed == false)
		{
			behavior = D3DCREATE_HARDWARE_VERTEXPROCESSING;
			confirmed = confirmDevice(caps, behavior);
		}

		//
		if (confirmed == false)
		{
			behavior = D3DCREATE_MIXED_VERTEXPROCESSING;
			confirmed = confirmDevice(caps, behavior);
		}
	}

	// confirm the device/format for SW vertex processing
	if (confirmed == false)
		behavior = D3DCREATE_SOFTWARE_VERTEXPROCESSING;

	return behavior;
}

//
static D3DFORMAT findDepthStencilFormat(LPDIRECT3D9 d3d, int adapter, D3DDEVTYPE deviceType, D3DFORMAT format)
{
	D3DFORMAT __format[] =
	{
		D3DFMT_D24S8,
		D3DFMT_D24X4S4,
		D3DFMT_D24X8,
		D3DFMT_D15S1,
		D3DFMT_D32,
		D3DFMT_D16,
		D3DFMT_UNKNOWN,
	};

	int count = 0;
	while (__format[count] != D3DFMT_UNKNOWN)
	{
		if (SUCCEEDED(d3d->CheckDeviceFormat(adapter,
			deviceType,
			format,
			D3DUSAGE_DEPTHSTENCIL,
			D3DRTYPE_SURFACE,
			__format[count])))
		{
			if (SUCCEEDED(d3d->CheckDepthStencilMatch(adapter,
				deviceType,
				format,
				format,
				__format[count])))
			{
				return __format[count];
			}
		}

		count ++;
	}

	return D3DFMT_UNKNOWN;
}
#endif //_WIN32

} //hydra
