/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "direct3d9.h"

namespace hydra
{

direct3d9 * _direct3d9 = NULL;

direct3d9::direct3d9()
{
	//
	_direct3d9 = this;
	
	//
	m_width = 1280;
	m_height = 720;

#if defined(_XBOX) || defined(_WIN32)
	m_d3d = NULL;
	m_device = NULL;

	// Set up the structure used to create the D3DDevice. Since we are now
	// using more complex geometry, we will create a device with a zbuffer.
	ZeroMemory(&m_present, sizeof(D3DPRESENT_PARAMETERS));
#endif //_XBOX
}
direct3d9::~direct3d9()
{
#if defined(_WIN32) || defined(_XBOX)
	if (m_device)
	{
		m_device->Release();
		m_device = NULL;
	}
	if (m_d3d)
	{
		m_d3d->Release();
		m_d3d = NULL;
	}
#endif //_WIN32

	_direct3d9 = NULL;
}

//
bool direct3d9::beginScene()
{
#if defined(_WIN32) || defined(_XBOX)
	if (FAILED(m_device->Clear(0L,
		NULL,
		D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER | D3DCLEAR_STENCIL,
		D3DCOLOR_XRGB(64, 64, 64),
		1.0f,
		0L)))
		return false;

	if (FAILED(m_device->BeginScene()))
		return false;
#endif //_WIN32

	return true;
}
void direct3d9::endScene()
{
#if defined(_WIN32) || defined(_XBOX)
	m_device->EndScene();
#endif //_WIN32
}

//
void direct3d9::present()
{
#if defined(_WIN32) || defined(_XBOX)
	// Show the frame on the primary surface.
	HRESULT result = m_device->Present(NULL, NULL, NULL, NULL);

	if (result == D3DERR_DEVICELOST)
	{
		//GetDXUTState().SetDeviceLost( true );
	}
	else if (result == D3DERR_DRIVERINTERNALERROR)
	{
		// When D3DERR_DRIVERINTERNALERROR is returned from Present(),
		// the application can do one of the following:
		//
		// - End, with the pop-up window saying that the application cannot continue
		//   because of problems in the display adapter and that the user should
		//   contact the adapter manufacturer.
		//
		// - Attempt to restart by calling IDirect3DDevice9::Reset, which is essentially the same
		//   path as recovering from a lost device. If IDirect3DDevice9::Reset fails with
		//   D3DERR_DRIVERINTERNALERROR, the application should end immediately with the message
		//   that the user should contact the adapter manufacturer.
		//
		// The framework attempts the path of resetting the device
		//
		//GetDXUTState().SetDeviceLost( true );
	}
#endif //_WIN32
}

//
void direct3d9::setViewport(int left, int top, int right, int bottom)
{
	if (right == 0 || bottom == 0)
		return;

#if 0//defined(_WIN32) || defined(_XBOX)
	D3DVIEWPORT9 viewport;
	viewport.X = left;
	viewport.Y = top;
	viewport.Width = right;
	viewport.Height = bottom;
	viewport.MinZ = 0.0f;
	viewport.MaxZ = 1.0f;
	m_device->SetViewport(&viewport);
#endif //_WIN32

	IGraphics::setViewport(left, top, right, bottom);
}

#if defined(_WIN32) || defined(_XBOX)
static _D3DSHADEMODE __SHADEMODE[] =
{
	D3DSHADE_FLAT,
	D3DSHADE_GOURAUD,
	D3DSHADE_PHONG,
};
static _D3DFILLMODE __FILLMODE[] =
{
	D3DFILL_POINT,
	D3DFILL_WIREFRAME,
	D3DFILL_SOLID,
};
static _D3DBLEND __BLEND[] =
{
	D3DBLEND_ZERO,
	D3DBLEND_ONE,
	D3DBLEND_SRCCOLOR,
	D3DBLEND_INVSRCCOLOR,
	D3DBLEND_SRCALPHA,
	D3DBLEND_INVSRCALPHA,
	D3DBLEND_DESTALPHA,
	D3DBLEND_INVDESTALPHA,
	D3DBLEND_DESTCOLOR,
	D3DBLEND_INVDESTCOLOR,
	D3DBLEND_SRCALPHASAT,
};
static _D3DCULL __CULL[] =
{
	D3DCULL_NONE,
	D3DCULL_CW,
	D3DCULL_CCW,
};
static _D3DCMPFUNC __CMPFUNC[] =
{
	D3DCMP_NEVER,
	D3DCMP_LESS,
	D3DCMP_EQUAL,
	D3DCMP_LESSEQUAL,
	D3DCMP_GREATER,
	D3DCMP_NOTEQUAL,
	D3DCMP_GREATEREQUAL,
	D3DCMP_ALWAYS,
};

static _D3DRENDERSTATETYPE __RENDERSTATETYPE[] =
{
	D3DRS_ZENABLE,
	D3DRS_STENCILENABLE,
	D3DRS_ALPHATESTENABLE,
	D3DRS_ALPHABLENDENABLE,
	D3DRS_SRCBLEND,
	D3DRS_DESTBLEND,
	D3DRS_CULLMODE,
	D3DRS_SHADEMODE,
	D3DRS_FILLMODE,
	D3DRS_ZFUNC,
	D3DRS_ALPHAFUNC,
	D3DRS_ALPHAREF,
};
#endif //_WIN32

void direct3d9::setRenderState(int state, unsigned long value)
{
#if defined(_WIN32) || defined(_XBOX)
	switch (state)
	{
	case RS_ZENABLE:
	case RS_STENCILENABLE:
	case RS_ALPHABLENDENABLE:
	case RS_ALPHATESTENABLE:
	case RS_ALPHAREF:
		m_device->SetRenderState(__RENDERSTATETYPE[state], value);
		break;

	case RS_CULLMODE:
		m_device->SetRenderState(__RENDERSTATETYPE[state], __CULL[value]);
		break;

	case RS_SRCBLEND:
	case RS_DESTBLEND:
		m_device->SetRenderState(__RENDERSTATETYPE[state], __BLEND[value]);
		break;

	case RS_SHADEMODE:
		m_device->SetRenderState(__RENDERSTATETYPE[state], __SHADEMODE[value]);
		break;
	case RS_FILLMODE:
		m_device->SetRenderState(__RENDERSTATETYPE[state], __FILLMODE[value]);
		break;

	case RS_ZFUNC:
	case RS_ALPHAFUNC:
		m_device->SetRenderState(__RENDERSTATETYPE[state], __CMPFUNC[value]);
		break;
	}
#endif //_WIN32
}

//
void direct3d9::setRenderTarget(unsigned long index, ISurface * surface)
{

}

//
void direct3d9::initialize()
{
#if defined(_WIN32) || defined(_XBOX)
	m_device->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	m_device->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);

	m_device->SetRenderState(D3DRS_ZENABLE, TRUE);
	m_device->SetRenderState(D3DRS_ZFUNC, D3DCMP_LESSEQUAL);
	m_device->SetRenderState(D3DRS_STENCILENABLE, TRUE);

	for (int n=0; n<MAX_SAMPLER_STAGE; n++)
		m_device->SetTexture(n, NULL);

		/*
		m_device->SetTextureStageState(n, D3DTSS_TEXCOORDINDEX, 0);
		m_device->SetTextureStageState(n, D3DTSS_COLOROP, D3DTOP_MODULATE);
		m_device->SetTextureStageState(n, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		m_device->SetTextureStageState(n, D3DTSS_COLORARG2, D3DTA_CURRENT);
		*/
#endif //_WIN32
}

#if defined(_WIN32) || defined(_XBOX)
// Returns the string for the given D3DFORMAT.
LPTSTR direct3d9::formatToString(D3DFORMAT format)
{
	LPTSTR msg = NULL;
	switch (format)
	{
	case D3DFMT_UNKNOWN:         msg = TEXT("UNKNOWN"); break;
	case D3DFMT_A8R8G8B8:        msg = TEXT("A8R8G8B8"); break;
	case D3DFMT_X8R8G8B8:        msg = TEXT("X8R8G8B8"); break;
	case D3DFMT_R5G6B5:          msg = TEXT("R5G6B5"); break;
	case D3DFMT_X1R5G5B5:        msg = TEXT("X1R5G5B5"); break;
	case D3DFMT_A1R5G5B5:        msg = TEXT("A1R5G5B5"); break;
	case D3DFMT_A4R4G4B4:        msg = TEXT("A4R4G4B4"); break;
	case D3DFMT_A8:              msg = TEXT("A8"); break;
	case D3DFMT_X4R4G4B4:        msg = TEXT("X4R4G4B4"); break;
	case D3DFMT_A2B10G10R10:     msg = TEXT("A2B10G10R10"); break;
	case D3DFMT_A8B8G8R8:        msg = TEXT("A8B8G8R8"); break;
	case D3DFMT_X8B8G8R8:        msg = TEXT("X8B8G8R8"); break;
	case D3DFMT_G16R16:          msg = TEXT("G16R16"); break;
	case D3DFMT_A2R10G10B10:     msg = TEXT("A2R10G10B10"); break;
	case D3DFMT_A16B16G16R16:    msg = TEXT("A16B16G16R16"); break;
	case D3DFMT_L8:              msg = TEXT("L8"); break;
	case D3DFMT_A8L8:            msg = TEXT("A8L8"); break;
	case D3DFMT_V8U8:            msg = TEXT("V8U8"); break;
	case D3DFMT_L6V5U5:          msg = TEXT("L6V5U5"); break;
	case D3DFMT_X8L8V8U8:        msg = TEXT("X8L8V8U8"); break;
	case D3DFMT_Q8W8V8U8:        msg = TEXT("Q8W8V8U8"); break;
	case D3DFMT_V16U16:          msg = TEXT("V16U16"); break;
	case D3DFMT_A2W10V10U10:     msg = TEXT("A2W10V10U10"); break;
	case D3DFMT_UYVY:            msg = TEXT("UYVY"); break;
	case D3DFMT_YUY2:            msg = TEXT("YUY2"); break;
	case D3DFMT_DXT1:            msg = TEXT("DXT1"); break;
	case D3DFMT_DXT3:            msg = TEXT("DXT3"); break;
	case D3DFMT_DXT5:            msg = TEXT("DXT5"); break;
	case D3DFMT_D32:             msg = TEXT("D32"); break;
	case D3DFMT_D24S8:           msg = TEXT("D24S8"); break;
	case D3DFMT_D24X8:           msg = TEXT("D24X8"); break;
	case D3DFMT_D16:             msg = TEXT("D16"); break;
	case D3DFMT_L16:             msg = TEXT("L16"); break;
	case D3DFMT_VERTEXDATA:      msg = TEXT("VERTEXDATA"); break;
	case D3DFMT_INDEX16:         msg = TEXT("INDEX16"); break;
	case D3DFMT_INDEX32:         msg = TEXT("INDEX32"); break;
	case D3DFMT_Q16W16V16U16:    msg = TEXT("Q16W16V16U16"); break;
	case D3DFMT_R16F:            msg = TEXT("R16F"); break;
	case D3DFMT_G16R16F:         msg = TEXT("G16R16F"); break;
	case D3DFMT_A16B16G16R16F:   msg = TEXT("A16B16G16R16F"); break;
	case D3DFMT_R32F:            msg = TEXT("R32F"); break;
	case D3DFMT_G32R32F:         msg = TEXT("G32R32F"); break;
	case D3DFMT_A32B32G32R32F:   msg = TEXT("A32B32G32R32F"); break;
#if defined(_WIN32)
	case D3DFMT_R8G8B8:          msg = TEXT("R8G8B8"); break;
	case D3DFMT_R3G3B2:          msg = TEXT("R3G3B2"); break;
	case D3DFMT_A8R3G3B2:        msg = TEXT("A8R3G3B2"); break;
	case D3DFMT_A8P8:            msg = TEXT("A8P8"); break;
	case D3DFMT_P8:              msg = TEXT("P8"); break;
	case D3DFMT_A4L4:            msg = TEXT("A4L4"); break;
	case D3DFMT_DXT2:            msg = TEXT("DXT2"); break;
	case D3DFMT_DXT4:            msg = TEXT("DXT4"); break;
	case D3DFMT_D16_LOCKABLE:    msg = TEXT("D16_LOCKABLE"); break;
	case D3DFMT_D15S1:           msg = TEXT("D15S1"); break;
	case D3DFMT_MULTI2_ARGB8:    msg = TEXT("MULTI2_ARGB8"); break;
	case D3DFMT_CxV8U8:          msg = TEXT("CxV8U8"); break;
#endif

	default:                     msg = TEXT("Unknown format"); break;
	}

	return msg;
}
#endif //_WIN32

} //hydra

