/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#ifndef DIRECT3D9_H_
#define DIRECT3D9_H_

#include "graphics/igraphics.h"

#if defined(_WIN32) || defined(_XBOX)
// Enable extra D3D debugging in debug builds if using the debug DirectX runtime.
// This makes D3D objects work well in the debugger watch window, but slows down
// performance slightly.
#if defined(DEBUG) || defined(_DEBUG)
	#ifndef D3D_DEBUG_INFO
	#define D3D_DEBUG_INFO
	#endif //D3D_DEBUG_INFO
#endif //DEBUG

#ifndef D3D_OVERLOADS
#define D3D_OVERLOADS
#endif //D3D_OVERLOADS

#if defined(__BORLANDC__)
#define sqrtf(x) (float) sqrt(x)
#endif //__BORLANDC__

#include <d3d9.h>
#include <d3dx9.h>

#if defined (_MSC_VER)
#pragma warning(disable:4100)
#endif //_MSC_VER

#if defined (_MSC_VER)
#pragma comment(lib, "d3d9.lib")
#pragma comment(lib, "d3dx9.lib")
#endif //_MSC_VER

#if defined(_WIN32)
	#if defined (_MSC_VER)
	#pragma comment(lib, "dxguid.lib")
	#pragma comment(lib, "dxerr.lib")
	#endif //_MSC_VER
#endif //_WIN32

#if defined(_NVCG)
	#include <cg/cgD3D9.h>

	#if defined (_MSC_VER)
	#pragma comment(lib, "cgD3D9.lib")
	#endif //_MSC_VER
#endif //_NVCG
#endif //_WIN32

namespace hydra
{
	
class direct3d9: public IGraphics 
{
public:
	direct3d9();
	virtual ~direct3d9();

public:

	//
    bool beginScene();
    void endScene();

    void present();

    //
    void setRenderState(int state, unsigned long value);

    void setViewport(int left, int top, int right, int bottom);

	//
	void setRenderTarget(unsigned long index, ISurface * surface);

#if defined(_WIN32) || defined(_XBOX)
public:
    inline LPDIRECT3DDEVICE9 getDevice() { return m_device; }
#endif //_WIN32

protected:
    void initialize();

#if defined(_WIN32) || defined(_XBOX)
protected:
    LPTSTR formatToString(D3DFORMAT format);
#endif //_WIN32

#if defined(_WIN32) || defined(_XBOX)
protected:
	LPDIRECT3D9 m_d3d;
	LPDIRECT3DDEVICE9 m_device;

	D3DPRESENT_PARAMETERS m_present;
#endif //_WIN32
};

extern direct3d9 * _direct3d9;

} //hydra

#endif //DIRECT3D9_H_
