/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

// #undef _USRDLL

#if defined(_WIN32)
#include "direct3d9-win32.h"
#elif defined(_XBOX)
#include "direct3d9-xenon.h"
#endif

#include "direct3d9-indexbuffer.h"
#include "direct3d9-vertexbuffer.h"
#include "direct3d9-sampler.h"
#include "direct3d9-shader.h"
#include "direct3d9-cgshader.h"


#include "kernel.h"

#if defined (_MSC_VER)
	#pragma comment(lib, "hydra.lib")
#endif //_MSC_VER

namespace hydra
{
static MODULEINFO moduleInfo = {
	"direct3d9",
	"Direct3D9 hardware-shader base graphics module.",
	"GRAPHICS",
	"(c) 2004-2008 Enside, Inc. All rights reserved.",
	"hunee (hunee@enside.net)",
	"0.0.1",
};
LIBEXPORT const MODULEINFO * getModuleInfo()
{
	return &moduleInfo;
}

//
static IGraphics * _graphics = NULL;
LIBEXPORT void * loadModule()
{
#if defined(_WIN32)
	_graphics = new direct3d9Win32;
#elif defined(_XBOX)
	_graphics = new direct3d9Xenon;
#endif

	if (_graphics == NULL)
		return NULL;

		//
	registerKernelModule(direct3d9Indexbuffer);
	registerKernelModule(direct3d9Vertexbuffer);

	registerKernelModule(direct3d9Sampler);
	registerKernelModule(direct3d9Shader);

#if defined(_NVCG)
	registerKernelModule(direct3d9CgShader);
#endif //_NVCG

	return _graphics;
}
LIBEXPORT void unloadModule()
{
	//
	unregisterKernelModule(direct3d9Indexbuffer);
	unregisterKernelModule(direct3d9Vertexbuffer);

	unregisterKernelModule(direct3d9Sampler);
	unregisterKernelModule(direct3d9Shader);

#if defined(_NVCG)
	unregisterKernelModule(direct3d9CgShader);
#endif //_NVCG

	SAFE_DELETE(_graphics);
}

} //hydra

