/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

// #undef _USRDLL

#include "opengl-win32.h"
#include "opengl-darwin.h"
#include "opengl-linux.h"

//#include "opengl-ps3.h"

#include "opengl-indexbuffer.h"
#include "opengl-vertexbuffer.h"
#include "opengl-sampler.h"
#include "opengl-shader.h"
#include "opengl-cgshader.h"

#include "kernel.h"

#if defined (_MSC_VER)
	#pragma comment(lib, "hydra.lib")
#endif //_MSC_VER

namespace hydra
{
static MODULEINFO moduleInfo = {
	"opengl",
	"OpenGL hardware-shader base graphics module.",
	"GRAPHICS",
	"(c) 2004-2008 Enside, Inc. All rights reserved.",
	"hunee (hunee@enside.net)",
	"0.0.1",
};
LIBEXPORT const MODULEINFO * getModuleInfo()
{
	return &moduleInfo;
}

//
static IGraphics * _graphics = NULL;

LIBEXPORT void * loadModule()
{
#if defined(_WIN32)
	_graphics = new openglWin32;
#elif defined(__APPLE__)
	_graphics = new openglDarwin;
#elif defined(_LINUX)
	_graphics = new openglLinux;
#elif defined(_PS3)
	_graphics = new openglPS3;
#endif
	if (_graphics == NULL)
		return NULL;

	//
	registerKernelModule(openglIndexbuffer);
	registerKernelModule(openglVertexbuffer);

	registerKernelModule(openglSampler);
	registerKernelModule(openglShader);

#if defined(_NVCG)
	registerKernelModule(openglCgShader);
#endif //_NVCG

	return _graphics;
}
LIBEXPORT void unloadModule()
{
	//
	unregisterKernelModule(openglIndexbuffer);
	unregisterKernelModule(openglVertexbuffer);

	unregisterKernelModule(openglSampler);
	unregisterKernelModule(openglShader);

#if defined(_NVCG)
	unregisterKernelModule(openglCgShader);
#endif //_NVCG

	SAFE_DELETE(_graphics);
}

} //hydra

