/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "opengl-cgshader.h"

#include "opengl-sampler.h"

#if defined(_NVCG)

namespace hydra
{

IMPLEMENT_MODULE(openglCgShader, usr/lib/graphics/shader.cg)

openglCgShader::openglCgShader()
{

}

//
bool openglCgShader::loadFromMemory(const char * code)
{
	CGcontext cgContext = _opengl->getCgContext();

	CGprofile profile = cgGLGetLatestProfile(CG_GL_VERTEX);
	cgGLSetOptimalOptions(profile);

	m_vertexProgram = cgCreateProgram(cgContext,
		CG_SOURCE,
		code,
		profile,
		"vs_main",
		NULL);
	if (!m_vertexProgram)
	{
		const char * listing = cgGetLastListing(cgContext);
		if (!listing)
			listing = "No listing available!";

		TRACE("# openglCgShader::loadFromMemory()\n")
		TRACE0(">> ERROR: cgCreateProgram(vs_main) : %s\n", listing)

		return false;
	}

	profile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
	cgGLSetOptimalOptions(profile);

	m_fragmentProgram = cgCreateProgram(cgContext,
		CG_SOURCE,
		code,
		profile,
		"ps_main",
		NULL);
	if (!m_fragmentProgram)
	{
		const char * listing = cgGetLastListing(cgContext);
		if (!listing)
			listing = "No listing available!";

		TRACE("# openglCgShader::loadFromMemory()\n")
		TRACE0(">> ERROR: cgCreateProgram(ps_main) : %s\n", listing)

		return false;
	}

	return true;
}
bool openglCgShader::loadFromMemory(const char * vpcode, const char * fpcode)
{
	return true;
}

//
void openglCgShader::setSampler(int stage, ISampler * sampler)
{
	glActiveTexture(GL_TEXTURE0 + stage);
	glBindTexture(GL_TEXTURE_2D, ((openglSampler *) sampler)->getHandle());

	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}

static int __TEXTUREFILTERTYPE[] =
{
	GL_NEAREST,
	GL_NEAREST,
	GL_LINEAR,
	GL_LINEAR,
	GL_NEAREST,
	GL_NEAREST_MIPMAP_NEAREST,
	GL_NEAREST_MIPMAP_LINEAR,
	GL_NEAREST_MIPMAP_LINEAR,
	GL_LINEAR,
	GL_LINEAR_MIPMAP_NEAREST,
	GL_LINEAR_MIPMAP_LINEAR,
	GL_LINEAR_MIPMAP_LINEAR,
};
static int __TEXTUREADDRESS[] =
{
	GL_REPEAT,
	GL_CLAMP,
};
static int __SAMPLERSTATETYPE[] =
{
	GL_TEXTURE_WRAP_S,
	GL_TEXTURE_WRAP_T,
	GL_TEXTURE_MAG_FILTER,
	GL_TEXTURE_MIN_FILTER,
};
void openglCgShader::setSamplerState(unsigned long sampler, int type, unsigned long value)
{
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	switch (type)
	{
	case SAMP_ADDRESSU:
	case SAMP_ADDRESSV:
		glTexParameteri(GL_TEXTURE_2D, __SAMPLERSTATETYPE[type], __TEXTUREADDRESS[value]);
		break;

	case SAMP_MAGFILTER:
	case SAMP_MINFILTER:
		glTexParameteri(GL_TEXTURE_2D, __SAMPLERSTATETYPE[type], __TEXTUREFILTERTYPE[value]);
		break;

	case SAMP_MIPFILTER:
		{
		GLint filter;
		glGetTexParameteriv(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, &filter);
		if (filter == GL_NEAREST)
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, __TEXTUREFILTERTYPE[value + 4]);
		else
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, __TEXTUREFILTERTYPE[value + 8]);

		//
		glGetTexParameteriv(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, &filter);
		if (filter == GL_NEAREST)
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, __TEXTUREFILTERTYPE[value + 4]);
		else
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, __TEXTUREFILTERTYPE[value + 8]);
		}
		break;
	}
}

//
void openglCgShader::begin()
{
	cgGLLoadProgram(m_vertexProgram);
	cgGLLoadProgram(m_fragmentProgram);

	cgGLBindProgram(m_vertexProgram);
	cgGLBindProgram(m_fragmentProgram);

	cgGLEnableProfile(cgGLGetLatestProfile(CG_GL_VERTEX));
	cgGLEnableProfile(cgGLGetLatestProfile(CG_GL_FRAGMENT));
}
void openglCgShader::end()
{
	cgGLDisableProfile(cgGLGetLatestProfile(CG_GL_VERTEX));
	cgGLDisableProfile(cgGLGetLatestProfile(CG_GL_FRAGMENT));
}

} //hydra

#endif //_NVCG
