/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "opengl-darwin.h"

#if defined(__APPLE__)

AGLPbuffer m_pbuffer; // the equivalent to the drawable
GLuint m_texName;

namespace hydra
{

// aglChoosePixelFormat has been deprecated on Mac OS X v10.5 use aglCreatePixelFormat
// as shown below.
#if (AGL_VERSION_3_0)
	#define CREATEPIXELFORMAT(attrib) aglCreatePixelFormat(attrib)
	#define SETDRAWABLE(context, draw) aglSetWindowRef(context, draw)
#else
	#define CREATEPIXELFORMAT(attrib) aglChoosePixelFormat(NULL, 0, attrib)
	#define SETDRAWABLE(context, draw) aglSetDrawable(context, GetWindowPort(draw))
#endif

//
openglDarwin::openglDarwin()
{
	m_context = NULL;
}

openglDarwin::~openglDarwin()
{
	if (m_context)
	{
		SETDRAWABLE(m_context, NULL);

/*
		glDeleteTextures(1, &m_texName);

		aglDestroyPBuffer(m_pbuffer);
		m_pbuffer = NULL;
*/			
		aglSetCurrentContext(NULL);
		aglDestroyContext(m_context);

		m_context = NULL;
	}
}

bool openglDarwin::open(HWND hwnd)
{
	// check for existance of OpenGL	
	if ((Ptr) kUnresolvedCFragSymbolAddress == (Ptr) aglChoosePixelFormat)
		return false;

	// get display ID to use for a mask
	// the main display as configured via System Preferences
	CGDirectDisplayID displayID = CGMainDisplayID();
	CGOpenGLDisplayMask openGLDisplayMask = CGDisplayIDToOpenGLDisplayMask(displayID);

	// Solely as an example of possible use, this pixel format limits
	// the possible renderers to those supported by the screen mask.
	// In this case the main display.
	GLint attrib[] = {
		AGL_RGBA,
		AGL_DOUBLEBUFFER,
		AGL_RED_SIZE, 8,
		AGL_ALPHA_SIZE, 8, 
		AGL_DEPTH_SIZE, 32,
		AGL_ACCELERATED,
		AGL_CLOSEST_POLICY,
		AGL_NO_RECOVERY,

#if (AGL_VERSION_3_0)
		AGL_DISPLAY_MASK,
		openGLDisplayMask, // New to Mac OS X v10.5
#endif
		//AGL_SAMPLE_BUFFERS_ARB, 1,
		//AGL_SAMPLES_ARB, m_samples,
		AGL_NONE
	};

	AGLPixelFormat fmt = CREATEPIXELFORMAT(attrib); // New to Mac OS X v10.5
	if (fmt == NULL)
		return false;
	
	// create an AGL context
	m_context = aglCreateContext(fmt, NULL);							
	if (m_context == NULL)
	{
		if (fmt)
			aglDestroyPixelFormat(fmt);

		// remove multi-sample attribs
#if (AGL_VERSION_3_0)
		attrib[13] = attrib[14] = attrib[15] = attrib[16] = 0;
#else
		attrib[11] = attrib[12] = attrib[13] = attrib[14] = 0;
#endif

		fmt = CREATEPIXELFORMAT(attrib); // New to Mac OS X v10.5
		if (fmt == NULL)
			return false;

		//
		m_context = aglCreateContext(fmt, NULL);							
		if (m_context == NULL)
			return false;
	}

	// pixel format is no longer needed
	aglDestroyPixelFormat(fmt);	


	//GrafPtr portSave = NULL;
    //GetPort(&portSave);
    //SetPort((GrafPtr) GetWindowPort(hwnd));

/*
	//
	aglCreatePBuffer(m_width, m_height, GL_TEXTURE_2D, GL_RGBA, 0, &m_pbuffer);

	// update and draw pbuffer content for current target context (this ensures we are actually drawing on the same renderer)
	GLint vs = aglGetVirtualScreen(m_context);
	aglSetPBuffer(m_context, m_pbuffer, 0, 0, vs);

	glGenTextures(1, &m_texName);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
*/
	
	SETDRAWABLE(m_context, hwnd);

	// make the context the current context
	aglSetCurrentContext(m_context);

	// VBL SYNC
	//GLint swap = 1;
	//aglSetInteger(m_context, AGL_SWAP_INTERVAL, &swap);


	//
	SizeWindow(hwnd, m_width, m_height, true);

	//Rect rect;
	//GetWindowBounds(hwnd, kWindowContentRgn, &rect);
	//InvalWindowRect(hwnd, &rect);
	
#if defined(AGL_VERSION_3_0)
	TRACE(">> AGL_VERSION_3_0\n")
#endif

	return opengl::open(hwnd);
}

void openglDarwin::present()
{
	//glBindTexture(GL_TEXTURE_2D, m_texName);
	//aglTexImagePBuffer(m_context, m_pbuffer, GL_FRONT);

	aglSwapBuffers(m_context); 
}

void openglDarwin::setViewport(int left, int top, int right, int bottom)
{
	aglUpdateContext(m_context);
	
	//
	opengl::setViewport(left, top, right, bottom);
}

} //hydra

#endif //__APPLE__
