/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "opengl-indexbuffer.h"

namespace hydra
{
IMPLEMENT_MODULE(openglIndexbuffer, usr/lib/graphics/indexbuffer)

openglIndexbuffer::openglIndexbuffer()
{
	//
	m_buffer = NULL;
	m_objectId = GL_INVALID_HANDLE_VALUE;
}

openglIndexbuffer::~openglIndexbuffer()
{
	SAFE_FREE(m_buffer);

	if (m_objectId != GL_INVALID_HANDLE_VALUE)
	{
		glDeleteBuffers(1, &m_objectId);
		m_objectId = GL_INVALID_HANDLE_VALUE;
	}
}

bool openglIndexbuffer::create(int length, unsigned long format)
{
	if (!IIndexbuffer::create(length, format))
		return false;

	//
	unsigned int stride;
	switch (format)
	{
	case FMT_INDEX16:
		stride = sizeof(unsigned short);
		break;
	case FMT_INDEX32:
		stride = sizeof(unsigned int);
		break;
	}

	//
	m_buffer = MALLOC(length, stride);
	if (!m_buffer)
		return false;

	glGenBuffers(1, &m_objectId);

	return true;
}

void * openglIndexbuffer::map()
{
	return m_buffer;
}
void openglIndexbuffer::unmap()
{
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, m_objectId);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, m_length * sizeof(unsigned short), m_buffer, GL_STATIC_DRAW);

	GLint paramArrayObjectSize = 0;
	glGetBufferParameteriv(GL_ELEMENT_ARRAY_BUFFER, GL_BUFFER_SIZE, &paramArrayObjectSize);
	if (paramArrayObjectSize <= 0)
	{
	}
}

//
void openglIndexbuffer::bind()
{
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, m_objectId);
}

// This macro helps us to pass an integer value via the "pointer" argument of
// functions like glTexCoordPointer, glNormalPointer, and glVertexPointer.
// Normally, we would use the "pointer" argument to pass the vertex data in,
// but when we use the glBindBuffer function of ARB_vertex_buffer_object,
// this argument is used as an offset instead.
#define BUFFER_OFFSET(i) ((char *) NULL + (i))

static GLenum __PRIMITIVETYPE[] = {
	GL_POINTS,
	GL_LINES,
	GL_LINE_STRIP,
	GL_TRIANGLES,
	GL_TRIANGLE_STRIP,
	GL_TRIANGLE_FAN,
	GL_QUADS,
};

void openglIndexbuffer::draw(int primitiveType)
{
	glDrawElements(__PRIMITIVETYPE[primitiveType],
		m_length,
		GL_UNSIGNED_SHORT,
		BUFFER_OFFSET(0));
}

void openglIndexbuffer::draw(int primitiveType, unsigned int startIndex, unsigned int indexCount, unsigned int numVertices)
{
	glDrawElements(__PRIMITIVETYPE[primitiveType],
		indexCount,
		GL_UNSIGNED_SHORT,
		BUFFER_OFFSET(startIndex * sizeof(unsigned short)));
}
void openglIndexbuffer::draw(int primitiveType, unsigned int startVertex, unsigned int numVertices, unsigned int startIndex, unsigned int indexCount)
{
	glDrawElements(__PRIMITIVETYPE[primitiveType],
		numVertices,
		GL_UNSIGNED_SHORT,
		BUFFER_OFFSET(0));
}

} //hydra
