/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "opengl-linux.h"

#include "kernel.h"

#if defined(_LINUX)

namespace hydra
{

openglLinux::openglLinux()
{
	m_context = NULL;
}
openglLinux::~openglLinux()
{
	if (m_context)
	{
		if (!glXMakeCurrent(m_hwnd, None, NULL))
		{
			TRACE0(">> ERROR: releasing drawing context : killGLWindow\n");
		}

		glXDestroyContext(m_hwnd, m_context);
		m_context = NULL;
	}

	//
	if (m_fullscreen)
	{
		XF86VidModeSwitchToMode(m_hwnd, m_screen, &m_deskMode);
		XF86VidModeSetViewPort(m_hwnd, m_screen, 0, 0);
	}
}

//
bool openglLinux::open(HWND hwnd)
{
	m_hwnd = hwnd;
	
	m_screen = DefaultScreen(m_hwnd);

	int modeNum;
	XF86VidModeModeInfo **modes;
	XF86VidModeGetAllModeLines(m_hwnd, m_screen, &modeNum, &modes);

	//
	int bestMode = 0;
	m_deskMode = *modes[0];

	for (int i = 0; i < modeNum; i++)
	{
		if ((modes[i]->hdisplay == m_width) && (modes[i]->vdisplay == m_height))
		{
			bestMode = i;
		}
	}

	int attrListSgl[] = {
		GLX_RGBA, GLX_RED_SIZE, 4,
		GLX_GREEN_SIZE, 4,
		GLX_BLUE_SIZE, 4,
		GLX_DEPTH_SIZE, 16,
		None
	};

	int attrListDbl[] = {
		GLX_RGBA, GLX_DOUBLEBUFFER,
		GLX_RED_SIZE, 4,
		GLX_GREEN_SIZE, 4,
		GLX_BLUE_SIZE, 4,
		GLX_DEPTH_SIZE, 16,
		None
	};

	XVisualInfo * vi = glXChooseVisual(m_hwnd, m_screen, attrListDbl);
	if (vi == NULL)
	{
		vi = glXChooseVisual(m_hwnd, m_screen, attrListSgl);
		TRACE(">> single buffered : true\n");
	}
	else
	{
		TRACE(">> double buffered : true\n");
	}

	Colormap cmap = XCreateColormap(m_hwnd, RootWindow(m_hwnd, vi->screen),vi->visual, AllocNone);

	XSetWindowAttributes attr;
	attr.colormap = cmap;
	attr.border_pixel = 0;

	if (m_fullscreen)
	{
		XF86VidModeSwitchToMode(m_hwnd, m_screen, modes[bestMode]);
		XF86VidModeSetViewPort(m_hwnd, m_screen, 0, 0);

		int width = modes[bestMode]->hdisplay;
		int height = modes[bestMode]->vdisplay;
		XFree(modes);

		attr.override_redirect = True;
		attr.event_mask = ExposureMask | KeyPressMask | ButtonPressMask | StructureNotifyMask;
		m_window = XCreateWindow(m_hwnd,
			RootWindow(m_hwnd, vi->screen),
			0, 0, width, height,
			0, vi->depth,
			InputOutput,
			vi->visual,
			CWBorderPixel | CWColormap | CWEventMask | CWOverrideRedirect,
			&attr);

		//XWarpPointer(m_hwnd, m_window, 0, 0, 0, 0, 0, 0);
		XMapRaised(m_hwnd, m_window);
		XGrabKeyboard(m_hwnd, m_window, True, GrabModeAsync,GrabModeAsync, CurrentTime);
		XGrabPointer(m_hwnd, m_window, True, ButtonPressMask, GrabModeAsync, GrabModeAsync, m_window, None, CurrentTime);
	}
	else
	{
		attr.event_mask = ExposureMask | KeyPressMask | ButtonPressMask | StructureNotifyMask;
		m_window = XCreateWindow(m_hwnd,
			RootWindow(m_hwnd, vi->screen),
			0, 0, m_width, m_height,
			0,
			vi->depth,
			InputOutput,
			vi->visual,
			CWBorderPixel | CWColormap | CWEventMask, &attr);

		Atom wmDelete = XInternAtom(hwnd, "WM_DELETE_WINDOW", True);
		XSetWMProtocols(m_hwnd, m_window, &wmDelete, 1);

		char title[MAX_PATH];
		sprintf(title, "%s for %s", PLATFORMNAME, PLATFORM);

		XSetStandardProperties(m_hwnd,
			m_window,
			title,
			PLATFORMNAME,
			None,
			NULL,
			0,
			NULL);
		XMapRaised(m_hwnd, m_window);
	}

	int x, y;
	unsigned width, height;
	unsigned int bpp;

	Window winDummy;
	unsigned int borderDummy;

	XGetGeometry(hwnd,
		m_window,
		&winDummy,
		&x, &y, &width, &height,
		&borderDummy,
		&bpp);

    //
	m_context = glXCreateContext(m_hwnd, vi, 0, GL_TRUE);
	glXMakeCurrent(m_hwnd, m_window, m_context);

	TRACE(">> direct rendering: %s\n", glXIsDirect(m_hwnd, m_context) ? "true" : "false");

	XMapWindow(m_hwnd, m_window);

	//
	return opengl::initialize(hwnd);
}

void openglLinux::present()
{
	glXSwapBuffers(m_hwnd, m_window);
}

} //hydra

#endif //_LINUX
