/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "opengl-sampler.h"

namespace hydra
{
IMPLEMENT_MODULE(openglSampler, usr/lib/graphics/sampler)

openglSampler::openglSampler()
{
	//
	m_bits = NULL;
	m_texture = GL_INVALID_HANDLE_VALUE;
	
	//
	//m_target = GL_ARB_texture_rectangle_supported ? GL_TEXTURE_RECTANGLE_ARB : GL_TEXTURE_2D;
	m_target = GL_TEXTURE_2D;
}
openglSampler::~openglSampler()
{
	SAFE_FREE(m_bits);

	if (m_texture != GL_INVALID_HANDLE_VALUE)
	{
		glDeleteTextures(1, &m_texture);
		m_texture = GL_INVALID_HANDLE_VALUE;
	}
}

//
bool openglSampler::create(int width, int height, unsigned long format, int mipLevels)
{
	if (!ISampler::create(width, height, format, mipLevels))
		return false;

	//
	m_bits = MALLOC(m_linearSize, sizeof(char));
	if (!m_bits)
		return false;

	glGenTextures(1, &m_texture);
	glBindTexture(m_target, m_texture);

	return true;
}

//
void * openglSampler::map()
{
	return m_bits;
}

static GLenum __FORMAT[] = {
	GL_RGB,

	GL_RGB,
	GL_RGBA,

	GL_RGBA,
	GL_RGBA,
	GL_RGBA,

	GL_ALPHA,
	GL_LUMINANCE,
	GL_LUMINANCE_ALPHA,

	GL_COMPRESSED_RGB_S3TC_DXT1_EXT,
	GL_COMPRESSED_RGBA_S3TC_DXT3_EXT,
	GL_COMPRESSED_RGBA_S3TC_DXT5_EXT,
};
static GLint __COMPONENTS[] = {
	0,
	3, 4,
	4, 4, 4,
	1, 1, 2,
	3, 4, 4,
};

void openglSampler::unmap()
{
	// Upload Texture
	glBindTexture(m_target, m_texture);
	//glTexParameteri(m_target, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

#if defined(__APPLE__)
	//glTexParameteri(m_target, GL_TEXTURE_STORAGE_HINT_APPLE, GL_STORAGE_CACHED_APPLE);
		
	//glPixelStorei(GL_UNPACK_CLIENT_STORAGE_APPLE, GL_TRUE);
#endif

	GLenum format = __FORMAT[m_format];
	GLint components = __COMPONENTS[m_format];

	int levels = m_mipLevels ? m_mipLevels : 1;

	switch (format)
	{
	case GL_ALPHA:
	case GL_LUMINANCE:
	case GL_LUMINANCE_ALPHA:
	case GL_RGB:
	case GL_RGBA:
		glTexImage2D(m_target,
			0,
			components,
			m_width,
			m_height,
			0,
			format,
			GL_UNSIGNED_BYTE,
			m_bits);
		break;

		///////////////////////////////////////////////////////////////////////////
	case GL_COMPRESSED_RGB_S3TC_DXT1_EXT:
	case GL_COMPRESSED_RGBA_S3TC_DXT3_EXT:
	case GL_COMPRESSED_RGBA_S3TC_DXT5_EXT:
		{
			int width = m_width;
			int height = m_height;
			char * bits = (char *) m_bits;

			for (int n=0; n<levels; n++)
			{
				int linearSize = (((width + 3) >> 2) * ((height + 3) >> 2)) << components;
				glCompressedTexImage2D(m_target,
					n,
					format,
					width,
					height,
					0,
					linearSize,
					bits);

				bits += linearSize;
				width = max(width >> 1, 1);
				height = max(height >> 1, 1);
				
				//TRACE(">> linerSize = %d\n", linearSize)
			}
		}
		break;
	}
}

} //hydra
