/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "opengl-shader.h"

#include "opengl-sampler.h"

namespace hydra
{
IMPLEMENT_MODULE(openglShader, usr/lib/graphics/shader)

openglShader::openglShader()
{
	//
	m_ext = ".gl";

	m_programObject = 0;
}

openglShader::~openglShader()
{
	if (m_programObject)
	{
		glDeleteObjectARB(m_programObject);
		m_programObject = 0;
	}
}

bool openglShader::loadFromMemory(const char * code)
{
	//TRACE("# openglShader::loadFromMemory()\n")

	// find shader tags
	char * vertexShaderString = (char *) strstr(code, "[Vertex shader]");
	char * fragmentShaderString = (char *) strstr(code, "[Fragment shader]");

	const GLcharARB * shaderStrings[2];

	// A couple of defines to make life easier for HLSL users
	shaderStrings[0] =
		"#define saturate(x) clamp(x,0.0,1.0)\n"
		"#define lerp mix\n";

    m_programObject = glCreateProgramObjectARB();
    if (!m_programObject)
        return false;

    // Select the wanted string pieces and compute line numbers at which each shader begins
    int vsLine = 0;
    if (vertexShaderString != NULL)
    {
        *vertexShaderString = '\0';
        vertexShaderString += 15;
        while (*vertexShaderString == '\n' || *vertexShaderString == '\r')
            vertexShaderString ++;

        char * str = (char *) code;
        while (str < vertexShaderString)
        {
            if (*str == '\n')
                vsLine ++;

            str ++;
        }
    }

    int fsLine = 0;
    if (fragmentShaderString != NULL)
    {
        *fragmentShaderString = '\0';
        fragmentShaderString += 17;
        while (*fragmentShaderString == '\n' || *fragmentShaderString == '\r')
            fragmentShaderString ++;

        char * str = (char *) code;
        while (str < fragmentShaderString)
        {
            if (*str == '\n')
                fsLine ++;

            str ++;
        }
    }

    // vertex shader
    if (vertexShaderString != NULL)
    {
        shaderStrings[1] = vertexShaderString;

        //
        GLhandleARB object = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
        glShaderSourceARB(object, 2, shaderStrings, NULL);
        glCompileShaderARB(object);

        GLint compiled = 0;
        glGetObjectParameterivARB(object, GL_OBJECT_COMPILE_STATUS_ARB, &compiled);
        if (!compiled)
        {
            GLsizei length;
            char infoLog[2048];
            glGetInfoLogARB(object, sizeof(infoLog), &length, infoLog);
            glDeleteObjectARB(object);

            TRACE0(">> ERROR: vertexShader: %s\n", infoLog);

            return false;
        }

        glAttachObjectARB(m_programObject, object);
    }

    // fragment shader
    if (fragmentShaderString != NULL)
    {
        shaderStrings[1] = fragmentShaderString;

        //
        GLhandleARB object = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
        glShaderSourceARB(object, 2, shaderStrings, NULL);
        glCompileShaderARB(object);

        // check if shader compiled
        GLint compiled = 0;
        glGetObjectParameterivARB(object, GL_OBJECT_COMPILE_STATUS_ARB, &compiled);
        if (!compiled)
        {
            GLsizei length;
            char infoLog[2048];
            glGetInfoLogARB(object, sizeof(infoLog), &length, infoLog);
            glDeleteObjectARB(object);

            TRACE0(">> ERROR: fragmentShader: %s\n", infoLog);

            return false;
        }

        glAttachObjectARB(m_programObject, object);
    }

	// make sure position attrib is bound to location 0
	//glBindAttribLocationARB(m_programObject, 0, "position");

	// link
	glLinkProgramARB(m_programObject);

	GLint linked;
	glGetObjectParameterivARB(m_programObject, GL_OBJECT_LINK_STATUS_ARB, &linked);
	if (!linked)
	{
		outputDebugString(">> ERROR!! Shaders failed to link, exiting...\n");
		return false;
	}

	// validate
	glValidateProgramARB(m_programObject);

	GLint validated;
	glGetObjectParameterivARB(m_programObject, GL_OBJECT_VALIDATE_STATUS_ARB, &validated);
	if (!validated)
	{
		outputDebugString(">> ERROR!! Shaders failed to validate, exiting...\n");
		return false;
	}


	// bind sampler
	GLint _MAX_TEXTURE_UNITS_ARB;
	glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &_MAX_TEXTURE_UNITS_ARB);

	//
	glUseProgramObjectARB(m_programObject);

	char TEX[MAX_PATH];
	for (int n=0; n<_MAX_TEXTURE_UNITS_ARB; n++)
	{
		sprintf(TEX, "TEX%d", n);

		GLint location = glGetUniformLocationARB(m_programObject, TEX);
		glUniform1iARB(location, n);
	}

	glUseProgramObjectARB(0);

	return true;
}
bool openglShader::loadFromMemory(const char * vpcode, const char * fpcode)
{
	return true;
}

void openglShader::begin()
{
	glUseProgramObjectARB(m_programObject);

	/*
	// load to current matrix
	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf((float *) &gfx->getProjection());

	// load to current matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf((float *) &gfx->getView());
	*/
}
void openglShader::end()
{
	glUseProgramObjectARB(0);
}

//
void openglShader::setSampler(int stage, ISampler * sampler)
{
	glActiveTexture(GL_TEXTURE0 + stage);
	glBindTexture(GL_TEXTURE_2D, ((openglSampler *) sampler)->getHandle());

	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}

static int __TEXTUREFILTERTYPE[] =
{
	GL_NEAREST,
	GL_NEAREST,
	GL_LINEAR,
	GL_LINEAR,
	GL_NEAREST,
	GL_NEAREST_MIPMAP_NEAREST,
	GL_NEAREST_MIPMAP_LINEAR,
	GL_NEAREST_MIPMAP_LINEAR,
	GL_LINEAR,
	GL_LINEAR_MIPMAP_NEAREST,
	GL_LINEAR_MIPMAP_LINEAR,
	GL_LINEAR_MIPMAP_LINEAR,
};
static int __TEXTUREADDRESS[] =
{
	GL_REPEAT,
	GL_CLAMP,
};
static int __SAMPLERSTATETYPE[] =
{
	GL_TEXTURE_WRAP_S,
	GL_TEXTURE_WRAP_T,
	GL_TEXTURE_MAG_FILTER,
	GL_TEXTURE_MIN_FILTER,
};
void openglShader::setSamplerState(unsigned long sampler, int type, unsigned long value)
{
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	switch (type)
	{
	case SAMP_ADDRESSU:
	case SAMP_ADDRESSV:
		glTexParameteri(GL_TEXTURE_2D, __SAMPLERSTATETYPE[type], __TEXTUREADDRESS[value]);
		break;

	case SAMP_MAGFILTER:
	case SAMP_MINFILTER:
		glTexParameteri(GL_TEXTURE_2D, __SAMPLERSTATETYPE[type], __TEXTUREFILTERTYPE[value]);
		break;

	case SAMP_MIPFILTER:
		{
		GLint filter;
		glGetTexParameteriv(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, &filter);
		if (filter == GL_NEAREST)
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, __TEXTUREFILTERTYPE[value + 4]);
		else
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, __TEXTUREFILTERTYPE[value + 8]);

		//
		glGetTexParameteriv(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, &filter);
		if (filter == GL_NEAREST)
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, __TEXTUREFILTERTYPE[value + 4]);
		else
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, __TEXTUREFILTERTYPE[value + 8]);
		}
		break;
	}
}

//
void openglShader::setFloat(const char * name, const float value)
{
	GLint location = glGetUniformLocationARB(m_programObject, name);
	glUniform1fARB(location, value);
}
void openglShader::setFloatArray(const char * name, const float * value, int count)
{

}
void openglShader::setVector(const char * name, const float4 & value)
{
	GLint location = glGetUniformLocationARB(m_programObject, name);
	glUniform4fvARB(location, 1, (GLfloat *) &value);
}
void openglShader::setVectorArray(const char * name, const float4 * value, int count)
{

}
void openglShader::setMatrix(const char * name, const float4x4 & value)
{
	GLint location = glGetUniformLocationARB(m_programObject, name);
	glUniformMatrix4fvARB(location, 1, 0, (GLfloat *) &value);
}
void openglShader::setMatrixArray(const char * name, const float4x4 * value, int count)
{

}

} //hydra
