/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "opengl-vertexbuffer.h"

namespace hydra
{
IMPLEMENT_MODULE(openglVertexbuffer, usr/lib/graphics/vertexbuffer)

openglVertexbuffer::openglVertexbuffer()
{
	//
	m_buffer = NULL;
	m_objectId = GL_INVALID_HANDLE_VALUE;
}

openglVertexbuffer::~openglVertexbuffer()
{
	SAFE_FREE(m_buffer);

	if (m_objectId != GL_INVALID_HANDLE_VALUE)
	{
		glDeleteBuffers(1, &m_objectId);
		m_objectId = GL_INVALID_HANDLE_VALUE;
	}
}

bool openglVertexbuffer::create(int length, unsigned long fvf)
{
	if (!IVertexbuffer::create(length, fvf))
		return false;

	//
	m_buffer = MALLOC(length, m_stride);
	if (!m_buffer)
		return false;

	glGenBuffers(1, &m_objectId);

	return true;
}

//
void * openglVertexbuffer::map()
{
	return m_buffer;
}
void openglVertexbuffer::unmap()
{
	glBindBuffer(GL_ARRAY_BUFFER, m_objectId);
	glBufferData(GL_ARRAY_BUFFER, m_length * m_stride, m_buffer, GL_STATIC_DRAW);

	GLint paramArrayObjectSize = 0;
	glGetBufferParameteriv(GL_ARRAY_BUFFER, GL_BUFFER_SIZE, &paramArrayObjectSize);
	if (paramArrayObjectSize <= 0)
	{
	}
}

// This macro helps us to pass an integer value via the "pointer" argument of
// functions like glTexCoordPointer, glNormalPointer, and glVertexPointer.
// Normally, we would use the "pointer" argument to pass the vertex data in,
// but when we use the glBindBuffer function of ARB_vertex_buffer_object,
// this argument is used as an offset instead.
#define BUFFER_OFFSET(i) ((char *) NULL + (i))

void openglVertexbuffer::bind()
{
	glBindBuffer(GL_ARRAY_BUFFER, m_objectId);

	//glDisableClientState(GL_COLOR_ARRAY);
	//glDisableClientState(GL_NORMAL_ARRAY);
	//glDisableClientState(GL_TEXTURE_COORD_ARRAY);

	//
	int offset = 0;

	unsigned long position = m_fvf & FVF_POSITION_MASK;
	if (position == FVF_XYZ)
	{
		glEnableClientState(GL_VERTEX_ARRAY);
		glVertexPointer(3, GL_FLOAT, m_stride, BUFFER_OFFSET(offset));

		offset += sizeof(float) * 3;
	}
	else if (position == FVF_XYZRHW)
	{
		glEnableClientState(GL_VERTEX_ARRAY);
		glVertexPointer(4, GL_FLOAT, m_stride, BUFFER_OFFSET(offset));

		offset += sizeof(float) * 4;
	}
	else if (position >= FVF_XYZB1)
	{
		int weights = (position >> 1) - 2;

		glEnableClientState(GL_VERTEX_ARRAY);
		glVertexPointer(3, GL_FLOAT, m_stride, BUFFER_OFFSET(offset));

		offset += sizeof(float) * (3 + weights);
	}

	//
	if (m_fvf & FVF_NORMAL)
	{
		glEnableClientState(GL_NORMAL_ARRAY);
		glNormalPointer(GL_FLOAT, m_stride, BUFFER_OFFSET(offset));

		offset += sizeof(float) * 3;
	}
	if (m_fvf & FVF_PSIZE)
	{
		offset += sizeof(float);
	}
	if (m_fvf & FVF_DIFFUSE)
	{
		glEnableClientState(GL_COLOR_ARRAY);
		glColorPointer(4, GL_UNSIGNED_BYTE, m_stride, BUFFER_OFFSET(offset));

		offset += sizeof(unsigned long);
	}
	if (m_fvf & FVF_SPECULAR)
	{
		offset += sizeof(float);
	}

	//
	if (m_fvf & FVF_TEXCOUNT_MASK)
	{
		// only one tu/tv set currently supported
		int texCoords = ((m_fvf & FVF_TEXCOUNT_MASK) >> FVF_TEXCOUNT_SHIFT);
		if (texCoords > 8)
			texCoords = 8;

		for (int n=0; n<texCoords; n++)
			glClientActiveTexture(GL_TEXTURE0 + n);

		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, m_stride, BUFFER_OFFSET(offset));
	}
}

static GLenum __PRIMITIVETYPE[] = {
	GL_POINTS,
	GL_LINES,
	GL_LINE_STRIP,
	GL_TRIANGLES,
	GL_TRIANGLE_STRIP,
	GL_TRIANGLE_FAN,
	GL_QUADS,
};

void openglVertexbuffer::draw(int primitiveType)
{
	bind();

	glDrawArrays(__PRIMITIVETYPE[primitiveType], 0, m_length);
}
void openglVertexbuffer::draw(int primitiveType, unsigned int startVertex, unsigned int vertexCount)
{
	glDrawArrays(__PRIMITIVETYPE[primitiveType], startVertex, vertexCount);
}

} //hydra
