/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "opengl-win32.h"

#if defined(_WIN32)

namespace hydra
{

static void setWindowPos(HWND hwnd, int x, int y, int width, int height);

// fullscreen
static bool setDisplayMode(int dwWidth, int dwHeight, int bitsPerPel);
static void restoreDisplayMode();

openglWin32::openglWin32()
{
	m_context = NULL;
	m_hdc = NULL;
}
openglWin32::~openglWin32()
{
	//TRACE("# openglWin32::~openglWin32()\n")

	if (m_fullscreen)
		restoreDisplayMode();

	if (m_context)
	{
		wglDeleteContext(m_context);
		m_context = NULL;
	}
	wglMakeCurrent(NULL, NULL);

	if (m_hdc)
	{
		ReleaseDC(m_hwnd, m_hdc);
		m_hdc = NULL;
	}
}

//
bool openglWin32::open(HWND hwnd)
{
	m_hwnd = hwnd;

	setWindowPos(m_hwnd, 0, 0, m_width, m_height);

	//
	PIXELFORMATDESCRIPTOR pfd;
	memset(&pfd, 0, sizeof(PIXELFORMATDESCRIPTOR));

	pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);
	pfd.nVersion = 1;
	pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
	pfd.iPixelType = PFD_TYPE_RGBA;
	pfd.dwLayerMask = PFD_MAIN_PLANE;
	pfd.cColorBits = 0;
	pfd.cDepthBits = 32;
	pfd.cStencilBits = 8;

	//
	m_hdc = GetDC(m_hwnd);
	int pixelFormat = ChoosePixelFormat(m_hdc, &pfd);
	if (pixelFormat < 0)
		return false;

	// what did we get I wonder
	memset(&pfd, 0, sizeof(PIXELFORMATDESCRIPTOR));
	pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);

	if (!DescribePixelFormat(m_hdc, pixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd))
		return false;

	// check hardware opengl
	BOOL genericFormat = (pfd.dwFlags & PFD_GENERIC_FORMAT) != 0;
	BOOL genericAccel  = (pfd.dwFlags & PFD_GENERIC_ACCELERATED) != 0;

	m_accel = false;
	m_miniport = false;

	// Determine hardware driver status by the combination of these
	// two flags.  Both FALSE means ICD, both TRUE means MCD.
	if (genericFormat == genericAccel)
	{
		m_accel = true;

		if (genericFormat)
			m_miniport = true;
	}

	// Set the format
	if (!SetPixelFormat(m_hdc, pixelFormat, &pfd))
		return false;

	// Link the Win Device Context with the OGL Rendering Context
	m_context = wglCreateContext(m_hdc);
	if (!m_context)
		return false;

	// Specify the target DeviceContext (window) of the subsequent OGL calls
	wglMakeCurrent(m_hdc, m_context);

	//
	if (m_fullscreen)
		setDisplayMode(m_width, m_height, m_bpp);

	return opengl::open(m_hwnd);
}

//
void openglWin32::present()
{
	SwapBuffers(m_hdc);
}

static void setWindowPos(HWND hwnd, int x, int y, int width, int height)
{
	// Set the window's initial style
	//DWORD style = WS_POPUP;

	DWORD style = WS_POPUP;
	//DWORD style = WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE;
	//style= style & (~WS_THICKFRAME);

	// Set the window's initial width
	RECT rect;
	::SetRect(&rect, 0, 0, width, height);
	::AdjustWindowRect(&rect, style, FALSE);

	width = (rect.right  - rect.left);
	height = (rect.bottom - rect.top);

	//style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	::SetWindowLong(hwnd, GWL_STYLE, style);
	::SetWindowLong(hwnd, GWL_EXSTYLE, WS_EX_TOPMOST);

	::SetWindowPos(hwnd,
		HWND_NOTOPMOST,
		x,
		y,
		width,
		height,
		SWP_SHOWWINDOW);
}

// fullscreen
static bool setDisplayMode(int width, int height, int bitsPerPel)
{
	HDC hDC = GetDC(NULL);
	bitsPerPel = GetDeviceCaps(hDC, BITSPIXEL);
	ReleaseDC(NULL, hDC);

	DEVMODE dm;

	dm.dmPelsWidth = width;
	dm.dmPelsHeight = height;
	dm.dmBitsPerPel = bitsPerPel;
	dm.dmDisplayFrequency = 0;
	dm.dmFields = DM_BITSPERPEL |
		DM_PELSWIDTH |
		DM_PELSHEIGHT |
		DM_DISPLAYFREQUENCY;

	//if (ChangeDisplaySettings(&dm, CDS_FULLSCREEN) == DISP_CHANGE_SUCCESSFUL)
	if (ChangeDisplaySettings(&dm, NULL) == DISP_CHANGE_SUCCESSFUL)
		return true;

	return false;
}
static void restoreDisplayMode()
{
	ChangeDisplaySettings(NULL, 0);
}

} //hydra

#endif //_WIN32
