/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "opengl.h"

//
#include "kernel.h"

namespace hydra
{

opengl * _opengl = NULL;

opengl::opengl()
{
    //
    _opengl = this;

	//
	m_width = 1280;
	m_height = 720;
	
	m_modeFSAA = FSAANice;
}
opengl::~opengl()
{
	//TRACE("# opengl::~opengl()\n")

#if defined(_NVCG)
	//reenable default state
	cgGLDisableProfile(cgGLGetLatestProfile(CG_GL_VERTEX));
	cgGLDisableProfile(cgGLGetLatestProfile(CG_GL_FRAGMENT));
#endif //_NVCG

	//
	_opengl = NULL;
}

//
bool opengl::open(HWND hwnd)
{
	char * renderer = (char *) glGetString(GL_RENDERER);
	char * version = (char *) glGetString(GL_VERSION);
	char * slversion = (char *) glGetString(GL_SHADING_LANGUAGE_VERSION);
	char * vendor = (char *) glGetString(GL_VENDOR);
	
	TRACE("# GL_INFO\n");
	
	TRACE(">> GL_RENDERER: %s\n", vendor)
	TRACE(">> GL_VERSION: %s\n", version)
	TRACE(">> GL_SHADING_LANGUAGE_VERSION: %s\n", slversion);
	TRACE(">> GL_VENDOR: %s\n", renderer)

	//
    checkGLExtensions();

#if defined(_NVCG)
	cgGLSetDebugMode(CG_FALSE);
#endif //_NVCG

	//
	return IGraphics::open(hwnd);
}

//
static int __CULL[] =
{
	GL_CULL_FACE,
	GL_CW,
	GL_CCW,
};

static int __BLEND[] =
{
	GL_ZERO,
	GL_ONE,
	GL_SRC_COLOR,
	GL_ONE_MINUS_SRC_COLOR,
	GL_SRC_ALPHA,
	GL_ONE_MINUS_SRC_ALPHA,
	GL_DST_ALPHA,
	GL_ONE_MINUS_DST_ALPHA,
	GL_DST_COLOR,
	GL_ONE_MINUS_DST_COLOR,
	GL_SRC_ALPHA_SATURATE,
};
static int __SHADEMODE[] =
{
	GL_FLAT,
	GL_SMOOTH,
	GL_SMOOTH,
};
static int __FILLMODE[] =
{
	GL_POINT,
	GL_LINE,
	GL_FILL,
};
static int __CMPFUNC[] =
{
	GL_NEVER,
	GL_LESS,
	GL_EQUAL,
	GL_LEQUAL,
	GL_GREATER,
	GL_NOTEQUAL,
	GL_GEQUAL,
	GL_ALWAYS,
};

static int __RENDERSTATETYPE[] =
{
	GL_DEPTH_TEST,
	GL_STENCIL_TEST,
	GL_ALPHA_TEST,
	GL_BLEND,
};

void opengl::setRenderState(int state, unsigned long value)
{
	switch (state)
	{
	case RS_ZENABLE:
	case RS_STENCILENABLE:
	case RS_ALPHABLENDENABLE:
	case RS_ALPHATESTENABLE:
		switch (value)
		{
		case TRUE:
			glEnable(__RENDERSTATETYPE[state]);
			break;
		default:
			glDisable(__RENDERSTATETYPE[state]);
		}
		break;

	case RS_CULLMODE:
		switch (value)
		{
		case CULL_NONE:
			glDisable(GL_CULL_FACE);
			break;

		default:
			glEnable(GL_CULL_FACE);

			glFrontFace(__CULL[value]);
			glCullFace(GL_FRONT);
			break;
		}
		break;

	case RS_SRCBLEND:
		{
			GLint blend;
			glGetIntegerv(GL_BLEND_DST, &blend);
			glBlendFunc(__BLEND[value], blend);
		}
		break;

	case RS_DESTBLEND:
		{
			GLint blend;
			glGetIntegerv(GL_BLEND_SRC, &blend);
			glBlendFunc(blend, __BLEND[value]);
		}
		break;

	case RS_SHADEMODE:
		glShadeModel(__SHADEMODE[value]);
		break;
	case RS_FILLMODE:
		glPolygonMode(GL_FRONT, __FILLMODE[value]);
		break;

	case RS_ZFUNC:
		glDepthFunc(__CMPFUNC[value]);
		break;
	case RS_ALPHAFUNC:
		{
			GLfloat ref;
			glGetFloatv(GL_ALPHA_TEST_REF, &ref);

			glAlphaFunc(__CMPFUNC[value], ref);
		}
		break;

	case RS_ALPHAREF:
		{
			GLint func;
			glGetIntegerv(GL_ALPHA_TEST_FUNC, &func);

			glAlphaFunc(func, (GLfloat) value / 255.0f);
		}
		break;
	}
}


//
bool opengl::beginScene()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	return true;
}
void opengl::endScene()
{
	//glFlush();
}

//
void opengl::setViewport(int left, int top, int right, int bottom)
{
	if (right == 0 || bottom == 0)
		return;

	GLint viewportDims[4];
	glGetIntegerv(GL_VIEWPORT, viewportDims);
	
	if ((viewportDims[2] != m_width) || (viewportDims[3] != m_height))
	{
		//glViewport(left, top, right, bottom);
		//glScissor(left, top, right, bottom);
		
		glViewport(0, 0, right, bottom);
	}

	IGraphics::setViewport(left, top, right, bottom);
}

//
void opengl::setRenderTarget(unsigned long index, ISurface * surface)
{

}

//
void opengl::initialize()
{
	glShadeModel(GL_SMOOTH);

	glClearColor(0.2f, 0.2f, 0.4f, 1.0f);

#if defined(_PS3)
	glClearDepthf(1.0f);
#else
	glClearDepth(1.0f);
#endif //_PS3

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glEnable(GL_TEXTURE_2D);

	//
	switch (m_modeFSAA)
	{
	case FSAAOff:
		glDisable(GL_MULTISAMPLE_ARB);
		break;
	case FSAAFast:
		glEnable(GL_MULTISAMPLE_ARB);
		glHint(GL_MULTISAMPLE_FILTER_HINT_NV, GL_FASTEST);
		break;
	case FSAANice:
		glEnable(GL_MULTISAMPLE_ARB);
		glHint(GL_MULTISAMPLE_FILTER_HINT_NV, GL_NICEST);
		break;
	}
}

#if !defined(GL_GLEXT_PROTOTYPES) 
// gl extensions

// GL_VERSION_1_3
PFNGLACTIVETEXTUREPROC glActiveTexture = NULL;
PFNGLCLIENTACTIVETEXTUREPROC glClientActiveTexture = NULL;
PFNGLCOMPRESSEDTEXIMAGE2DPROC glCompressedTexImage2D = NULL;

// GL_VERSION_1_5
PFNGLBINDBUFFERPROC glBindBuffer = NULL;
PFNGLDELETEBUFFERSPROC glDeleteBuffers = NULL;
PFNGLGENBUFFERSPROC glGenBuffers = NULL;
PFNGLBUFFERDATAPROC glBufferData = NULL;

PFNGLGETBUFFERPARAMETERIVPROC glGetBufferParameteriv = NULL;

// GL_ARB_shader_objects
PFNGLCREATEPROGRAMOBJECTARBPROC glCreateProgramObjectARB = NULL;
PFNGLDELETEOBJECTARBPROC glDeleteObjectARB = NULL;
PFNGLCREATESHADEROBJECTARBPROC glCreateShaderObjectARB = NULL;
PFNGLSHADERSOURCEARBPROC glShaderSourceARB = NULL;
PFNGLCOMPILESHADERARBPROC glCompileShaderARB = NULL;
PFNGLATTACHOBJECTARBPROC glAttachObjectARB = NULL;
PFNGLGETINFOLOGARBPROC glGetInfoLogARB = NULL;
PFNGLLINKPROGRAMARBPROC glLinkProgramARB = NULL;
PFNGLUSEPROGRAMOBJECTARBPROC glUseProgramObjectARB = NULL;
PFNGLGETUNIFORMLOCATIONARBPROC glGetUniformLocationARB = NULL;
PFNGLVALIDATEPROGRAMARBPROC glValidateProgramARB = NULL;

PFNGLUNIFORM1FARBPROC glUniform1fARB = NULL;
PFNGLUNIFORM1FVARBPROC glUniform1fvARB = NULL;
PFNGLUNIFORM1IARBPROC glUniform1iARB = NULL;
PFNGLUNIFORM1IVARBPROC glUniform1ivARB = NULL;
PFNGLUNIFORM2FARBPROC glUniform2fARB = NULL;
PFNGLUNIFORM2FVARBPROC glUniform2fvARB = NULL;
PFNGLUNIFORM2IARBPROC glUniform2iARB = NULL;
PFNGLUNIFORM2IVARBPROC glUniform2ivARB = NULL;
PFNGLUNIFORM3FARBPROC glUniform3fARB = NULL;
PFNGLUNIFORM3FVARBPROC glUniform3fvARB = NULL;
PFNGLUNIFORM3IARBPROC glUniform3iARB = NULL;
PFNGLUNIFORM3IVARBPROC glUniform3ivARB = NULL;
PFNGLUNIFORM4FARBPROC glUniform4fARB = NULL;
PFNGLUNIFORM4FVARBPROC glUniform4fvARB = NULL;
PFNGLUNIFORM4IARBPROC glUniform4iARB = NULL;
PFNGLUNIFORM4IVARBPROC glUniform4ivARB = NULL;
PFNGLUNIFORMMATRIX2FVARBPROC glUniformMatrix2fvARB = NULL;
PFNGLUNIFORMMATRIX3FVARBPROC glUniformMatrix3fvARB = NULL;
PFNGLUNIFORMMATRIX4FVARBPROC glUniformMatrix4fvARB = NULL;

PFNGLGETOBJECTPARAMETERFVARBPROC glGetObjectParameterfvARB = NULL;
PFNGLGETOBJECTPARAMETERIVARBPROC glGetObjectParameterivARB = NULL;


// GL_ARB_vertex_shader
PFNGLBINDATTRIBLOCATIONARBPROC glBindAttribLocationARB;

#if defined(_WIN32)
#define GETPROCADDRESS(procname)    wglGetProcAddress(procname)
#elif defined(_LINUX)
#define GETPROCADDRESS(procname)    glXGetProcAddress((const GLubyte *) procname)
#endif

#endif //GL_GLEXT_PROTOTYPES

//
GLboolean GL_ARB_texture_rectangle_supported = FALSE;
GLboolean GL_EXT_paletted_texture_supported = FALSE;

void opengl::checkGLExtensions()
{
	//TRACE("# opengl::checkGLExtensions()")

#if !defined(GL_GLEXT_PROTOTYPES)
	// GL_VERSION_1_3
	glActiveTexture			= (PFNGLACTIVETEXTUREPROC) GETPROCADDRESS("glActiveTexture");
	glClientActiveTexture	= (PFNGLCLIENTACTIVETEXTUREPROC) GETPROCADDRESS("glClientActiveTexture");
	glCompressedTexImage2D	= (PFNGLCOMPRESSEDTEXIMAGE2DPROC) GETPROCADDRESS("glCompressedTexImage2D");

	// GL_VERSION_1_5
    glBindBuffer            = (PFNGLBINDBUFFERPROC) GETPROCADDRESS("glBindBuffer");
    glDeleteBuffers         = (PFNGLDELETEBUFFERSPROC) GETPROCADDRESS("glDeleteBuffers");
    glGenBuffers            = (PFNGLGENBUFFERSPROC) GETPROCADDRESS("glGenBuffers");
    glBufferData            = (PFNGLBUFFERDATAPROC) GETPROCADDRESS("glBufferData");
    glGetBufferParameteriv  = (PFNGLGETBUFFERPARAMETERIVPROC) GETPROCADDRESS("glGetBufferParameteriv");
#endif

	TRACE("# GL_EXTENSIONS\n");
	
	// check for existance in the list of extension names, and if it's there
    // look up all of the functions and set the flag stating multitexturing
    // is availible.
	const char * extensions = (char *) glGetString(GL_EXTENSIONS);
	if (strstr(extensions, "GL_ARB_multisample"))
	{
		TRACE(">> GL_ARB_multisample\n");
	}

    if (strstr(extensions, "GL_ARB_texture_rectangle") || strstr(extensions, "GL_EXT_texture_rectangle"))
	{
		TRACE(">> GL_ARB_texture_rectangle\n");
		GL_ARB_texture_rectangle_supported = TRUE;
    }
    if (strstr(extensions, "GL_EXT_paletted_texture"))
	{
		TRACE(">> GL_EXT_paletted_texture\n");
        GL_EXT_paletted_texture_supported = TRUE;
        //glColorTableEXT = ("glColorTableEXT");
	}
			
	if (strstr(extensions, "GL_ARB_shader_objects") &&
        strstr(extensions, "GL_ARB_shading_language_100") &&
        strstr(extensions, "GL_ARB_vertex_shader") &&
        strstr(extensions, "GL_ARB_fragment_shader"))
	{
		TRACE(">> GL_ARB_shader_objects\n")
		TRACE(">> GL_ARB_shading_language_100\n");
		TRACE(">> GL_ARB_vertex_shader\n");
		TRACE(">> GL_ARB_fragment_shader\n");

#if !defined(GL_GLEXT_PROTOTYPES)
		glCreateProgramObjectARB	= (PFNGLCREATEPROGRAMOBJECTARBPROC) GETPROCADDRESS("glCreateProgramObjectARB");
		glDeleteObjectARB		    = (PFNGLDELETEOBJECTARBPROC) GETPROCADDRESS("glDeleteObjectARB");
		glCreateShaderObjectARB	    = (PFNGLCREATESHADEROBJECTARBPROC) GETPROCADDRESS("glCreateShaderObjectARB");
		glShaderSourceARB		    = (PFNGLSHADERSOURCEARBPROC) GETPROCADDRESS("glShaderSourceARB");
		glCompileShaderARB		    = (PFNGLCOMPILESHADERARBPROC) GETPROCADDRESS("glCompileShaderARB");
		glGetObjectParameterfvARB	= (PFNGLGETOBJECTPARAMETERFVARBPROC) GETPROCADDRESS("glGetObjectParameterfvARB");
		glGetObjectParameterivARB	= (PFNGLGETOBJECTPARAMETERIVARBPROC) GETPROCADDRESS("glGetObjectParameterivARB");
		glAttachObjectARB	        = (PFNGLATTACHOBJECTARBPROC) GETPROCADDRESS("glAttachObjectARB");
		glGetInfoLogARB		        = (PFNGLGETINFOLOGARBPROC) GETPROCADDRESS("glGetInfoLogARB");
		glLinkProgramARB		    = (PFNGLLINKPROGRAMARBPROC) GETPROCADDRESS("glLinkProgramARB");
		glUseProgramObjectARB		= (PFNGLUSEPROGRAMOBJECTARBPROC) GETPROCADDRESS("glUseProgramObjectARB");
		glGetUniformLocationARB		= (PFNGLGETUNIFORMLOCATIONARBPROC) GETPROCADDRESS("glGetUniformLocationARB");
		glValidateProgramARB		= (PFNGLVALIDATEPROGRAMARBPROC) GETPROCADDRESS("glValidateProgramARB");

		glUniform1fARB	            = (PFNGLUNIFORM1FARBPROC) GETPROCADDRESS("glUniform1fARB");
		glUniform1fvARB		        = (PFNGLUNIFORM1FVARBPROC) GETPROCADDRESS("glUniform1fvARB");
		glUniform1iARB		        = (PFNGLUNIFORM1IARBPROC) GETPROCADDRESS("glUniform1iARB");
		glUniform1ivARB	            = (PFNGLUNIFORM1IVARBPROC) GETPROCADDRESS("glUniform1ivARB");

		glUniform2fARB				= (PFNGLUNIFORM2FARBPROC) GETPROCADDRESS("glUniform2fARB");
		glUniform2fvARB	            = (PFNGLUNIFORM2FVARBPROC) GETPROCADDRESS("glUniform2fvARB");
		glUniform2iARB		        = (PFNGLUNIFORM2IARBPROC) GETPROCADDRESS("glUniform2iARB");
		glUniform2ivARB		        = (PFNGLUNIFORM2IVARBPROC) GETPROCADDRESS("glUniform2ivARB");

		glUniform3fARB	            = (PFNGLUNIFORM3FARBPROC) GETPROCADDRESS("glUniform3fARB");
		glUniform3fvARB				= (PFNGLUNIFORM3FVARBPROC) GETPROCADDRESS("glUniform3fvARB");
		glUniform3iARB	            = (PFNGLUNIFORM3IARBPROC) GETPROCADDRESS("glUniform3iARB");
		glUniform3ivARB		        = (PFNGLUNIFORM3IVARBPROC) GETPROCADDRESS("glUniform3ivARB");

		glUniform4fARB		        = (PFNGLUNIFORM4FARBPROC) GETPROCADDRESS("glUniform4fARB");
		glUniform4fvARB	            = (PFNGLUNIFORM4FVARBPROC) GETPROCADDRESS("glUniform4fvARB");
		glUniform4iARB				= (PFNGLUNIFORM4IARBPROC) GETPROCADDRESS("glUniform4iARB");
		glUniform4ivARB	            = (PFNGLUNIFORM4IVARBPROC) GETPROCADDRESS("glUniform4ivARB");

		glUniformMatrix2fvARB		= (PFNGLUNIFORMMATRIX2FVARBPROC) GETPROCADDRESS("glUniformMatrix2fvARB");
		glUniformMatrix3fvARB		= (PFNGLUNIFORMMATRIX3FVARBPROC) GETPROCADDRESS("glUniformMatrix3fvARB");
		glUniformMatrix4fvARB	    = (PFNGLUNIFORMMATRIX4FVARBPROC) GETPROCADDRESS("glUniformMatrix4fvARB");
#endif //GL_GLEXT_PROTOTYPES

		TRACE(">> GL_ARB_vertex_shader\n")
#if !defined(GL_GLEXT_PROTOTYPES)
		glBindAttribLocationARB	= (PFNGLBINDATTRIBLOCATIONARBPROC) GETPROCADDRESS( "glBindAttribLocationARB" );
#endif //GL_GLEXT_PROTOTYPES
	}
}

} //hydra

