/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#ifndef HID_H_
#define HID_H_

#if defined(__APPLE__)

#include <IOKit/IOKitLib.h>
#include <IOKit/IOCFPlugIn.h>

#ifdef MACOS_10_0_4
	#include <IOKit/hidsystem/IOHIDUsageTables.h>
#else
	// The header was moved here in Mac OS X 10.1
	#include <Kernel/IOKit/hidsystem/IOHIDUsageTables.h>
#endif

#include <IOKit/hid/IOHIDLib.h>
#include <IOKit/hid/IOHIDKeys.h>

struct recElement
{
    IOHIDElementCookie cookie;  // unique value which identifies element, will NOT change
    long min;                   // reported min value possible
    long max;                   // reported max value possible

    // runtime variables used for auto-calibration
    long minReport;             // min returned value
    long maxReport;             // max returned value

    struct recElement * next;   // next element in list
};
typedef struct recElement recElement;

struct recDevice
{
    IOHIDDeviceInterface ** interface;   // interface to device, NULL = no interface

    char product[256];          // name of product
    long usage;                 // usage page from IOUSBHID Parser.h which defines general usage
    long usagePage;             // usage within above page from IOUSBHID Parser.h which defines specific usage

    long axes;                  // number of axis (calculated, not reported by device)
    long buttons;               // number of buttons (calculated, not reported by device)
    long hats;                  // number of hat switches (calculated, not reported by device)
    long elements;              // number of total elements (shouldbe total of above) (calculated, not reported by device)

    recElement * firstAxis;
    recElement * firstButton;
    recElement * firstHat;

    int removed;
    int uncentered;

    struct recDevice * next;	// next device
};
typedef struct recDevice recDevice;

// set up for C function definitions, even when using C++
#ifdef __cplusplus
extern "C" {
#endif

extern recDevice * HIDBuildDevice(io_object_t hidDevice);
extern recDevice * HIDDisposeDevice(recDevice ** device);

extern SInt32 HIDGetElementValue(recDevice * device, recElement * element);
extern SInt32 HIDScaledCalibratedValue(recDevice * device, recElement * element, long min, long max);

// ends C function definitions when using C++
#ifdef __cplusplus
}
#endif

#endif //__APPLE__

#endif HID_H_
