/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "file-stream.h"

#if defined(__APPLE__) || defined(_LINUX)

namespace hydra
{

IMPLEMENT_MODULE(fileStream, usr/lib/file)

fileStream::fileStream()
{
	m_stream = NULL;
}
fileStream::~fileStream()
{
	if (m_stream)
	{
		fclose(m_stream);
		m_stream = NULL;
	}
}

bool fileStream::open(const char * pathName, int mode)
{
	const char * modeString;
	switch (mode)
	{
	case CREATE:
		modeString = "wb";
		break;

	case READONLY:
		modeString = "rb";
		break;

	case READWRITE:
		modeString = "r+b";
		break;
	}

	//
	m_stream = fopen(pathName, modeString);
	if (!m_stream)
	{
		TRACE(">> ERROR: fileStream::open(%s) failed\n", pathName);
		return false;
	}

	return true;
}

//
size_t fileStream::read(void * ptr, size_t count)
{
	return fread(ptr, 1, count, m_stream);
}
size_t fileStream::write(void * ptr, size_t count)
{
	return fwrite(ptr, 1, count, m_stream);
}

//
int fileStream::seek(long offset, int origin)
{
	return fseek(m_stream, offset, origin);
}

//
size_t fileStream::getSize()
{
	fseek(m_stream, 0, SEEK_END);
	long size = ftell(m_stream);

	fseek(m_stream, 0, SEEK_SET);

	return size;
}

} //hydra

#endif //__APPLE || _LINUX
