/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "file-win32.h"

#if defined(_WIN32) || defined(_XBOX)

namespace hydra
{

IMPLEMENT_MODULE(fileWin32, usr/lib/file)

fileWin32::fileWin32()
{
	m_handle = NULL;
}
fileWin32::~fileWin32()
{
	if (m_handle)
	{
		CloseHandle(m_handle);
		m_handle = NULL;
	}
}

bool fileWin32::open(const char * pathName, int mode)
{
	DWORD desiredAccess;
	DWORD shareMode;
	DWORD creationDisposition;

	switch (mode)
	{
	case CREATE:
		desiredAccess = GENERIC_READ | GENERIC_WRITE;
		shareMode = FILE_SHARE_WRITE;
		creationDisposition = CREATE_ALWAYS;
		break;

	case READONLY:
		desiredAccess = GENERIC_READ;
		shareMode = FILE_SHARE_READ;
		creationDisposition = OPEN_EXISTING;
		break;

	case READWRITE:
		desiredAccess = GENERIC_READ | GENERIC_WRITE;
		shareMode = FILE_SHARE_WRITE;
		creationDisposition = OPEN_EXISTING;
		break;
	}

	m_handle = CreateFile(pathName,
		desiredAccess,
		shareMode,
		NULL,
		creationDisposition,
		0,
		NULL);
	if (m_handle == INVALID_HANDLE_VALUE)
	{
		TRACE(">> Error: CreateFile(%s) error = 0x%x\n", pathName, m_handle)
		return NULL;
	}

	return true;
}

//
size_t fileWin32::read(void * ptr, size_t count)
{
	DWORD readBytes;
	ReadFile(m_handle, ptr, count, &readBytes, NULL);
	ASSERT(count == readBytes)

	return readBytes;
}
size_t fileWin32::write(void * ptr, size_t count)
{
	DWORD writeBytes;
	WriteFile(m_handle, ptr, count, &writeBytes, NULL);
	ASSERT(count == writeBytes)

	return writeBytes;
}

//
int fileWin32::seek(long offset, int origin)
{
	return SetFilePointer(m_handle, offset, NULL, origin);
}

//
size_t fileWin32::getSize()
{
	size_t size = GetFileSize(m_handle, NULL);
	return size;
}

} //hydra

#endif //_WIN32 || _XBOX
