/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "input-xinput.h"

#if defined(_WIN32) || defined(_XBOX)

namespace hydra
{

// Desc: Converts SHORT thumbstick values to FLOAT, while enforcing a deadzone
static inline FLOAT convertThumbstickValue(SHORT thumbstickValue, SHORT deadZone)
{
	if (thumbstickValue > +deadZone)
	{
		return (thumbstickValue - deadZone) / (32767.0f - deadZone);
	}
	if (thumbstickValue < -deadZone)
	{
		return (thumbstickValue + deadZone + 1.0f) / (32767.0f - deadZone);
	}

	return 0.0f;
}

static unsigned char padStatus[USER_MAX_COUNT];		// old pad status

#if defined(_WIN32)
	IMPLEMENT_MODULE(inputXInput, usr/lib/input.xinput)
#elif defined(_XBOX)
	IMPLEMENT_MODULE(inputXInput, usr/lib/input)
#endif

inputXInput::inputXInput()
{
}

//
bool inputXInput::open(HWND hwnd)
{
	for (int n = 0; n < USER_MAX_COUNT; n ++)
		padStatus[n] = 0;

	return true;
}

void inputXInput::update()
{
	// loop through all gamepads
	for (int n = 0; n < USER_MAX_COUNT; n ++)
	{
		// No device, set all inputs to zero
		m_padInfo[n].x[0] = m_padInfo[n].x[1] = m_padInfo[n].x[2] = 0;
		m_padInfo[n].y[0] = m_padInfo[n].y[1] = m_padInfo[n].y[2] = 0;
		m_padInfo[n].z[0] = m_padInfo[n].z[1] = m_padInfo[n].z[2] = 0;

		m_padInfo[n].gyro = 0;
		m_padInfo[n].buttons = 0;

		// Read the input state
		XINPUT_STATE inputState;
		if (XInputGetState(n, &inputState) == ERROR_SUCCESS)
		{
			// device is plugged in
			if (padStatus[n] == 0)
			{
				// Store the capabilities of the device
				XINPUT_CAPABILITIES caps;
				XInputGetCapabilities(n, XINPUT_FLAG_GAMEPAD, &caps);

				// Update device status for next time
				padStatus[n] = 1;
			}

			// Put Xbox device input for the gamepad into our custom format
			m_padInfo[n].x[0] = convertThumbstickValue(inputState.Gamepad.sThumbLX, XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE);
			m_padInfo[n].y[0] = convertThumbstickValue(inputState.Gamepad.sThumbLY, XINPUT_GAMEPAD_LEFT_THUMB_DEADZONE);
			m_padInfo[n].x[1] = convertThumbstickValue(inputState.Gamepad.sThumbRX, XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE);
			m_padInfo[n].y[1] = convertThumbstickValue(inputState.Gamepad.sThumbRY, XINPUT_GAMEPAD_RIGHT_THUMB_DEADZONE);

			// Get the boolean buttons that have been pressed since the last
			// call. Each button is represented by one bit.
			m_padInfo[n].buttons = inputState.Gamepad.wButtons;

			// XINPUT_GAMEPAD_TRIGGER_THRESHOLD
			// Figure out if the left trigger has been pressed or released
			m_padInfo[n].x[2] = inputState.Gamepad.bLeftTrigger;
			m_padInfo[n].y[2] = inputState.Gamepad.bRightTrigger;

			if (inputState.Gamepad.bLeftTrigger > XINPUT_GAMEPAD_TRIGGER_THRESHOLD)
				m_padInfo[n].buttons |= BUTTON_L2;
			if (inputState.Gamepad.bRightTrigger > XINPUT_GAMEPAD_TRIGGER_THRESHOLD)
				m_padInfo[n].buttons |= BUTTON_R2;
		}
		else
		{
			// Update device status for next time
			padStatus[n] = 0;
		}
	}
}

} //hydra

#endif //_WIN32 || _XBOX
