/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

// #undef _USRDLL

#include "input-dinput8.h"
#include "input-darwin.h"
#include "input-linux.h"
#include "input-xinput.h"

#include "file-stream.h"
#include "file-win32.h"

#include "window-win32.h"
#include "window-carbon.h"
#include "window-linux.h"

//#include "input-ps3.h"

#include "kernel.h"

#if defined (_MSC_VER)
	#pragma comment(lib, "hydra.lib")
#endif //_MSC_VER

namespace hydra
{

#if defined(_WIN32)
	static char * getOSVersion();
#endif
	
static MODULEINFO moduleInfo = {
	"system",
	"System module.",
	"SYSTEM",
	"(c) 2004-2008 Enside, Inc. All rights reserved.",
	"hunee (hunee@enside.net)",
	"0.0.1",
};
LIBEXPORT const MODULEINFO * getModuleInfo()
{
	return &moduleInfo;
}

LIBEXPORT void * loadModule()
{
#if defined(_WIN32)
	registerKernelModule(windowWin32);
	registerKernelModule(fileWin32);
	registerKernelModule(inputDInput8);
	registerKernelModule(inputXInput);

	//
	TRACE(">> %s\n", getOSVersion())
	
#elif defined(__APPLE__)
	registerKernelModule(windowCarbon);
	registerKernelModule(fileStream);
	registerKernelModule(inputDarwin);

	UInt32 response;
	Gestalt(gestaltSystemVersion, (SInt32 *) &response);
	TRACE(">> Mac OS X version %x\n", response)

	if (response < 0x01020)
	{
		TRACE0("must have Mac OS v10.2 or later to use aglSurfaceTexture.")
		return false;
	}
	
#elif defined(_LINUX)
	registerKernelModule(windowLinux);
	registerKernelModule(fileStream);
	registerKernelModule(inputLinux);
	
	//
	int vidModeMajorVersion, vidModeMinorVersion;
	XF86VidModeQueryVersion(m_hwnd, &vidModeMajorVersion, &vidModeMinorVersion);
	TRACE(">> Linux/X11 version %d %d\n", vidModeMajorVersion, vidModeMinorVersion);

#endif

	return NULL;
}
LIBEXPORT void unloadModule()
{
#if defined(_WIN32)
	unregisterKernelModule(fileWin32);
	unregisterKernelModule(inputDInput8);
	unregisterKernelModule(inputXInput);
	unregisterKernelModule(windowWin32);

#elif defined(__APPLE__)
	unregisterKernelModule(fileStream);
	unregisterKernelModule(inputDarwin);
	unregisterKernelModule(windowCarbon);

#elif defined(_LINUX)
	unregisterKernelModule(fileStream);
	unregisterKernelModule(inputLinux);
	unregisterKernelModule(windowLinux);
#endif
}


#if defined(_WIN32)
static char * getOSVersion()
{
	static char strOperatingSystem[MAX_PATH];

	// Try calling GetVersionEx using the OSVERSIONINFOEX structure.
	// If that fails, try using the OSVERSIONINFO structure.
	OSVERSIONINFOEX osvi;
	ZeroMemory(&osvi, sizeof (OSVERSIONINFOEX));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

	if (!GetVersionEx((OSVERSIONINFO *) &osvi))
	{
		// If OSVERSIONINFOEX doesn't work, try OSVERSIONINFO.
		osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		if (!GetVersionEx((OSVERSIONINFO *) &osvi))
			return NULL;
	}

	switch (osvi.dwPlatformId)
	{
	case VER_PLATFORM_WIN32_NT:
		// Test for the product.
		if (osvi.dwMajorVersion <= 4)
			lstrcpy(strOperatingSystem, "Microsoft Windows NT ");
		if (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0)
			lstrcpy(strOperatingSystem, "Microsoft Windows 2000 ");
		if (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1)
			lstrcpy(strOperatingSystem, "Microsoft Windows XP ");
		if (osvi.dwMajorVersion == 6 && osvi.dwMinorVersion == 0)
			lstrcpy(strOperatingSystem, "Microsoft Windows Vista ");

		// Test for product type.
		HKEY hKey;
		char szProductType[80];
		DWORD dwSize;
		LONG lResult;

		// Query the registry to retrieve information.
		lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
			"SYSTEM\\CurrentControlSet\\Control\\ProductOptions",
			0,
			KEY_QUERY_VALUE,
			&hKey);
		if (lResult != ERROR_SUCCESS)
			return NULL;

		lResult = RegQueryValueEx(hKey,
			"ProductType",
			NULL,
			NULL,
			(LPBYTE) szProductType,
			&dwSize);
		RegCloseKey(hKey);

		if (lResult != ERROR_SUCCESS)
			return NULL;

		if (lstrcmpi("WINNT", szProductType) == 0)
			lstrcat(strOperatingSystem, "Professional ");
		if (lstrcmpi("LANMANNT", szProductType) == 0)
		{
			// Decide between Windows 2000 Advanced Server and Windows .NET Enterprise Server.
			if (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1)
				lstrcat(strOperatingSystem, "Standard Server ");
			else
				lstrcat(strOperatingSystem, "Server ");
		}
		if (lstrcmpi ("SERVERNT", szProductType) == 0)
		{
			// Decide between Windows 2000 Advanced Server and Windows .NET Enterprise Server.
			if (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1)
				lstrcat(strOperatingSystem, "Enterprise Server ");
			else
				lstrcat(strOperatingSystem, "Advanced Server ");
		}

		// Display version, service pack (if any), and build number.
		if (osvi.dwMajorVersion <= 4)
		{
			// NB: NT 4.0 and earlier.
			wsprintf(strOperatingSystem, "%sversion %d.%d %s (Build %d)",
				strOperatingSystem,
				osvi.dwMajorVersion,
				osvi.dwMinorVersion,
				osvi.szCSDVersion,
				osvi.dwBuildNumber & 0xFFFF);
		}
		else if ((osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1 ) ||
				 (osvi.dwMajorVersion == 6 && osvi.dwMinorVersion == 0))
		{
			// Windows XP and .NET server.
			typedef BOOL (CALLBACK* LPFNPROC)(HANDLE, BOOL *);

			BOOL bIsWindows64Bit;

			// Load the Kernel32 DLL.
			HINSTANCE hKernelDLL = LoadLibrary("kernel32");
			if (hKernelDLL != NULL)
			{
				// Only XP and .NET Server support IsWOW64Process so... Load dynamically!
				LPFNPROC DLLProc = (LPFNPROC) GetProcAddress(hKernelDLL, "IsWow64Process");

				// If the function address is valid, call the function.
				if (DLLProc != NULL)
					(DLLProc) (GetCurrentProcess(), &bIsWindows64Bit);
				else
					bIsWindows64Bit = false;

				// Free the DLL module.
				FreeLibrary(hKernelDLL);
			}

			// IsWow64Process ();
			if (bIsWindows64Bit)
				lstrcat(strOperatingSystem, "64-Bit ");
			else
				lstrcat(strOperatingSystem, "32-Bit ");
		}

		//else
		{
			// Windows 2000 and everything else.
			wsprintf(strOperatingSystem, "%s%s (Build %d)",
				strOperatingSystem,
				osvi.szCSDVersion,
				osvi.dwBuildNumber & 0xFFFF);
		}
		break;

	case VER_PLATFORM_WIN32_WINDOWS:
		// Test for the product.
		if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 0)
		{
			lstrcpy(strOperatingSystem, "Microsoft Windows 95 ");
			if (osvi.szCSDVersion[1] == 'C')
				lstrcat(strOperatingSystem, "OSR 2.5 ");
			else if (osvi.szCSDVersion[1] == 'B')
				lstrcat(strOperatingSystem, "OSR 2 ");
		}

		if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 10)
		{
			lstrcpy(strOperatingSystem, "Microsoft Windows 98 ");
			if (osvi.szCSDVersion[1] == 'A')
				lstrcat(strOperatingSystem, "SE ");
		}

		if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 90)
		{
			lstrcpy(strOperatingSystem, "Microsoft Windows Me ");
		}
		break;

	case VER_PLATFORM_WIN32s:
		lstrcpy(strOperatingSystem, "Microsoft Win32s ");
		break;

	default:
		lstrcpy(strOperatingSystem, "Unknown Windows ");
		break;
	}

	return strOperatingSystem;
}
#endif

} //hydra

