/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "window-win32.h"

#include "kernel.h"

#if defined(_WIN32)

namespace hydra
{

static bool registerClass(HINSTANCE instance, LPCTSTR className);
static HRESULT WINAPI wndProc(HWND wnd, UINT message, WPARAM wParam, LPARAM lParam);

IMPLEMENT_MODULE(windowWin32, usr/lib/window)

windowWin32::windowWin32()
{
}
windowWin32::~windowWin32()
{
	if (m_hwnd)
	{
		::DestroyWindow(m_hwnd);
		m_hwnd = NULL;
	}
}

//
bool windowWin32::create(int width, int height, const char * title)
{
	HINSTANCE instance = GetModuleHandle(0);
	if (!registerClass(instance, PLATFORMNAME))
		return false;

	// Set the window's initial style
	unsigned long style = WS_POPUP;//WS_OVERLAPPEDWINDOW | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	//style= style & (~WS_THICKFRAME);

	// Create Window
	m_hwnd = ::CreateWindow(PLATFORMNAME,
		title,
		style,
		0,
		0,
		width,
		height,
		NULL,
		NULL,
		instance,
		NULL);
	if (!m_hwnd)
		return false;

	//
	::DragAcceptFiles(m_hwnd, TRUE);

	::ShowWindow(m_hwnd, TRUE);
	::UpdateWindow(m_hwnd);

	::SetCursor(LoadCursor(NULL, IDC_ARROW));

	return true;
}

int windowWin32::run()
{
	MSG	msg;

	while (TRUE)
	{
		// Look for messages, if none are found then
		// update the state and display it
		if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (!GetMessage(&msg, NULL, 0, 0))
				break;

			// Translate and dispatch the message
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			if (!idle())
				break;
		}
	}

	// WM_QUIT was posted, so exit
	return (int) msg.wParam;
}

//
static bool registerClass(HINSTANCE instance, LPCTSTR className)
{
	//TRACE("# ::registerClass(0x%x, %s)\n", instance, className)

	// Register Class
	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style = 0;//CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc = (WNDPROC) wndProc;
	wcex.cbClsExtra = 0;
	wcex.cbWndExtra = 0;
	wcex.hInstance = instance;
	wcex.hIcon = ::LoadIcon(instance, IDI_APPLICATION);
	wcex.hIconSm = ::LoadIcon(instance, IDI_APPLICATION);
	wcex.hCursor = ::LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground = (HBRUSH) COLOR_BACKGROUND;
	wcex.lpszMenuName = NULL;
	wcex.lpszClassName = className;
	if (!::RegisterClassEx(&wcex))
		return false;

	return true;
}
static HRESULT WINAPI wndProc(HWND wnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		//case WM_SETCURSOR:
		//	SetCursor(NULL);
		//	break;

	case WM_DESTROY:
		PostQuitMessage(0);
		break;

	case WM_SIZE:
		//if (type == SIZE_MINIMIZED)
		//framework->onSize((unsigned int) wParam, LOWORD(lParam), HIWORD(lParam));
		//graphicsModule()->setViewport(0, 0, LOWORD(lParam), HIWORD(lParam));
		break;

	case WM_DROPFILES:
		//framework->onDropFiles((HDROP) wParam);
		break;

	case MM_WOM_DONE:
		// A waveform-audio data block has been played and can now be freed.
		//waveOutUnprepareHeader((HWAVEOUT) wParam, (LPWAVEHDR) lParam, sizeof(WAVEHDR) );

		// Free hData memory.
		//waveOutClose((HWAVEOUT) wParam);
		break;

	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_ESCAPE:	// CLOSE
			PostMessage(wnd, WM_CLOSE, 0, 0);
			break;
		}
		break;

	case WM_NCHITTEST:
		return HTCAPTION;

	default:
		return (HRESULT) DefWindowProc(wnd, message, wParam, lParam);
	}

	return 0;
}

} //hydra

#endif //_WIN32
