/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */

#include "my-view.h"

#include "my-application.h"

#include "system/iinput.h"
#include "graphics/igraphics.h"

using namespace hydra;

myView::myView()
{
    m_voice = NULL;
	m_mesh = NULL;

	m_logo = NULL;
	
	//camera
	m_pos.x = 0.0f;
	m_pos.y = -1.0f;
	m_pos.z = 8.0f;

	m_axis.x = 0.0f;
	m_axis.y = TODEGREE(90.0f);
	m_axis.z = 0.0f;

	//
	for (int n=0; n<MAX_SPRITES; n++)
	{
		m_sampler[n] = NULL;
	}
}

myView::~myView()
{
	destroy();
}

void myView::destroy()
{
    SAFE_RELEASE(m_voice);
    SAFE_RELEASE(m_mesh);

	SAFE_RELEASE(m_logo);
	
	//
	for (int n=0; n<MAX_SPRITES; n++)
	{
		SAFE_RELEASE(m_sampler[n]);
	}
}

#define MAX_MODELS	(4)
static const char * m2models[MAX_MODELS] = {
	"ArcaneTitan.m2",
	"Akama.m2",
	"DragonAzurgoz.m2",
	"DragonNefarian.m2",
};
static int model = 0;
static int voice = 0;
//static int music = 0;

bool myView::loadData()
{
	m_logo = loadSampler("textures/hydra_logo_64.dds");
	if (!m_logo)
		return false;
	
	char pathName[MAX_PATH];
	/*for (int n=0; n<MAX_SPRITES; n++)
	{
		sprintf(pathName, "textures/%d.DDS", n);

		m_sampler[n] = loadSampler(pathName);
		if (!m_sampler[n])
			return false;
	}*/

    //
    sprintf(pathName, "music/%d.mp3", voice);
    m_voice = loadVoice(pathName);
    if (!m_voice)
        return false;
    voice ++;

    //
    m_voice->play(0);

    //
	sprintf(pathName, "models/%s", m2models[model]);
    m_mesh = loadMesh(pathName);
    if (!m_mesh)
        return false;
	model ++;

	//
	float aspect = (float) graphicsModule()->getWidth() / (float) graphicsModule()->getHeight();
	m_camera.setPerspectiveFov(40.0f, aspect, 0.1f, 1000.0f);

	return true;
}

//
bool myView::update(float deltaTime)
{
	Application * app = static_cast <Application *> (Application::instance());
	PadInfo * padInfo = app->getInput()->getPadInfo(0);

	m_pos.x  += padInfo->x[0] / 50.0f;
	m_pos.y  += padInfo->y[0] / 50.0f;
	m_pos.z  += padInfo->y[1] / 50.0f;
	m_axis.y += padInfo->x[1] / 50.0f;

	static unsigned long buttons;
	if (padInfo->buttons != buttons)
	{
		buttons = padInfo->buttons;

		if (buttons & BUTTON_R1)
		{
			char pathName[MAX_PATH];
			sprintf(pathName, "models/%s", m2models[model]);
			model ++;
			if (model > (MAX_MODELS - 1))
				model = 0;

			SAFE_RELEASE(m_mesh);

			m_mesh = loadMesh(pathName);
			if (!m_mesh)
				return false;
		}
	}

	//update camera
	m_camera.setPos(m_pos);
	m_camera.setRotationYawPitchRoll(m_axis.y, m_axis.x, m_axis.z);

	//
	graphicsModule()->setViewMatrix(m_camera.getViewMatrix());
	graphicsModule()->setProjectionMatrix(m_camera.getProjectionMatrix());

	return true;
}

//
void myView::draw()
{
	Application * app = static_cast <Application *> (Application::instance());

	//draw sprite
	/*graphicsModule()->begin();
	for (int k=0; k<10; k++)
	{
		for (int n=0; n<MAX_SPRITES; n++)
		{
			fixed2 pos(0,0);
			int width = (graphicsModule()->getWidth() - m_sampler[n]->getWidth());
			int height = (graphicsModule()->getHeight() - m_sampler[n]->getHeight());

			if (width != 0)
				pos.x = rand() % width;
			if (height != 0)
				pos.y = rand() % height;

			graphicsModule()->draw(m_sampler[n], pos);
		}
	}
	graphicsModule()->end();
	*/
	
    //draw mesh
    m_mesh->draw();


	//draw logo
	graphicsModule()->begin();
	fixed2 pos(0,0);
	pos.x = graphicsModule()->getWidth() - m_logo->getWidth();
	pos.y = graphicsModule()->getHeight() - m_logo->getHeight();

	graphicsModule()->setRenderState(RS_ALPHABLENDENABLE, TRUE);
	graphicsModule()->setRenderState(RS_SRCBLEND, BLEND_SRCALPHA);
	graphicsModule()->setRenderState(RS_DESTBLEND, BLEND_DESTALPHA);

	graphicsModule()->draw(m_logo, pos);
	graphicsModule()->end();
	
	//fps
	char fps[MAX_PATH];
	sprintf(fps, "%s %.2f fps", getBuildInfo(), app->getFPS());
	graphicsModule()->drawText(fixed2(10, 10), fps, 0xffffffff);

	char info[MAX_PATH];
	for (int n=0; n<USER_MAX_COUNT; n++)
	{
		//padinfo
		PadInfo * padInfo = app->getInput()->getPadInfo(n);

		sprintf(info, "BTN: (0x%x) GYRO(%4d) X(%.2f, %.2f, %.2f) Y(%.2f, %.2f, %.2f) Z(%.2f, %.2f, %.2f)",
			padInfo->buttons, padInfo->gyro,
			padInfo->x[0], padInfo->x[1], padInfo->x[2],
			padInfo->y[0], padInfo->y[1], padInfo->y[2],
			padInfo->z[0], padInfo->z[1], padInfo->z[2]);
		graphicsModule()->drawText(fixed2(30, 50 + n * (graphicsModule()->getGlyphHeight() + 1)), info);
	}
}
